/* doUndoMoveLock.h
 */
#ifndef _DOUNDOMOVELOCK_H
#define _DOUNDOMOVELOCK_H

#include "osl/apply_move/doUndoMoveLockForward.h"
#include "osl/move.h"
#include <boost/scoped_ptr.hpp>
#include <boost/type_traits.hpp>
namespace osl
{
  namespace state
  {
    class SimpleState;
  }
  namespace apply_move
  {
  struct DoUndoMoveLockCleanUp
  {
    virtual ~DoUndoMoveLockCleanUp();
  };
  /**
   * slow but useful lock of doUndoMove.
   * {
   *    DoUndoMoveLock lock(state, move); // doMove(move) here
   *    ...
   *    // undoMove here at the end of the scope
   *  }
   * use ApplyMove::doUndoMove for game playing.
   */
  class DoUndoMoveLock
  {
    state::SimpleState *state;
    boost::scoped_ptr<DoUndoMoveLockCleanUp> cleaner;
  public:
    template <class State>
    DoUndoMoveLock(State& state, Move move)
    {
      BOOST_STATIC_ASSERT(! boost::is_const<State>::value);
      init(state, move);
    }
    ~DoUndoMoveLock();
  private:
    template <class State>
    void init(State& state, Move move);
  };
  
  } // namespace apply_move
  using apply_move::DoUndoMoveLock;
} // namespace osl

#endif /* _DOUNDOMOVELOCK_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
