/* doUndoMoveStack.h
 */
#ifndef _DOUNDOMOVESTACK_H
#define _DOUNDOMOVESTACK_H

#include "osl/move.h"
#include "osl/apply_move/doUndoMoveLock.h"
#include "osl/stl/stack.h"

namespace osl
{
  namespace apply_move
  {
  template <class State>
  class DoUndoMoveStack
  {
    stack<DoUndoMoveLock*> move_stack;
    int memory_emergency;
  public:
    DoUndoMoveStack();
    ~DoUndoMoveStack();
    void push(State& s, Move m)
    {
      DoUndoMoveLock *l = new DoUndoMoveLock(s, m);
      move_stack.push(l);
    }
    void pop()
    {
      DoUndoMoveLock *l = move_stack.top();
      delete l;
      move_stack.pop();
    }
    void clear()
    {
      while (! move_stack.empty())
	pop();
    }
    void setEmergency()
    {
      memory_emergency = true;
    }
    bool empty() const { return move_stack.empty(); }
  };

  template <class State>
  DoUndoMoveStack<State>::DoUndoMoveStack()
    : memory_emergency(false)
  {
  } 
  template <class State>
  DoUndoMoveStack<State>::~DoUndoMoveStack()
  {
    if (! memory_emergency)
      clear();
  } 

  } // namespace apply_move
  using apply_move::DoUndoMoveStack;
} // namespace osl

#endif /* _DOUNDOMOVESTACK_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
