/* bigramPosition.h
 */
#ifndef _BIGRAMPOSITION_H
#define _BIGRAMPOSITION_H

#include "osl/pieceStand.h"
#include "osl/category/categoryEnv.h"
#include "osl/category/categoryTraits.h"
#include "osl/container/moveStack.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/misc/carray.h"
#include "osl/stl/vector.h"
#include "osl/misc/carray.h"

#include <boost/scoped_ptr.hpp>
#include <map>

namespace osl
{
  namespace category
  {
    struct BigramHelper
    {
      BigramHelper();
      int moveToIndex(osl::Move move, osl::Player player) const;
      int moveToValue(osl::Move move) const;
      int toIndex(osl::Position pos, osl::Move move, osl::Player player) const;
      osl::Move indexToMove(int value,
			    const CategoryEnv::effect_state_t& state) const;
    private:
      CArray<int, PTYPE_SIZE> pieceMap;
      int moveToIndex(osl::Move move, bool from) const;
    };

    struct BigramTable
    {
      typedef std::map<int,osl::stl::vector<std::pair<int, int> > > map_t;
      BigramTable();
      BigramTable(const char *oslHome, const char *name);
      void init(const char *filename);
      void generate(const CategoryEnv& env, MoveLogProbVector& out,
		    int back) const;

    private:
      boost::scoped_ptr<map_t> table;
      int min;
    };

    struct BigramPosition : public SpecialProbabilityCategory
    {
      static BigramTable probTable;
      static const char *getName() { return "BigramPosition"; }
      static int minProb() { return 10; }
      static void generate(const CategoryEnv& env, MoveLogProbVector& out) {
	probTable.generate(env, out, 1);
      }
    };

    struct SamePlayerBigramPosition : public SpecialProbabilityCategory
    {
      static BigramTable probTable;
      static const char *getName() { return "SamePlayerBigramPosition"; }
      static int minProb() { return 10; }
      static void generate(const CategoryEnv& env, MoveLogProbVector& out) {
	probTable.generate(env, out, 2);
      }
    };

    struct CombinedBigramPosition : public SpecialProbabilityCategory
    {
      static const char *getName() { return "CombinedBigramPosition"; }
      static int minProb() { return 10; }
      static void generate(const CategoryEnv& env, MoveLogProbVector& out) {
	MoveLogProbVector b, sb;
	BigramPosition::generate(env, b);
	SamePlayerBigramPosition::generate(env, sb);
	size_t i, j;
	i = j = 0;
	while (true)
	{
	  if (i == b.size() && j == sb.size())
	    break;
	  else if (i == b.size())
	    out.push_back(sb[j++]);
	  else if (j == sb.size())
	    out.push_back(b[i++]);
	  else
	  {
	    if (b[i] < sb[j])
	      out.push_back(b[i++]);
	    else
	      out.push_back(sb[j++]);
	  }
	}
      }
    };

    struct BigramHistoryTable
    {
      typedef std::map<int,int> map_t;
      BigramHistoryTable(int dummy);
      void setupTable(const SimpleState& state,
		      const CategoryEnv::effect_state_t& eState,
		      const container::MoveStack& history);
      void generate(const CategoryEnv& env, MoveLogProbVector& out) const;
    private:
      boost::scoped_ptr<map_t> table;
      static const int DECAY_VALUE = 150;
      static const int RETAIN_REALIZATION_PROBABILITY = 500;
    };

    struct BigramHistory : public SpecialProbabilityCategory
    {
      static BigramHistoryTable probTable;
      static const char *getName() { return "BigramHistory"; }
      static int minProb() { return 150; }

      static void setupTable(const SimpleState& state,
			     const CategoryEnv::effect_state_t& eState,
			     const container::MoveStack& history)
      {
	probTable.setupTable(state, eState, history);
      }
      static void generate(const CategoryEnv& env, MoveLogProbVector& out)
      {
	probTable.generate(env, out);
      }
    };

    struct KingNeighborBigramTable
    {
      typedef std::map<int,osl::stl::vector<std::pair<int, int> > > map_t;
      KingNeighborBigramTable();
      KingNeighborBigramTable(const char *oslHome, const char *name);
      void init(const char *filename);
      void generate(const CategoryEnv& env, MoveLogProbVector& out,
		    int back) const;
      int probability(const CategoryEnv& env, Move, int back) const;
    private:
      bool isIn5x5(const osl::Position kingPosition,
		   const osl::Position targetPosition) const;
      bool isAttackByMajorPiece(const osl::category::CategoryEnv::effect_state_t&,
				const osl::Position pos,
				const osl::Ptype ptype,
				const osl::Player turn) const;
      boost::scoped_ptr<map_t> table;
      int min;
    };

    struct BigramAttack : public SpecialProbabilityCategory
    {
      static KingNeighborBigramTable probTable;
      static const char *getName() { return "BigramAttack"; }
      static int minProb() { return 30; }

      static void generate(const CategoryEnv& env, MoveLogProbVector& out)
      {
	probTable.generate(env, out, 2);
      }
      static int probability(const CategoryEnv& env, Move move)
      {
	return probTable.probability(env, move, 2);
      };
    };

    struct BigramDefense : public SpecialProbabilityCategory
    {
      static KingNeighborBigramTable probTable;
      static const char *getName() { return "BigramDefense"; }
      static int minProb() { return 30; }

      static void generate(const CategoryEnv& env, MoveLogProbVector& out)
      {
	probTable.generate(env, out, 1);
      }
    };
  } // namespace category
} // namespace osl

#endif /* _BIGRAMPOSITION_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
