/* check.h
 */
#ifndef _CHECK_H
#define _CHECK_H

#include "osl/category/categoryEnv.h"
#include "osl/category/probTable.h"
#include "osl/category/categoryTraits.h"
#include "osl/move_generator/addEffect_.h"
#include "osl/move_action/store.h"
#include "osl/effect_util/effectUtil.h"
#include "osl/effect_util/additionalEffect.h"

namespace osl
{
  namespace category
  {
    /**
     * 
     */
    struct Check : public StandardProbabilityCategory
    {
      static Interpolator<PieceValueTable> probTable;
      static const char *getName() { return "Check"; }
      static const bool AdjustSendOff = true;
      static void generate(const CategoryEnv& env, MoveVector& out)
      {
	assert((! EffectUtil::isKingInCheck(env.state->getTurn(), *env.state))
	       || (env.state->dump(), 0));
	move_action::Store store(out);
	const Position target = 
	  env.state->getKingPosition(alt(env.state->getTurn()));
	move_generator::GenerateAddEffect<true>::
	  generate(env.state->getTurn(), *env.state, target, store);
      }
    };

    struct EnoughEffectCheck : public FixedProbabilityCategory
    {
      static FixedValueTable probTable; // 200
      static const char *getName() { return "EnoughEffectCheck"; }
      static size_t getIndex(const CategoryEnv&, Move)
      {
	return 0;
      }
      static std::pair<bool,size_t>
      getIndexIf(const CategoryEnv& e, Move m, int /*piece_value*/)
      {
	if (!isInCategory(e, m))
	  return std::make_pair(false, 0u);
	return std::make_pair(true, getIndex(e,m));
      }
      static bool isInCategory(const CategoryEnv& e, Move m)
      {
	Player P = e.state->getTurn();
	if (EffectUtil::isKingInCheck(alt(P), *e.state))
	{
	  const int defense = e.state->countEffect(alt(P),m.to());
	  const int offense = e.state->countEffect(P,m.to());

	  return ((defense < offense)
		  || (defense == offense
		      && AdditionalEffect::hasEffect(*e.state, m.to(), P)));
	}
	return false;
      }
    };
  } // namespace category
} // namespace osl

#endif /* _CHECK_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
