/* pawn.h
 */
#ifndef OSL_CATEGORY_PAWN_H
#define OSL_CATEGORY_PAWN_H

#include "osl/category/categoryTraits.h"
#include "osl/category/categoryEnv.h"
#include "osl/container/moveStack.h"
#include "osl/container/moveVector.h"
namespace osl
{
  namespace category
  {
    struct ForwardPawnWithRookSupport : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "ForwardPawnWithRookSupport"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (! (move.ptype() == PAWN))
	  return false;
	if (move.isDrop())
	  return false;
	const Player player = move.player();
	const Offset front = Board_Table.getOffset(player, U);
	for (int i = PtypeTraits<ROOK>::indexMin;
	     i < PtypeTraits<ROOK>::indexLimit; i++)
	{
	  const Piece rook = env.state->getPieceOf(i);
	  if (! rook.isOnBoardByOwner(player))
	    continue;
	  const Offset32 diff(move.to(), rook.position());
	  if (front == Board_Table.getShortOffset(diff))
	    return true;
	}
	return false;
      }
    };
    struct DanglingPawn : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "DanglingPawn"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (! (move.ptype() == PAWN))
	  return false;
	const Player player = move.player();
	const Position head
	  = Board_Table.nextPosition(player, move.to() ,U);
	if (! head.canPromote(player))
	  return false;
	if (move.isDrop())
	  return env.state->getPieceOnBoard(head).isEmpty()
	    && ((! env.state->hasEffectBy(alt(player), head))
		|| env.state->hasEffectBy(player, head));
	return true;		// move
      }
    };
    /**
     * Ѥ 
     *
     * Ѥ꤬ͤʵ⤹뤬Ϣ³á⤫ͤ
     */
    struct JoiningPawns : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "JoiningPawns"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	const Move lastMove=env.history->lastMove();
	return move.isDrop()
	  && (move.ptype() == PAWN)
	  && lastMove.isNormal()
	  && (lastMove.capturePtype() == PAWN)
	  && (Board_Table.nextPosition(move.player(), move.to() ,U)
	      == lastMove.to());
      }
      static void generate(const CategoryEnv& env, MoveVector& moves)
      {
	const Move last_move=env.history->lastMove();
	if (! last_move.isNormal())
	  return;
	if (last_move.capturePtype() != PAWN)
	  return;
	const Player P = env.state->getTurn();
	if (! env.state->hasPieceOnStand(P, PAWN))
	  return;
	const Position to
	  = Board_Table.nextPosition(alt(P), last_move.to(), U);
	if (env.state->getPieceAt(to) != Piece::EMPTY())
	  return;
	moves.push_back(Move(to, PAWN, P));
      }
    };

    /**
     * Ǥ
     * ͤŬ
     */
    struct PawnBeforePawn : public FixedProbabilityCategory
    {
      static FixedValueTable probTable; // 200
      static const char *getName() { return "PawnBeforePawn"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (env.limit < 1000)
	  return false;

	return move.ptype() == PAWN
	  && env.state->getPieceAt(Board_Table.nextPosition(move.player(), move.to(), U)).ptype()
	  == PAWN;
      }
      static void generate(const CategoryEnv& env, MoveVector& moves)
      {
	if (env.limit < 1000)
	  return;

	for (int i = PtypeTraits<PAWN>::indexMin;
	     i < PtypeTraits<PAWN>::indexLimit;
	     i++)
	{
	  const Player player = env.state->getTurn();
	  const Piece p = env.state->getPieceOf(i);
	  const bool hasPawn = env.state->hasPieceOnStand(player, PAWN);
	  if (p.isOnBoard() && p.ptype() == PAWN
	      && p.owner() == alt(player))
	  {
	    const Position middle = Board_Table.nextPosition(player, p.position(), D);
	    const Position origin = (Board_Table.nextPosition(player, middle, D));
	    if (!env.state->getPieceAt(middle).isEmpty())
	      continue;

	    const Piece candidate = env.state->getPieceAt(origin);
	    if (candidate.ptype() == PAWN // ⤬ΤǤϼʬνͭ
		&& !middle.canPromote(player)) // Τ̥ƥ꡼ 
	    {
	      moves.push_back(Move(origin, middle, PAWN, PTYPE_EMPTY,
				   false, player));
	    }
	    else if (hasPawn && !env.state->isPawnMaskSet(player, middle.x()))
	    {
	      moves.push_back(Move(middle, PAWN, player));
	    }
	  }
	}
      }
    };

    /**
     * ü
     *
     * Ǥ
     */
    struct SideAttackDrop : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "SideAttackDrop"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN) 
	  return false;
	if (! move.isDrop())
	  return false;

	if (move.player() == BLACK)
	{
	  const Position king = env.state->getKingPosition<WHITE>();
	  if ((king.x() >= 8) && (king.y() <= 3))
	  {
	    return (move.to().x() == 9)
	      && (move.to().y() <= 4);
	  }
	  if ((king.x() <= 2) && (king.y() <= 3))
	  {
	    return (move.to().x() == 1)
	      && (move.to().y() <= 4);
	  }
	  return false;
	}
	else
	{
	  assert(move.player() == WHITE);
	  const Position king = env.state->getKingPosition<BLACK>();
	  if ((king.x() >= 8) && (king.y() >= 7))
	  {
	    return (move.to().x() == 9)
	      && (move.to().y() >= 6);
	  }
	  if ((king.x() <= 2) && (king.y() >= 7))
	  {
	    return (move.to().x() == 1)
	      && (move.to().y() >= 6);
	  }
	  return false;
	}
      }
    };
    
    /**
     * ü
     *
     * ޤͼ
     */
    struct SideAttack : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "SideAttack"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN) 
	  return false;
	if (move.player() == BLACK)
	{
	  const Position king = env.state->getKingPosition<WHITE>();
	  if ((king.x() >= 8) && (king.y() <= 3))
	  {
	    if (move.from() == Position(9,5))
	    {
	      return env.state->getPieceOnBoard(Position(9,3)).ptype()
		== PAWN;
	    }
	    if (move.from() == Position(9,6))
	    {
	      return env.state->getPieceOnBoard(Position(9,4)).ptype()
		== PAWN;
	    }
	    return false;
	  }
	  if ((king.x() <= 2) && (king.y() <= 3))
	  {
	    if (move.from() == Position(1,5))
	    {
	      return env.state->getPieceOnBoard(Position(1,3)).ptype()
		== PAWN;
	    }
	    if (move.from() == Position(1,6))
	    {
	      return env.state->getPieceOnBoard(Position(1,4)).ptype()
		== PAWN;
	    }
	    return false;
	  }
	  return false;
	}
	else
	{
	  assert(move.player() == WHITE);
	  const Position king = env.state->getKingPosition<BLACK>();
	  if ((king.x() >= 8) && (king.y() >= 7))
	  {
	    if (move.from() == Position(9,5))
	    {
	      return env.state->getPieceOnBoard(Position(9,7)).ptype()
		== PAWN;
	    }
	    if (move.from() == Position(9,4))
	    {
	      return env.state->getPieceOnBoard(Position(9,6)).ptype()
		== PAWN;
	    }
	    return false;
	  }
	  if ((king.x() <= 2) && (king.y() >= 7))
	  {
	    if (move.from() == Position(1,5))
	    {
	      return env.state->getPieceOnBoard(Position(1,7)).ptype()
		== PAWN;
	    }
	    if (move.from() == Position(1,4))
	    {
	      return env.state->getPieceOnBoard(Position(1,6)).ptype()
		== PAWN;
	    }
	    return false;
	  }
	  return false;
	}
      }
    };

    /**
     * ư
     */
    struct DropAfterOpposingPawn : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "DropAfterOpposingPawn"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	const Move last_move=env.history->lastMove();
	if (! last_move.isNormal()) 
	  return false;
	const Player P = move.player();
	return move.isDrop()
	  && (move.ptype() == PAWN)
	  && (Board_Table.nextPosition(P,move.to(),U).canPromote(P)
	      || (move.to() == last_move.from()))
	  && (last_move.capturePtype() == PAWN)
	  && (Board_Table.nextPosition(alt(P),last_move.from(),U)
	      == last_move.to());
      }
      static void generate(const CategoryEnv& env, MoveVector& out)
      {
	const Move last_move=env.history->lastMove();
	if (! (last_move.isNormal()
	       && (last_move.capturePtype() == PAWN))) // äΤϤʤ
	  return;
	const Player P = env.state->getTurn();
	if (! env.state->hasPieceOnStand(P, PAWN))
	  return;
	Position to = last_move.from();
	if (! (Board_Table.nextPosition(alt(P),last_move.from(),U)
		   == last_move.to()))
	  return;
	Position next = Board_Table.nextPosition(P, to, U);
	while (next.isOnBoard() && next.canPromote(P)
	       && (env.state->getPieceOnBoard(to) == Piece::EMPTY()))
	{
	  //  OnBoard ʤǤƤ
	  out.push_back(Move(to, PAWN, P));
	  to = next;
	  next = Board_Table.nextPosition(P, to, U);
	}
      }
    };

    /**
     * 
     *
     * ˶𤬤äƤⵤˤʤǽɤ餷
     */
    struct ImmediateAntiPawnPromotion : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "AntiPawnPromotion!"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	const Move last_move=env.history->lastMove();
	if (! last_move.isNormal()) 
	  return false;
	const Player P = move.player();
	if (move.isDrop()
	    && (move.ptype() == PAWN)
	    && (last_move.ptype() == PAWN))
	{
	  const Position focus = Board_Table.nextPosition(P,move.to(),U);
	  return focus.canPromote(alt(P))
	    && (Board_Table.nextPosition(P,focus,U)
		== last_move.to());
	}
	return false;
      }
      static void generate(const CategoryEnv& env, MoveVector& out)
      {
	const Move last_move=env.history->lastMove();
	if (! last_move.isNormal()) 
	  return;
	const Player P = env.state->getTurn();
	if (last_move.ptype() == PAWN)
	{
	  const Position focus = 
	    Board_Table.nextPosition(alt(P),last_move.to(),U);
	  if (focus.canPromote(alt(P)))
	  {
	    const Position target =
	      Board_Table.nextPosition(alt(P),focus,U);
	    if ((env.state->getPieceAt(target) == Piece::EMPTY()) // Ϥ߽Фǽ
		&& env.state->hasPieceOnStand(P, PAWN)
		&& (! env.state->isPawnMaskSet(P, target.x())))
	    {
	      out.push_back(Move(target, PAWN, P));
	    }
	  }
	}
      }
    };
    /**
     * 碌Ƥ롢ǹ
     */
    struct GambitPawn : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "GambitPawn"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN) 
	  return false;
	const Player player = move.player();
	const Position to = move.to();
	if (env.state->hasEffectByPtype<SILVER>(player, to)
	    && env.state->hasEffectByPtype<PAWN>(alt(player), to))
	  return true;
	int pawns = env.state->countPiecesOnStand(player, PAWN);
	if (move.isDrop())
	  --pawns;
	if (pawns == 0)
	  return false;
	const Position head
	  = Board_Table.nextPosition(player, to ,U);
	if (! env.state->getPieceAt(head).isOnBoardByOwner(alt(player)))
	  return false;
	if (head.canPromote(player) && env.state->hasEffectBy(player, head))
	  return true;
	const Position head2
	  = Board_Table.nextPosition(player, head ,U);
	if (! env.state->getPieceAt(head2).isOnBoardByOwner(alt(player)))
	  return false;
	// Drop λϡ֤ȹ֤ϳƿ
	return ! env.state->hasEffectBy(alt(player), head)
	  || env.state->hasEffectBy(player, head);
      }
    };
    /**
     * 
     */
    struct AnchorPawn : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "AnchorPawn"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN) 
	  return false;
	if (! move.isDrop())
	  return false;
	const Player player = move.player();
	const Position to = move.to();
	if (! env.state->hasEffectByPtype<GOLD>(player, to))
	  return false;
	if ((player == BLACK && to.y() != 9)
	    || (player == WHITE && to.y() != 1))
	  return false;
	if (env.state->hasEffectByPtype<ROOK>(alt(player), to))
	  return true;
	const Position head
	  = Board_Table.nextPosition(player, to ,U);
	return env.state->getPieceOnBoard(head).isOnBoardByOwner(player)
	  && env.state->hasEffectByPtype<ROOK>(alt(player), head);
      }
    };

    struct MoveSaGWithPawn : FixedProbabilityCategory
    {
      static FixedValueTable probTable; // 400
      static const char*getName() { return "MoveSaGWithPawn"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN || !move.isDrop())
	  return false;
	const Player player = move.player();
	const Player opponent = alt(player);
	const Position to = move.to();
	const Position target =
	  Board_Table.nextPosition(player, to ,U);
	const Piece piece = env.state->getPieceAt(target);

	if (!piece.isEmpty() && piece.owner() == opponent)
	{
	  const Position left =
	    Board_Table.nextPosition(opponent, target ,L);
	  const Position right =
	    Board_Table.nextPosition(opponent, target ,R);
	  const Position down =
	    Board_Table.nextPosition(opponent, target ,D);
	  if (piece.ptype() == GOLD &&
	      ((left.isOnBoard() && env.state->hasEffectBy(player, left))
	       || (right.isOnBoard() && env.state->hasEffectBy(player, right))
	       || (down.isOnBoard() && env.state->hasEffectBy(player, down))))
	      return true;

	  const Position down_left =
	    Board_Table.nextPosition(player, target ,DL);
	  const Position down_right =
	    Board_Table.nextPosition(player, target ,DR);
	  const Position up_left =
	    Board_Table.nextPosition(player, target ,UL);
	  const Position up_right =
	    Board_Table.nextPosition(player, target ,UR);

	  if (piece.ptype() == SILVER &&
	      ((down_left.isOnBoard() && env.state->hasEffectBy(player, down_left))
	       || (down_right.isOnBoard() && env.state->hasEffectBy(player, down_right))
	       || up_left.isOnBoard() && env.state->hasEffectBy(player, up_left)
	       || (up_right.isOnBoard() && env.state->hasEffectBy(player, up_right))))
	      return true;
	}
	return false;
      }
    };

    struct PawnStopBishopEffect : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "PawnStopBishopEffect"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (! (move.ptype() == PAWN))
	  return false;

	const Player player = move.player();
	const Position target = move.to();
	if (!env.state->hasEffectByPtype<BISHOP>(alt(player), target))
	  return false;

	if ((move.isDrop() && env.state->hasEffectBy(player, target))
	    || env.state->countEffect(player, target) >
	    env.state->countEffect(alt(player), target))
	  return true;

	return false;
      }
    };

    struct PawnOnLance : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "PawnOnLance"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN || ! move.isDrop())
	  return false;

	const Player player = move.player();
	const Position target = Board_Table.nextPosition(player, move.to(), U);
	if (!target.isOnBoard())
	  return false;
	const Piece piece = env.state->getPieceAt(target);

	if (!piece.isEmpty() && piece.owner() == alt(player)
	    && piece.ptype() == LANCE)
	  return true;

	return false;
      }
    };

    struct PawnOnSilver : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "PawnOnSilver"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN)
	  return false;

	const Player player = move.player();
	const Position target = Board_Table.nextPosition(player, move.to(), U);
	if (!target.isOnBoard())
	  return false;
	const Piece piece = env.state->getPieceAt(target);

	if (!piece.isEmpty() && piece.owner() == alt(player)
	    && piece.ptype() == SILVER)
	  return true;

	return false;
      }
    };

    struct PawnAttackBase : public FixedProbabilityCategory
    {
      static FixedValueTable probTable; // 500
      static const char *getName() { return "PawnAttackBase";}
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.ptype() != PAWN || !move.isDrop())
	  return false;

	const Player player = move.player();
	const Position king = env.state->getKingPosition(alt(player));

	if (move.to().x() < king.x() - 1 || move.to().x() > king.x() + 1)
	  return false;

	if ((player == BLACK && (move.to().y() != 4 || king.y() > 2)) ||
	    (player == WHITE && (move.to().y() != 6 || king.y() < 8)))
	  return false;


	const Piece target = env.state->getPieceAt(Board_Table.nextPosition(player, move.to(), U));
	if (!target.isEmpty())
	  return false;
	return !env.state->hasEffectBy(alt(player), move.to()) ||
	  env.state->hasEffectBy(player, move.to());
      }
    };
  } // namespace category
} // namespace osl

#endif /* OSL_CATEGORY_PAWN_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
