/* generalSimpleHashTable.cc
 */
#include "osl/container/generalSimpleHashTable.h"
#include "osl/hash/hashKey.h"
#include "osl/stl/hash_map.h"
#ifdef OSL_SMP
#  include "osl/misc/lightMutex.h"
#  include <iostream>
#endif

template <typename Record>
struct osl::container::GeneralSimpleHashTable<Record>::Table 
{
  typedef hash_map<HashKey, Record> table_t;
  typedef typename table_t::const_iterator const_iterator;
#ifdef OSL_SMP
  static const unsigned int DIVSIZE=16;
#else
  static const unsigned int DIVSIZE=1;
#endif
  CArray<table_t,DIVSIZE> tables;

#ifdef OSL_SMP
  typedef osl::misc::LightMutex Mutex;
  CArray<Mutex,DIVSIZE> mutex;
#endif

  /** icc hash_map capacity äƤʤäΤǼʬǻ */
  const unsigned int capacity;
  int num_cache_hit, num_record_after_full;

  // if you use USE_GPL_POOL_ALLOCATOR, take care of the object size.
  // usually 256 is too large.
  BOOST_STATIC_ASSERT(sizeof(Record) <= 256);

  Table(unsigned int c) 
    : capacity(c), num_cache_hit(0), num_record_after_full(0)
  {
    for(size_t i=0;i<DIVSIZE;i++)
      tables[i]=table_t(c/DIVSIZE);
  }
  ~Table()
  {
  }
  void clear()
  {
    for(size_t i=0;i<DIVSIZE;i++){
#ifdef OSL_SMP
      Mutex::scoped_lock lk(mutex[i]);
#endif
      tables[i].clear();
    }
    num_cache_hit = 0;
    num_record_after_full = 0;
  }
  size_t size() const
  {
    size_t ret=0;
    for(size_t i=0;i<DIVSIZE;i++)
      ret+=tables[i].size();
    return ret;
  }
private:
  Record *findInLock(const HashKey& key,int i)
  {
    typename table_t::iterator pos = tables[i].find(key);
    if (pos == tables[i].end())
      return 0;

    ++num_cache_hit;
    return &pos->second;
  }
  static int keyToIndex(const HashKey& key)
  {
    unsigned long val=key.getSignature();
    return (val>>24)%DIVSIZE;
  }
public:
  Record *find(const HashKey& key)
  {
    int i=keyToIndex(key);
#ifdef OSL_SMP
    Mutex::scoped_lock lk(mutex[i]);
#endif
    return findInLock(key,i);
  }
    
  Record *allocate(const HashKey& key)
  {
    int i=keyToIndex(key);
#ifdef OSL_SMP
    Mutex::scoped_lock lk(mutex[i]);
#endif
    const size_t current_size = tables[i].size();
    if (current_size < capacity/DIVSIZE)
    {
      Record *record = &tables[i].operator[](key);
      if (current_size == tables[i].size())
	++num_cache_hit;
      return record;
    }
    // 䤵ʤ褦õ
    Record *result = findInLock(key,i);
    if ((result == 0) && capacity)
    {
#ifdef OSL_SMP
      if (capacity > 10000)
	std::cerr << "table full " << size() << " " << capacity << "\n";
      // SMPĶǤƤthreadꤲɬפ
      ++num_record_after_full;
      throw TableFull();
#else
      if (num_record_after_full++ == 0) 
	throw TableFull();
#endif
    }
    return result;
  }
};

  
template <typename Record>
osl::container::GeneralSimpleHashTable<Record>::
GeneralSimpleHashTable(unsigned int capacity) 
  : table(new Table(capacity))
{
}
  
template <typename Record>
osl::container::GeneralSimpleHashTable<Record>::
~GeneralSimpleHashTable() {
}

template <typename Record>
void osl::container::GeneralSimpleHashTable<Record>::
clear()
{
  table->clear();
}

template <typename Record>
Record * 
osl::container::GeneralSimpleHashTable<Record>::
allocate(const HashKey& key)
{
  return table->allocate(key);
}

template <typename Record>
Record * 
osl::container::GeneralSimpleHashTable<Record>::
find(const HashKey& key)
{
  return table->find(key);
}

template <typename Record>
const Record * 
osl::container::GeneralSimpleHashTable<Record>::
find(const HashKey& key) const
{
  return table->find(key);
}

template <typename Record>
unsigned int osl::container::GeneralSimpleHashTable<Record>::
size() const
{
  return table->size();
}

template <typename Record>
unsigned int osl::container::GeneralSimpleHashTable<Record>::
capacity() const
{
  return table->capacity;
}

template <typename Record>
int osl::container::GeneralSimpleHashTable<Record>::
numCacheHit() const
{
  return table->num_cache_hit;
}

template <typename Record>
int osl::container::GeneralSimpleHashTable<Record>::
numRecordAfterFull() const
{
  return table->num_record_after_full;
}

template <typename Record>
int osl::container::GeneralSimpleHashTable<Record>::
divSize() const
{
  return Table::DIVSIZE;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
