/* minorPieceBonus.h
 */
#ifndef EVAL_MINORPIECEBONUS_H
#define EVAL_MINORPIECEBONUS_H

#include "osl/eval/pieceEval.h"
#include "osl/progress/progress16.h"
#include "osl/state/simpleState.h"
#include <boost/static_assert.hpp>

namespace osl
{
  namespace eval
  {
    /**
     * ڤʤɤɾ
     */
    class MinorPieceBonus
    {
      CArray<int,2> pawn_on_stand;
      CArray<int,2> lance_on_stand;
      CArray<int,2> knight_on_stand;
      CArray<int,2> pawns;
      CArray<int,2> golds;
    public:
      MinorPieceBonus(const SimpleState& state) 
      {
	pawn_on_stand[playerToIndex(BLACK)] = state.countPiecesOnStand(BLACK, PAWN);
	pawn_on_stand[playerToIndex(WHITE)] = state.countPiecesOnStand(WHITE, PAWN);
	lance_on_stand[playerToIndex(BLACK)] = state.countPiecesOnStand(BLACK, LANCE);
	lance_on_stand[playerToIndex(WHITE)] = state.countPiecesOnStand(WHITE, LANCE);
	knight_on_stand[playerToIndex(BLACK)] = state.countPiecesOnStand(BLACK, KNIGHT);
	knight_on_stand[playerToIndex(WHITE)] = state.countPiecesOnStand(WHITE, KNIGHT);
	pawns[playerToIndex(BLACK)] = 0;
	pawns[playerToIndex(WHITE)] = 0;
	golds[playerToIndex(BLACK)] = 0;
	golds[playerToIndex(WHITE)] = 0;
	for (int i = PtypeTraits<PAWN>::indexMin;
	     i < PtypeTraits<PAWN>::indexLimit; i++)
	{
	  const Piece pawn = state.getPieceOf(i);
	  if (pawn.owner() == BLACK)
	    pawns[playerToIndex(BLACK)]++;
	  else
	    pawns[playerToIndex(WHITE)]++;
	}

	for (int i = PtypeTraits<GOLD>::indexMin;
	     i < PtypeTraits<GOLD>::indexLimit; i++)
	{
	  const Piece gold = state.getPieceOf(i);
	  golds[playerToIndex(gold.owner())]++;
	}
      }

      static int value(Progress16 progress16,
		       int black_pawn, int white_pawn,
		       int black_pawn_total, int white_pawn_total,
		       int black_lance, int white_lance,
		       int black_knight, int white_knight,
		       int black_gold, int white_gold)
      {
	int result = 0;
	if (black_pawn > 1)
	{
	  result -=  (black_pawn - 1) * progress16.value() * ( PtypeEvalTraits<PAWN>::val / 32) ;
	}
	else if (black_pawn == 0 && black_pawn_total < white_pawn_total){
	  result -= PtypeEvalTraits<PAWN>::val / 2;
	}
	if (black_pawn >= 9)
	{
	  result -= (black_pawn - 8) * progress16.value() * PtypeEvalTraits<PAWN>::val / 8;
	}
	if (white_pawn > 1)
	{
	  result += (white_pawn - 1) * progress16.value() * 
	    (PtypeEvalTraits<PAWN>::val / 32);
	}
	else if (white_pawn == 0 && white_pawn_total < black_pawn_total){
	  result += PtypeEvalTraits<PAWN>::val / 2;
	}
	if (white_pawn >= 9)
	{
	  result += (white_pawn - 8) * progress16.value() * PtypeEvalTraits<PAWN>::val / 8;
	}
	if (black_lance > 1)
	{
	  result -= (black_lance - 1) * progress16.value() * PtypeEvalTraits<LANCE>::val / 32;
	}
	if (black_lance > 2)
	{
	  result -= (black_lance - 2) * progress16.value() * PtypeEvalTraits<PAWN>::val / 24;
	}
	if (black_knight > 1)
	{
	  result -= (black_knight - 1) * progress16.value() * PtypeEvalTraits<KNIGHT>::val / 32;
	}
	if (black_knight > 2)
	{
	  result -= (black_knight - 2) * progress16.value() * PtypeEvalTraits<PAWN>::val / 8;
	}
	if (white_lance > 1)
	{
	  result += (white_lance - 1) * progress16.value() * PtypeEvalTraits<LANCE>::val / 32;
	}
	if (white_lance > 2)
	{
	  result -= (white_lance - 2) * progress16.value() * PtypeEvalTraits<PAWN>::val / 24;
	}	if (white_knight > 1)
	{
	  result += (white_knight - 1) * progress16.value() * PtypeEvalTraits<KNIGHT>::val / 32;
	}
	if (white_knight > 2)
	{
	  result -= (white_knight - 2) * progress16.value() * PtypeEvalTraits<PAWN>::val / 8;
	}
	if (black_gold >= 3)
	{
	  result += progress16.value() * PtypeEvalTraits<GOLD>::val
	    * (black_gold - 2) / 2;
	}
	else if (white_gold >= 3)
	{
	  result -= progress16.value() * PtypeEvalTraits<GOLD>::val
	    * (white_gold - 2) / 2;
	}

	return result;
      }
      int value(Progress16 progress16) const { 
	assert(pawn_on_stand[playerToIndex(BLACK)] >= 0);
	assert(pawn_on_stand[playerToIndex(WHITE)] >= 0);
	return value(progress16, 
		     pawn_on_stand[playerToIndex(BLACK)], 
		     pawn_on_stand[playerToIndex(WHITE)],
		     pawns[playerToIndex(BLACK)], pawns[playerToIndex(WHITE)],
		     lance_on_stand[playerToIndex(BLACK)],
		     lance_on_stand[playerToIndex(WHITE)],
		     knight_on_stand[playerToIndex(BLACK)],
		     knight_on_stand[playerToIndex(WHITE)],
		     golds[playerToIndex(BLACK)],
		     golds[playerToIndex(WHITE)]);
      }

      void update(const SimpleState& new_state, Move last_move)
      {
	const Player player = last_move.player();
	const Ptype ptype = last_move.ptype();
	if (last_move.isDrop()) {
	  if (ptype == PAWN) {
	    pawn_on_stand[playerToIndex(player)]--;
	    assert(pawn_on_stand[playerToIndex(BLACK)] >= 0);
	    assert(pawn_on_stand[playerToIndex(WHITE)] >= 0);
	  }
	  if (ptype == LANCE) {
	    lance_on_stand[playerToIndex(player)]--;
	  }
	  if (ptype == KNIGHT) {
	    knight_on_stand[playerToIndex(player)]--;
	  }
	  return;
	}
	const Ptype captured = last_move.capturePtype();
	if (captured != PTYPE_EMPTY) {
	  switch (unpromote(captured)) {
	  case PAWN:
	    pawn_on_stand[playerToIndex(player)]++;
	    pawns[playerToIndex(player)]++;
	    pawns[playerToIndex(alt(player))]--;
	    assert(pawns[playerToIndex(BLACK)] + pawns[playerToIndex(WHITE)] == 18);
	    assert(pawn_on_stand[playerToIndex(BLACK)] >= 0);
	    assert(pawn_on_stand[playerToIndex(WHITE)] >= 0);
	    break;
	  case LANCE:
	    lance_on_stand[playerToIndex(player)]++;
	    break;
	  case KNIGHT:
	    knight_on_stand[playerToIndex(player)]++;
	    break;
	  case GOLD:
	    golds[playerToIndex(player)]++;
	    golds[playerToIndex(alt(player))]--;
	    assert(golds[playerToIndex(BLACK)] + golds[playerToIndex(WHITE)] == 4);
	    break;
	  default:
	    ;
	  }
	}
      }

      int expect(const SimpleState& state, Move move, Progress16 progress16) const
      {
	MinorPieceBonus new_eval = *this;
	if (move.isDrop()){
	  const Ptype ptype = move.ptype();
	  if (ptype == PAWN) {
	    new_eval.pawn_on_stand[playerToIndex(state.getTurn())]--;
	    assert(new_eval.pawn_on_stand[playerToIndex(BLACK)] >= 0);
	    assert(new_eval.pawn_on_stand[playerToIndex(WHITE)] >= 0);
	  }
	  else if (ptype == LANCE) {
	    new_eval.lance_on_stand[playerToIndex(state.getTurn())]--;
	  }
	  else if (ptype == KNIGHT) {
	    new_eval.knight_on_stand[playerToIndex(state.getTurn())]--;
	  }
	  return new_eval.value(progress16);
	}
	Ptype ptype = move.capturePtype();
	if (ptype != PTYPE_EMPTY) {
	  if (unpromote(ptype) == PAWN) {
	    new_eval.pawn_on_stand[playerToIndex(state.getTurn())]++;
	    new_eval.pawns[playerToIndex(state.getTurn())]++;
	    new_eval.pawns[playerToIndex(alt(state.getTurn()))]--;
	    assert(new_eval.pawns[playerToIndex(BLACK)] + new_eval.pawns[playerToIndex(WHITE)] == 18);
	    assert(new_eval.pawn_on_stand[playerToIndex(BLACK)] >= 0);
	    assert(new_eval.pawn_on_stand[playerToIndex(WHITE)] >= 0);
	  }
	  else if (unpromote(ptype) == LANCE) {
	    new_eval.lance_on_stand[playerToIndex(state.getTurn())]++;
	  }
	  else if (unpromote(ptype) == KNIGHT) {
	    new_eval.knight_on_stand[playerToIndex(state.getTurn())]++;
	  }
	  else if (unpromote(ptype) == GOLD) {
	    new_eval.golds[playerToIndex(state.getTurn())]++;
	    new_eval.golds[playerToIndex(alt(state.getTurn()))]--;
	    assert(new_eval.golds[playerToIndex(BLACK)] + new_eval.golds[playerToIndex(WHITE)] == 4);
	  }
	}
	return new_eval.value(progress16);
      }
    };
  } // namespace eval
} // namespace osl

#endif /* EVAL_MINORPIECEBONUS_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
