/* progressEval.h
 */
#ifndef EVAL_PROGRESSEVAL_H
#define EVAL_PROGRESSEVAL_H

#include "osl/effect_util/pin.h"
#include "osl/eval/pieceEval.h"
#include "osl/eval/minorPieceBonus.h"
#include "osl/eval/ppair/piecePairPieceEval.h"
#include "osl/eval/endgame/attackDefense.h"
#include "osl/eval/mobilityTable.h"
#include "osl/progress/effect5x3.h"
#include "osl/progress/effect5x3d.h"
#include "osl/progress/progress32.h"
#include "osl/mobility/rookMobility.h"
#include "osl/mobility/bishopMobility.h"
#include "osl/mobility/lanceMobility.h"
#include "osl/state/numEffectState.h"
#include <boost/static_assert.hpp>
using namespace osl::mobility;
namespace osl
{
  namespace eval
  {
    /**
     * פȽפʬȤɾؿ.
     * ɬפǤСפȽפɾؿtemplate parameterˤ롥
     */
    class ProgressEval
    {
    public:
      typedef PiecePairPieceEval opening_eval_t;
      typedef endgame::AttackDefense endgame_eval_t;
      typedef progress::Effect5x3 progress_t;
      typedef progress::Effect5x3d defense_t;
    private:
      opening_eval_t opening_eval;
      endgame_eval_t endgame_eval;
      progress_t current_progress;
      defense_t defense_effect;
      MinorPieceBonus minor_piece_bonus;
      int pin_bonus;
      // Records pinned pieces of a player, not the opponent's
      // (owner of pieces == player)
      CArray<PieceMask, 2> pin_mask;
      int mobility_bonus;
      int attack_rooks;
      enum{
	INVALID=EvalTraits<BLACK>::MAX_VALUE+1,
      };
      mutable int cache;
    public:
      ProgressEval(const NumEffectState& state);
      void changeTurn() {}
      static bool initialized() 
      { 
	return opening_eval_t::initialized();
      } 
      static bool setUp(const char *filename)
      {
	return opening_eval_t::setUp(filename);
      }
      static bool setUp()
      {
	return opening_eval_t::setUp();
      }

      /** roundup  2^n Ǥ뤳 */
      static const int ROUND_UP = 64;
      /** ٥ڥʥƥ16 */
      static int attackDefenseBonusT16(Progress16 black, Progress16 white,
				       Progress16 black_defense,
				       Progress16 white_defense) 
      {
	return 
	  (white.value() * 2 - white_defense.value()
	   - black.value() * 2 + black_defense.value())
	  * 3060 / 2;
      }
      static int composeValue(int value_opening, int value_endgame, 
			      Progress16 progress16,
			      Progress16 black,
			      Progress16 white,
			      Progress16 black_defense,
			      Progress16 white_defense,
			      int minor_piece_bonus,
			      int pin_bonus, int mobility_bonus,
			      int attack_rooks)
      {
	BOOST_STATIC_ASSERT(((PtypeEvalTraits<BISHOP>::val * 2 + PtypeEvalTraits<PAWN>::val) %  16) == 0);
	BOOST_STATIC_ASSERT((PtypeEvalTraits<PAWN>::val % 32) == 0);
	assert(progress16.isValid());
	assert(black.isValid());
	assert(white.isValid());
	assert(value_opening % 2 == 0);
	assert(value_endgame % 2 == 0);
	
	/*
	  value_opening * (16 - progress16) + value_endgame * progress16
	  + bonus * (white_progress - black_progress) * prorgress16 / 16 */
	int result = value_opening*16 + 
	  progress16.value() * (value_endgame-value_opening +
				(attackDefenseBonusT16(black, white,
						       black_defense,
						       white_defense))
				/ 16);

	result += mobility_bonus * 12;
	result += minor_piece_bonus;
	result += pin_bonus * progress16.value() / 16;
	if (attack_rooks == 2)
	  result += (PtypeEvalTraits<KNIGHT>::val + PtypeEvalTraits<PAWN>::val) * 16;
	else if (attack_rooks == -2)
	  result -= (PtypeEvalTraits<KNIGHT>::val + PtypeEvalTraits<PAWN>::val) * 16;
	result &= ~(ROUND_UP-1);
	assert(result % 2 == 0);
	return result;
      }
      const Progress16 progress16() const { return current_progress.progress16(); }
      void invalidateCache(){ 
	cache=INVALID;
      }
      int value() const
      {
	if(cache!=INVALID){
	  return cache;
	}
	else{
	  cache=composeValue(opening_eval.value(), endgame_eval.value(), 
			     progress16(),
			     current_progress.progress16bonus(BLACK),
			     current_progress.progress16bonus(WHITE),
			     defense_effect.progress16(BLACK),
			     defense_effect.progress16(WHITE),
			     minor_piece_bonus.value(progress16()),
			     pin_bonus, mobility_bonus, attack_rooks);
	  return cache;
	}
      }
      const Progress32 progress32() const { 
	return Progress32(current_progress.progress16(BLACK).value()
			  + current_progress.progress16(WHITE).value()); 
      }
      static void setValues(const SimpleState&, Progress16 progress16,
			    container::PieceValues&);
      static void setValues(const SimpleState& s, container::PieceValues& o);

      int expect(const NumEffectState& state, Move move) const {
	if (move.isPass())
	  return value();
	progress_t new_progress = current_progress.expect(state, move);
	return composeValue(opening_eval.expect(state, move), endgame_eval.expect(state, move), 
			    progress16(), new_progress.progress16(BLACK),
			    new_progress.progress16(WHITE),
			    defense_effect.progress16(BLACK),
			    defense_effect.progress16(WHITE),
			    minor_piece_bonus.value(progress16()),
			    pin_bonus, mobility_bonus, attack_rooks);
      }
      void update(const NumEffectState& new_state, Move last_move);
    public:
      static int infty()
      {
	assert(endgame_eval_t::infty() <= opening_eval_t::infty());
	// פ
	return composeValue(opening_eval_t::infty(), 0, Progress16(0),
			    Progress16(0), Progress16(0), Progress16(0),
			    Progress16(0), 0, 0, 0, 0);
      }

      static int captureValue(PtypeO ptypeO)
      {
	// פ
	return composeValue(opening_eval_t::captureValue(ptypeO), 0, 
			    Progress16(0), Progress16(0), Progress16(0),
			    Progress16(0), Progress16(0), 0, 0, 0, 0);
      }

      const PieceMask pins(Player player) const { 
	return pin_mask[playerToIndex(player)]; 
      }
      // for debug
      int minorPieceValue() const { 
	return minor_piece_bonus.value(progress16()); 
      }
      int openingValue() const { return opening_eval.value(); }
      int endgameValue() const { return endgame_eval.value(); }

      // public for debugging purpose only
      // also updates pin_mask
      int calculatePinBonus(const NumEffectState& state);

      int calculateMobilityBonus(const NumEffectState& state);

      int calculateAttackRooks(const NumEffectState& state);
 
      int attackDefenseBonus() const
      {
	return progress16().value()
	  * attackDefenseBonusT16(current_progress.progress16bonus(BLACK),
				  current_progress.progress16bonus(WHITE),
				  defense_effect.progress16(BLACK),
				  defense_effect.progress16(WHITE))
	  / 16;
      }
    };

  } // namespace eval
} // namespace osl

#endif /* EVAL_PROGRESSEVAL_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
