/* boardKey.h
 */
#ifndef _BOARD_KEY_H
#define _BOARD_KEY_H
#include "osl/misc/carray.h"
#include "osl/pieceStand.h"
#include <cstddef>

namespace osl
{
namespace hash
{
  /**
   * ֤ޤ̤ξ֤Υϥåͤݻ뤿Υ饹.
   *
   * - 64bit ץåǤ unsigned long long, 2 (ͤʤ1)
   * - 32bit ץåǤ unsigned int, 4 
   * .
   *
   * elements[0] κǲbit player bit Ȥ롥
   */
  template<typename Integer,size_t SIZE>
  class GeneralBoardKey
  {
    CArray<Integer,SIZE> elements;
  public:
    GeneralBoardKey();
    typedef Integer int_t;
    size_t size() const{ 
      return SIZE; 
    }
    int_t operator[](size_t i) const{
      return elements[i];
    }
    int_t& operator[](size_t i){
      return elements[i];
    }
    /**
     * ˲򸡽Ф뤿rplayer bit0Ǥ뤳Ȥǧ
     */
    GeneralBoardKey& operator+=(const GeneralBoardKey& r)
    {
      assert(r.playerBit() == 0);
      for(size_t i=0;i<SIZE;i++)
	elements[i]+=r.elements[i];
      return *this;
    }
    /**
     * ˲򸡽Ф뤿rplayer bit0Ǥ뤳Ȥǧ
     */
    GeneralBoardKey& operator-=(const GeneralBoardKey& r)
    {
      assert(r.playerBit() == 0);
      for(size_t i=0;i<SIZE;i++)
	elements[i]-=r.elements[i];
      return *this;
    }
    void changeTurn()
    {
      elements[0]^=static_cast<int_t>(1);
    }
    /**
     *  whiteλblacksetǤ褦ˤ
     */
    void setPlayer(Player p) 
    { 
      elements[0]=(elements[0]& ~static_cast<int_t>(1))|playerToIndex(p);
    }
    bool playerBit() const
    {
      return (static_cast<int>(elements[0]) & 1);
    }
    bool isPlayerOfTurn(Player p) const
    {
      return playerBit() == playerToIndex(p);
    }
    /**
     * elements[0]κǲ̥ӥåȤ0ˤ
     */
    void setRandom();
  };

  template<typename Integer,size_t SIZE>
  inline bool operator==(const GeneralBoardKey<Integer,SIZE>& l,
			 const GeneralBoardKey<Integer,SIZE>& r)
  {
    for(size_t i=0;i<SIZE;i++)
      if (l[i]!=r[i]) return false;
    return true;
  }
  template<typename Integer,size_t SIZE>
  inline bool operator!=(const GeneralBoardKey<Integer,SIZE>& l,
			 const GeneralBoardKey<Integer,SIZE>& r)
  {
    return !(l==r);
  }
  /**
   * setǻȤΤߤ
   * full orderǤ뤳Ȱʳ˿̣Ϥʤ
   */
  template<typename Integer,size_t SIZE>
  inline bool operator<(const GeneralBoardKey<Integer,SIZE>& l,
			 const GeneralBoardKey<Integer,SIZE>& r)
  {
    for(size_t i=0;i<SIZE-1;i++)
      if (l[i]!=r[i]) return l[i]<r[i];
    return l[SIZE-1]<r[SIZE-1];
  }

  /**
   * BoardKey + hash tableѤsignature
   */
  template<typename BoardKeyBase>
  class GeneralSignatureBoardKey
  {
  public:
    typedef BoardKeyBase base_t;
    typedef typename BoardKeyBase::int_t int_t;
  private:
    BoardKeyBase board_key;
    unsigned int signature;
  public:
    GeneralSignatureBoardKey() : signature(0)
    {
    }
    unsigned int getSignature() const
    {
      return signature;
    }
    void setSignature(int s)
    {
      signature=s;
    }
    const BoardKeyBase& getBoardKey() const{ return board_key; }
    size_t size() const{ 
      return board_key.size(); 
    }
    int_t operator[](size_t i) const{
      return board_key[i];
    }
    int_t& operator[](size_t i){
      return board_key[i];
    }
    GeneralSignatureBoardKey& operator+=(const GeneralSignatureBoardKey& r)
    {
      board_key+=r.board_key;
      signature+=r.signature;
      return *this;
    }
    GeneralSignatureBoardKey& operator-=(const GeneralSignatureBoardKey& r)
    {
      board_key-=r.board_key;
      signature-=r.signature;
      return *this;
    }
    void changeTurn()
    {
      board_key.changeTurn();
    }
    void setPlayer(Player p) 
    { 
      board_key.setPlayer(p);
    }
    bool isPlayerOfTurn(Player p) const
    {
      return board_key.isPlayerOfTurn(p);
    }
    void setRandom();
  };

  template<typename T>
  inline bool operator==(const GeneralSignatureBoardKey<T>& l,
			 const GeneralSignatureBoardKey<T>& r)
  {
    /**
     * signature㤦¿
     */
    return l.getSignature() == r.getSignature() &&
      l.getBoardKey() == r.getBoardKey();
  }
  template<typename T>
  inline bool operator!=(const GeneralSignatureBoardKey<T>& l,
			 const GeneralSignatureBoardKey<T>& r)
  {
    return !(l==r);
  }
  /**
   * setǻȤΤߤ.
   * full orderǤ뤳Ȱʳ˿̣Ϥʤ
   */
  template<typename T>
  inline bool operator<(const GeneralSignatureBoardKey<T>& l,
			const GeneralSignatureBoardKey<T>& r)
  {
    if (l.getSignature() < r.getSignature()) return true;
    else if (r.getSignature() < l.getSignature()) return false;
    return l.getBoardKey() < r.getBoardKey();
  }

  /**
   * SignatureBoardKey + (piece stand)
   */
  template<typename SignatureBoardKeyBase>
  struct GeneralHashKey
  {
    typedef SignatureBoardKeyBase base_t;
    SignatureBoardKeyBase signatureKey;
    PieceStand pieceStand;

    typedef typename SignatureBoardKeyBase::int_t int_t;

    GeneralHashKey()
    {
    }
    const base_t& getSignatureKey() const{
      return signatureKey;
    }
    unsigned int getSignature() const
    {
      return signatureKey.getSignature();
    }
    void setSignature(int s)
    {
      signatureKey.setSignature(s);
    }
    const PieceStand& getPieceStand() const{
      return pieceStand;
    }
    void setPieceStand(const PieceStand& p){
      pieceStand=p;
    }
    size_t size() const{ 
      return signatureKey.size(); 
    }
    int_t operator[](size_t i) const{
      return signatureKey[i];
    }
    int_t& operator[](size_t i){
      return signatureKey[i];
    }
    /**
     * ξƱɤ.
     * ֤ۤʤΤϰۤʤ
     */
    bool isSameBoard(const GeneralHashKey& key) const
    {
      return signatureKey == key.getSignatureKey();
    }
    GeneralHashKey& operator+=(const GeneralHashKey& r)
    {
      signatureKey+=r.signatureKey;
      pieceStand.addAtmostOnePiece(r.pieceStand);
      return *this;
    }
    GeneralHashKey& operator-=(const GeneralHashKey& r)
    {
      signatureKey-=r.signatureKey;
      pieceStand.subAtmostOnePiece(r.pieceStand);
      return *this;
    }
    const PieceStand blackStand() const 
    { 
      return pieceStand;
    }
    void changeTurn()
    {
      signatureKey.changeTurn();
    }
    void setPlayer(Player p) 
    { 
      signatureKey.setPlayer(p);
    }
    bool isPlayerOfTurn(Player p) const
    {
      return signatureKey.isPlayerOfTurn(p);
    }
    Player turn() const { return isPlayerOfTurn(BLACK) ? BLACK : WHITE; }
    /**
     * pieceStandˤϿʤ
     */
    void setRandom();
    
  };
  template<typename T>
  inline bool operator==(const GeneralHashKey<T>& l,
			 const GeneralHashKey<T>& r)
  {
    /**
     * signatureKey㤦¿
     */
    return l.signatureKey == r.signatureKey &&
      l.pieceStand == r.pieceStand;
  }
  template<typename T>
  inline bool operator!=(const GeneralHashKey<T>& l,
			 const GeneralHashKey<T>& r)
  {
    return !(l==r);
  }
  /**
   * setǻȤΤߤ
   * full orderǤ뤳Ȱʳ˿̣Ϥʤ
   */
  template<typename T>
  inline bool operator<(const GeneralHashKey<T>& l,
			const GeneralHashKey<T>& r)
  {
    if (l.pieceStand < r.pieceStand) return true;
    else if (r.pieceStand < l.pieceStand) return false;
    return l.signatureKey < r.signatureKey;
  }

  typedef GeneralBoardKey<unsigned int,4> BoardKey32;
  typedef GeneralBoardKey<unsigned long long,2> BoardKey64;
  typedef GeneralSignatureBoardKey<BoardKey32> SignatureKey32;
  typedef GeneralSignatureBoardKey<BoardKey64> SignatureKey64;
  typedef GeneralHashKey<SignatureKey32> HashKey32;
  typedef GeneralHashKey<SignatureKey64> HashKey64;
} // namespace hash
} // namespace osl

#endif /* _BOARD_KEY_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

