#ifndef _MOVE_GENERATOR_ADD_EFFECT8_TCC
#define _MOVE_GENERATOR_ADD_EFFECT8_TCC
#include "osl/move_generator/addEffect8.h"
#include "osl/move_generator/open.h"
#include "osl/move_generator/addEffect8Table.h"
#include "osl/misc/loki.h"

#include <boost/static_assert.hpp>

using namespace osl;

namespace osl
{
  namespace move_generator
  {
    namespace {
      /**
       * ûĶdropˤ.
       * @param P(template) - ֤Υץ쥤䡼
       * @param T(template) - ֤Ȥμ(basic)
       * @param state - 
       * @param target - ζ̤position(redundant)
       * @param action - callback
       */
      template <Player P,Ptype T,class Action>
      void generateShortDrop(NumEffectState const& state,Position target,Action& action)
      {
	Position targetForP=target.positionForBlack(P);
	for(int i=0;;i++){
	  Position pos=Add_Effect8_Table.getDropPosition(T,targetForP,i);
	  if(pos.isPieceStand()) break;
	  pos=pos.positionForBlack<P>();
	  if(state.getPieceAt(pos).isEmpty() &&
	     (T!=PAWN || !state.isPawnMaskSet<P>(pos.x()))){
	    if((T==BISHOP || T==ROOK) && 
	       state.hasEffectBy<PlayerTraits<P>::opponent>(pos)) continue;
	    action.dropMove(pos,T,P);
	  }
	}
      }

      /**
       * ơlong drop롥
       * @param P(template) - ֤Υץ쥤䡼
       * @param T(template) - ֤Ȥμ(basic)
       * @param state - 
       * @param to - ɲä
       * @param from - ֤Ϥ
       * @param offset - ֤ؤoffset
       * @param action - callback
       */
      template <Player P,Ptype T,class Action>
      void generateLongDropWithOffset(NumEffectState const& state,Position to,Position from, Offset offset,int countMax,Action& action){
	const Player altP=PlayerTraits<P>::opponent;
	int count=0;
	for(;;){
	  Piece p;
	  for(;(p=state.getPieceAt(from)).isEmpty();from+=offset){
	    if(T==LANCE)
	      action.dropMove(from,T,P);
	    else if(!state.hasEffectBy<altP>(from)){
	      action.dropMove(from,T,P);
	      if(++count>=countMax) return;
	    }
	  }
	  if(!p.isOnBoardByOwner<P>()) return;
	  if(!Ptype_Table.getEffect(p.ptypeO(),from,to).hasEffect()) return;
	  from+=offset;
	}
      }
      
      /**
       * ĹĶdropˤ.
       * ɲ.
       * @param P(template) - ֤Υץ쥤䡼
       * @param T(template) - ֤Ȥμ(basic)
       * @param state - 
       * @param target - ζ̤position(redundant)
       * @param action - callback
       */
      template <Player P,Ptype T,class Action>
      void generateLongDrop(NumEffectState const& state,Position target,Action& action)
      {
	BOOST_STATIC_ASSERT(T==ROOK || T==BISHOP || T==LANCE);
	// 8˵ؤdropshortǺ롥
	generateShortDrop<P,T,Action>(state,target,action);
	Position targetForP=target.positionForBlack(P);
	// 8˵˶𤬤(ˤʤʤ)ľ
	for(int i=0;;i++){
	  Offset offset=Add_Effect8_Table.getLongDropDirect(T,targetForP,i);
	  if(offset.zero()) break;
	  offset=offset.blackOffset<P>();
	  Position pos=target+offset;
	  assert(pos.isOnBoard());
	  Position from=pos+offset;
	  if(state.getPieceAt(pos).isEmpty()){
	    Piece p=state.getPieceAt(from);
	    // ֤ûλΤOK
	    if(!p.isOnBoardByOwner<P>() ||
	       !state.hasEffectByPiece(p,pos)) continue;
	    from+=offset;
	  }
	  generateLongDropWithOffset<P,T,Action>(state,pos,from,offset,2,action);
	}
	for(int i=0;;i++){
	  PO po=Add_Effect8_Table.getLongDropPosition(T,targetForP,i);
	  if(po.first.isPieceStand()) break;
	  Position pos=po.first.positionForBlack<P>();
	  Offset offset=po.second.blackOffset<P>();
	  assert(!offset.zero());
	  generateLongDropWithOffset<P,T,Action>(state,pos-offset,pos,offset,2,action);
	}
	if(T==BISHOP){
	  for(int i=0;;i++){
	    POO poo=Add_Effect8_Table.getLongDrop2Position(T,targetForP,i);
	    if(poo.first.isPieceStand()) break;
	    Position pos=poo.first.positionForBlack<P>();
	    Offset offset1=poo.second.first.blackOffset<P>();
	    assert(!offset1.zero());
	    Offset offset2=poo.second.second.blackOffset<P>();
	    assert(!offset2.zero());
	    Piece p=state.getPieceAt(pos);
	    if(p.isEmpty()){
	      generateLongDropWithOffset<P,T,Action>(state,pos-offset1,
						     pos,offset1,2,action);
	      generateLongDropWithOffset<P,T,Action>(state,pos-offset2,
						     pos+offset2,offset2,1,
						     action);
	    }
	    else if(p.isOnBoardByOwner<P>()){
	      if(state.hasEffectByPiece(p,pos-offset1)){
		generateLongDropWithOffset<P,T,Action>(state,pos-offset1,
						       pos+offset1,offset1,
						       2,action);
	      }
	      if(state.hasEffectByPiece(p,pos-offset2)){
		generateLongDropWithOffset<P,T,Action>(state,pos-offset2,
						       pos+offset2,offset2,
						       2,action);
	      }
	    }
	  }
	}
      }

      /**
       * unblockableưunblockableĤ.
       * @param P(template) - ¦Υץ쥤䡼
       * @param T(template) - ¦ζμ
       * @param isPromote(template) - äƤι(뤫ǥå)
       * @param state - 
       * @param from - ⤷褦Ȥΰ
       * @param target - ⤷褦Ȥޥ
       */
      template <Player P,Ptype T,class Action,bool isPromote>
      void generateShortMove(NumEffectState const& state,Piece attacker,
			     Position target, Action& action)
      {
	Position from=attacker.position();
	Offset32 o32=Offset32(from,target).blackOffset32<P>();
	for(int i=0;;i++){
	  Offset o=Add_Effect8_Table.getShortMoveOffset(isPromote,T,o32,i);
	  if(o.zero()) break;
	  Position pos=target+o.blackOffset<P>();
	  if(!pos.isOnBoard()) continue;
	  if(isPromote &&
	     !from.canPromote<P>() &&
	     !pos.canPromote<P>()) continue;
	  if(!isPromote && 
	     (T==PAWN ? pos.canPromote<P>() :
	      (PtypeTraits<T>::isBasic 
	       && !PtypePlayerTraits<T,P>::canDropTo(pos))))
	    continue;
	  Piece p=state.getPieceAt(pos);
	  if(p.template canMoveOn<P>()){
	    action.unknownMove(from,pos,p,
			       (isPromote ? PtypeFuns<T>::promotePtype : T),
			       isPromote,P);
	  }
	}
      }

      /**
       * from˥ץ쥤䡼PμTζ𤬤ä̤ɤ.
       * ɲäޤ.
       * ֤ʤΤ
       * @param P(template) - Ĥץ쥤䡼
       * @param T - μ(ROOK, BISHOP, LANCEΤ줫)
       * @param state - ̤ξ
       * @param from - 𤬤Ȳꤹޥ
       * @param to - ̵ͭȽꤷޥ
       * @param blocker - ʤ˥֥åƤ֤
       * @param offset - tofromؤoffset(redundant)
       */
      template<Player P,Ptype T>
      bool canAddLongEffect(NumEffectState const& state,Position from,Position to,Piece& blocker,Offset offset)
      {
	Piece p;
	for(Position pos=to+offset;pos!=from;pos+=offset){
	  Piece p=state.getPieceAt(pos);
	  if(!p.isEmpty()){
	    if(!p.isOnBoardByOwner<P>() ||
	       !Ptype_Table.getEffect(p.ptypeO(),pos,to).hasEffect()){
	      blocker=p;
	      assert(!blocker.isEdge());
	      return false;
	    }
	  }
	}
	return true;
      }
      /**
       * from˥ץ쥤䡼PμTζ𤬤ä̤ɤ.
       * ɲäޤ.
       * ֤ʤΤ
       * @param P(template) - Ĥץ쥤䡼
       * @param T - μ(ROOK, BISHOP, LANCEΤ줫)
       * @param state - ̤ξ
       * @param from - 𤬤Ȳꤹޥ
       * @param to - ̵ͭȽꤷޥ
       * @param blocker - ʤ˥֥åƤ֤
       */
      template<Player P,Ptype T>
      bool canAddLongEffect(NumEffectState const& state,Position from,Position to,Piece& blocker)
      {
	Offset offset=Board_Table.getShortOffset(Offset32(from,to));
	return canAddLongEffect<P,T>(state,from,to,blocker,offset);
      }
      /**
       * ĹĶμ.
       * @param P(template) - ¦Υץ쥤䡼
       * @param T(template) - ¦ζμ
       * @param isPromote(template) - äƤι(뤫ǥå)
       * @param state - 
       * @param attacker - ⤷褦Ȥ
       * @param from - ⤷褦Ȥΰ
       * @param target - ⤷褦Ȥޥ
       */
      template <Player P,Ptype T,class Action,bool canPromote>
      void generateLongMove(NumEffectState const& state,Piece attacker,
			    Position target, Action& action)
      {
	Position from=attacker.position();
	Offset32 o32=Offset32(from,target).blackOffset32<P>();
	// ֤Ũζ
	OffsetPair op=Add_Effect8_Table.getBetweenOffset(T,o32);
	if(!op.first.zero()){
	  Position pos=target+op.first.blackOffset<P>();
	  Offset offset=op.second.blackOffset<P>();
	  if(pos.isOnBoard()){
	    Piece blocker=Piece::EMPTY();
	    if(!canAddLongEffect<P,T>(state,from,pos,blocker,-offset) &&
	       state.hasEffectByPiece(attacker,blocker.position())){
	      assert(!blocker.isEmpty() && !blocker.isEdge());
	      Position pos1=blocker.position();
	      if(blocker.isOnBoardByOwner<P>()){
		// open
		Open<P>::generate
		  (state,blocker,action,pos,
		   longToShort(Board_Table.getLongDirection<BLACK>(Offset32(pos,from).blackOffset32<P>())));
	      }
	      else{
		if(canPromote && T!=LANCE && 
		   (from.canPromote<P>() || pos1.canPromote<P>())){
		  action.unknownMove(from,pos1,state.getPieceAt(pos1),
				     PtypeFuns<T>::promotePtype,true,P);
		}
		else if (T!=LANCE || PtypePlayerTraits<KNIGHT,P>::canDropTo(pos1))
		  action.unknownMove(from,pos1,state.getPieceAt(pos1),T,false,P);
	      }
	    }
	  }
	}
	// ưû
	// promoteʤ
	for(int i=0;;i++){
	  Offset o=Add_Effect8_Table.getShortMoveOffset(false,T,o32,i);
	  if(o.zero()) break;
	  Position pos=target+o.blackOffset<P>();
	  if(pos.isOnBoard() && 
	     (T==LANCE || !canPromote || 
	      (!from.canPromote<P>() && (!pos.canPromote<P>()))) &&
	     state.getPieceAt(pos).template canMoveOn<P>() &&
	     state.hasEffectByPiece(attacker,pos) &&
	     // 2ܤ餺˹֤ưȤϤʤ
	     (T!=LANCE || PtypePlayerTraits<KNIGHT,P>::canDropTo(pos))
	     ){
	    action.unknownMove(from,pos,state.getPieceAt(pos),
			       T,false,P);
	  }
	}
	// promoteβǽ
	for(int i=0;;i++){
	  Offset o=Add_Effect8_Table.getShortMoveOffset(true,T,o32,i);
	  if(o.zero()) break;
	  Position pos=target+o.blackOffset<P>();
	  if(pos.isOnBoard() && state.getPieceAt(pos).template canMoveOn<P>() &&
	     (from.canPromote<P>() || pos.canPromote<P>()) &&
	     state.hasEffectByPiece(attacker,pos)
	     ){
	    action.unknownMove(from,pos,state.getPieceAt(pos),
			       PtypeFuns<T>::promotePtype,true,P);
	  }
	}
	// ưĹ
	if(T!=LANCE){
	  for(int i=0;;i++){
	    OffsetPair op=Add_Effect8_Table.getLongMoveOffset(T,o32,i);
	    if(op.first.zero()) break;
	    Position pos1=target+op.first.blackOffset<P>();
	    Position pos2=target+op.second.blackOffset<P>();
	    Piece blocker=Piece::EMPTY();
	    if(pos1.isOnBoard() && pos2.isOnBoard() &&
	       state.getPieceAt(pos1).template canMoveOn<P>() &&
	       state.hasEffectByPiece(attacker,pos1) &&
	       canAddLongEffect<P,T>(state,pos1,pos2,blocker)){
	      if(canPromote && 
		 (from.canPromote<P>() || pos1.canPromote<P>())){
		action.unknownMove(from,pos1,state.getPieceAt(pos1),
				   PtypeFuns<T>::promotePtype,true,P);
	      }
	      else action.unknownMove(from,pos1,state.getPieceAt(pos1),T,false,P);
	    }
	  }
	}
      }

      /**
       * promoteǽ­ûˤ Functor
       */
      template <Player P,Ptype T,class Action>
      class ShortPieceAction
      {
	NumEffectState const& state;
	Position target;
	Action& action;
      public:
	ShortPieceAction(NumEffectState const& s,Position p,Action& a)
	  :state(s),target(p),action(a)
	{}
	/**
	 * forEachOnBoardƤФ
	 */
	void operator()(Piece p)
	{
	  if (p.isPromotedNotKingGold())
	    {
	      generateShortMove<P,PtypeFuns<T>::promotePtype,Action,false>(state,p,target,action);
	    }
	  else
	    {
	      generateShortMove<P,T,Action,true>(state,p,target,action);
	      generateShortMove<P,T,Action,false>(state,p,target,action);
	    }  
	}
      };

      /**
       * Goldˤ Functor
       */
      template <Player P,class Action>
      class GoldAction
      {
	NumEffectState const& state;
	Position target;
	Action& action;
      public:
	GoldAction(NumEffectState const& s,Position p,Action& a)
	  :state(s),target(p),action(a)
	{}
	/**
	 * forEachOnBoardƤФ
	 */
	void operator()(Piece p)
	{
	  generateShortMove<P,GOLD,Action,false>(state,p,target,action);
	}
      };

      /**
       * promoteǽ­Ĺˤ Functor
       */
      template <Player P,Ptype T,class Action>
      class LongPieceAction
      {
	NumEffectState const& state;
	Position target;
	Action& action;
      public:
	LongPieceAction(NumEffectState const& s,Position p,Action& a)
	  :state(s),target(p),action(a)
	{}
	/**
	 * forEachOnBoardƤФ
	 */
	void operator()(Piece p)
	{
	  Position from=p.position();
	  if (p.isPromotedNotKingGold())
	    {
	      if (T==LANCE)
		generateShortMove<P,PtypeFuns<T>::promotePtype,Action,false>(state,p,target,action);
	      else
		generateLongMove<P,PtypeFuns<T>::promotePtype,Action,false>(state,p,target,action);
	    }
	  else
	    {
	      generateLongMove<P,T,Action,true>(state,p,target,action);
	    }  
	}
      };

      template<Player P,Ptype T,class Action>
      void generateShort(const NumEffectState& state,Position target,
			 Action& action)
      {
	BOOST_STATIC_ASSERT((PtypeTraits<T>::isBasic));
	BOOST_STATIC_ASSERT((PtypeTraits<T>::canPromote));
	typedef ShortPieceAction<P,T,Action> action_t;
	action_t gkAction(state,target,action);
	state.template forEachOnBoard<P,T,action_t>(gkAction);
	/** drop move */
	if (state.template hasPieceOnStand<P,T>()){
	  generateShortDrop<P,T,Action>(state,target,action);
	}
      }

      template<Player P,Ptype T,class Action>
      void generateLong(const NumEffectState& state,Position target,
			Action& action)
      {
	BOOST_STATIC_ASSERT((PtypeTraits<T>::isBasic));
	BOOST_STATIC_ASSERT((PtypeTraits<T>::canPromote));
	typedef LongPieceAction<P,T,Action> action_t;
	action_t gkAction(state,target,action);
	state.template forEachOnBoard<P,T,action_t>(gkAction);
      }

      template<Player P,class Action>
      void generateGold(const NumEffectState& state,Position target,
			Action& action)
      {
	typedef GoldAction<P,Action> action_t;
	action_t gkAction(state,target,action);
	state.template forEachOnBoard<P,GOLD,action_t>(gkAction);
	/** drop move */
	if (state.template hasPieceOnStand<P,GOLD>()){
	  generateShortDrop<P,GOLD,Action>(state,target,action);
	}
      }
    }

    template<Player P>
    template<class Action>
    void AddEffect8<P>::
    generateBigDrop(const NumEffectState& state,Action& action)
    {
      Position target=state.getKingPosition<PlayerTraits<P>::opponent>();
      if (state.template hasPieceOnStand<P,BISHOP>()){
	generateLongDrop<P,BISHOP,Action>(state,target,action);
      }
      if (state.template hasPieceOnStand<P,ROOK>()){
	generateLongDrop<P,ROOK,Action>(state,target,action);
      }
    }

    template<Player P>
    template<class Action>
    void AddEffect8<P>::
    generateNotBigDrop(const NumEffectState& state,Action& action)
    {
      Position target=state.getKingPosition<PlayerTraits<P>::opponent>();
      generateShort<P,PAWN,Action>(state,target,action);
      generateLong<P,LANCE,Action>(state,target,action);
      if (state.template hasPieceOnStand<P,LANCE>()){
	generateLongDrop<P,LANCE,Action>(state,target,action);
      }
      generateShort<P,KNIGHT,Action>(state,target,action);
      generateShort<P,SILVER,Action>(state,target,action);
      generateGold<P,Action>(state,target,action);
      // no king move
      generateLong<P,BISHOP,Action>(state,target,action);
      generateLong<P,ROOK,Action>(state,target,action);
    }

    template<Player P>
    template<class Action>
    void AddEffect8<P>::
    generate(const NumEffectState& state,Action& action)
    {
      generateNotBigDrop(state,action);
      generateBigDrop(state,action);
    }
  }
}
#endif /* _MOVE_GENERATOR_ADD_EFFECT8_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
