#ifndef _GENERATE_ESCAPE_MOVES_TCC
#define _GENERATE_ESCAPE_MOVES_TCC

#include "osl/move_generator/escape_.h"
#include "osl/move_action/concept.h"
#include "osl/move_action/store.h"
#include "osl/container/moveVector.h"
namespace osl
{
  namespace move_generator
  {
    template<Player P,class State,class Action>
    void Escape<P,State,Action>::
    generateDropAll(const State& state,Position to,Action& action)
    {
      generateDrop<PAWN>(state,to,action);
      generateDrop<LANCE>(state,to,action);
      generateDrop<KNIGHT>(state,to,action);
      generateDrop<SILVER>(state,to,action);
      generateDrop<GOLD>(state,to,action);
      generateDrop<BISHOP>(state,to,action);
      generateDrop<ROOK>(state,to,action);
    }

    template<Player P,class State,class Action>
    void Escape<P,State,Action>::
    generateBlockingKing(const State& state,Piece p,Position from,Action &action,move_action::NotKingOpenFilter<P,State,Action>& safe_action)
    {
      Position to=p.position();
      Offset offset=Board_Table.getShortOffset(Offset32(from,to));
      assert(!offset.zero());
      for(Position pos=to+offset;pos!=from;pos+=offset){
	assert(state.getPieceAt(pos).isEmpty()); 
	// ư
	typedef effect_action::SimpleMove<move_action::NotKingOpenFilter<P,State,Action> > simple_action_t;
	simple_action_t simpleAction(safe_action);
	state.template forEachEffectNotBy<P ,simple_action_t>(pos,p,simpleAction);
	// ֤
	generateDropAll(state,pos,action);
      }
    }

    template<Player P,class State,class Action>
    void Escape<P,State,Action>::
    generateBlocking(const State& state,Piece p,Position from,Action &action)
    {
      Position to=p.position();
      assert(from.isOnBoard());
      Offset offset=Board_Table.getShortOffset(Offset32(from,to));
      assert(!offset.zero());
      for(Position pos=to+offset;pos!=from;pos+=offset){
	assert(state.getPieceAt(pos).isEmpty());
	// ư
	typedef effect_action::SimpleMove<Action> simple_action_t;
	simple_action_t simpleAction(action);
	state.template forEachEffectNotBy<P ,simple_action_t>(pos,p,simpleAction);
	// ֤
	generateDropAll(state,pos,action);
      }
    }

    template<Player P,class State,class Action>
    template<Ptype Type>
    void Escape<P,State,Action>::
    generateMovesBy(const State& state,Piece p,Piece const attackerPiece,Action& action)
    {
      if(attackerPiece==Piece::EMPTY()){
	/** escape only */
	generateEscape<Type>(state,p,action);
      }
      else if(Type == KING){
	Position attackFrom=attackerPiece.position();
	move_action::NotKingOpenFilter<P,State,Action> safe_action(state,action);

	generateCaptureKing( state, p, attackFrom, safe_action );
	/** escape */
	generateEscape<Type>( state,p,action);
	/** 礤 */
	generateBlockingKing(state,p,attackFrom,action,safe_action);
      }
      else{
	Position attackFrom=attackerPiece.position();
	generateCapture( state, p, attackFrom, action );
	/** escape */
	generateEscape<Type>( state,p,action);
	/** 礤 */
	generateBlocking(state,p,attackFrom,action);
      }
    }

    template<Player P,class State,class Action>
    void Escape<P,State,Action>::
    generateKingEscape(const State& state,Move last_move,Action& action){
      Piece kingPiece=state.getPieceOf(KingTraits<P>::index);
      Piece attackerPiece;
#ifndef NDEBUG
      const bool isAttacked=
#endif
	state.template lastMoveIsCheck<P>(last_move,attackerPiece);
      assert(isAttacked); // ꤫ʤΤ˸Ƥ֤
      generateMovesBy<KING>(state,kingPiece,attackerPiece,action);
    }
    template<Player P,class State,class Action>
    template<Ptype Type>
    void Escape<P,State,Action>::
    generateMovesBy(const State& state,Piece p,Action& action)
    {
      Position target=p.position();
      Piece attackerPiece;
      bool isAttacked=
	state.template hasEffectBy<PlayerTraits<P>::opponent>(target,attackerPiece);
      assert(isAttacked); // ꤫ʤΤ˸Ƥ֤
      generateMovesBy<Type>(state,p,attackerPiece,action);
    }

    template<Player P,class State,class Action>
    void Escape<P,State,Action>::
    generateMoves(const State& state,Piece piece,Piece attackerPiece,Action& action)
    {
      switch(piece.ptype()){
      case PAWN: generateMovesBy<PAWN>(state,piece,attackerPiece,action); break;
      case LANCE: generateMovesBy<LANCE>(state,piece,attackerPiece,action); break;
      case KNIGHT: generateMovesBy<KNIGHT>(state,piece,attackerPiece,action); break;
      case SILVER: generateMovesBy<SILVER>(state,piece,attackerPiece,action); break;
      case PPAWN: generateMovesBy<PPAWN>(state,piece,attackerPiece,action); break;
      case PLANCE: generateMovesBy<PLANCE>(state,piece,attackerPiece,action); break;
      case PKNIGHT: generateMovesBy<PKNIGHT>(state,piece,attackerPiece,action); break;
      case PSILVER: generateMovesBy<PSILVER>(state,piece,attackerPiece,action); break;
      case GOLD: generateMovesBy<GOLD>(state,piece,attackerPiece,action); break;
      case BISHOP: generateMovesBy<BISHOP>(state,piece,attackerPiece,action); break;
      case PBISHOP: generateMovesBy<PBISHOP>(state,piece,attackerPiece,action); break;
      case ROOK: generateMovesBy<ROOK>(state,piece,attackerPiece,action); break;
      case PROOK: generateMovesBy<PROOK>(state,piece,attackerPiece,action); break;
      case KING: generateMovesBy<KING>(state,piece,attackerPiece,action); break;
      default: assert(0);
	
      }
    }
    template<Player P,class State,class Action>
    void Escape<P,State,Action>::
    generateMoves(const State& state,Piece piece,Action& action)
    {
      assert(piece.owner() == P);
      Position target=piece.position();
      Piece attackerPiece;
      state.template hasEffectBy<PlayerTraits<P>::opponent>(target,attackerPiece);
      generateMoves(state,piece,attackerPiece,action);
    }
  } // namespace move_generator

  template <class State>
  void GenerateEscapeKing::generate(const State& state, MoveVector& out)
  {
    typedef move_action::Store store_t;
    store_t store(out);
    const Piece king =state.getKingPiece(state.getTurn());
    move_generator::GenerateEscapeOfTurn::generate(state, king, store);
  }
} // namespace osl

#endif // _GENERATE_ESCAPE_MOVES_TCC
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

