#ifndef _MOVE_GENERATOR_PROMOTE_TCC
#define _MOVE_GENERATOR_PROMOTE_TCC

#include "osl/move_generator/promote_.h"
#include "osl/direction.h"
#include "osl/directionTraits.h"
#include "osl/ptypeTraits.h"
namespace osl
{
  namespace move_generator
  {
    namespace promote{
      template<Player P,Ptype T,bool noCapture,Direction Dir>
      class AlwaysPromoteDir{
	template <class State,class Action>
	static void generateIfValid(const State& state,Piece p, Action& action, 
				    Int2Type<true> /*isLong*/, Int2Type<true>){
	  Position pos=p.position();
	  const Offset offset=DirectionPlayerTraits<Dir,P>::offset();
	  assert(!offset.zero());
	  Position toPos=pos+offset;
	  Piece p1=state.getPieceAt(toPos);
	  while (p1.isEmpty()){
	    action.simpleMove(pos,toPos,PtypeFuns<T>::promotePtype,true,P);
	    toPos=toPos+offset;
	    p1=state.getPieceAt(toPos);
	  }
	  if (!noCapture && p1.canMoveOn<P>()){
	    action.unknownMove(pos,toPos,p1,PtypeFuns<T>::promotePtype,true,P);
	  }
	}
	// short move
	template <class State,class Action>
	static void generateIfValid(const State& state,Piece p, Action& action, 
				    Int2Type<false> /*isLong*/,Int2Type<true>){
	  Position pos=p.position();
	  const Offset offset=DirectionPlayerTraits<Dir,P>::offset();
	  Position toPos=pos+offset;
	  Piece p1=state.getPieceAt(toPos);
	  if (p1.isEmpty()){
	    action.simpleMove(pos,toPos,PtypeFuns<T>::promotePtype,true,P);
	  }
	  else if (!noCapture && p1.canMoveOn<P>()){
	    action.unknownMove(pos,toPos,p1,PtypeFuns<T>::promotePtype,true,P);
	  }
	}
	template <class State,class Action>
	static void generateIfValid(const State&, Piece, Action&, Int2Type<true>,Int2Type<false>){
	}
	template <class State,class Action>
	static void generateIfValid(const State&, Piece, Action&, Int2Type<false>,Int2Type<false>){
	}
      public:
	template<class State,class Action>
	static void generate(State const& state,Piece p,Action& action){
	  generateIfValid(state,p,action,
			  Int2Type<DirectionTraits<Dir>::isLong>(),
			  Int2Type<(PtypeTraits<T>::moveMask 
				    & DirectionTraits<Dir>::mask) !=0>());
	}
      };
      /**
       * ꤷ𤬾promoteǽʾpromoteư
       */
      template<Player P,Ptype T,bool noCapture>
      class AlwaysPromote{
      public:
	template<class State,class Action>
	static void generate(State const& state,Piece p,Action& action){
	  AlwaysPromoteDir<P,T,noCapture,UL>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,U>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,UR>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,L>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,R>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,DL>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,D>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,DR>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,UUL>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,UUR>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_UL>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_U>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_UR>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_L>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_R>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_DL>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_D>::generate(state,p,action);
	  AlwaysPromoteDir<P,T,noCapture,LONG_DR>::generate(state,p,action);
	}
      };
      template<Player P,Ptype T,bool noCapture,Direction Dir>
      class MayPromoteDir{
	template <class State,class Action>
	static void generateIfValid(const State& state,Piece p, Action& action, 
				    Int2Type<true> /*isLong*/, Int2Type<true>){
	  /**
	   * ΨŪʼǤϤʤ
	   */
	  Position pos=p.position();
	  const Offset offset=DirectionPlayerTraits<Dir,P>::offset();
	  assert(!offset.zero());
	  Position toPos=pos+offset;
	  Piece p1=state.getPieceAt(toPos);
	  while (p1.isEmpty()){
	    if (toPos.canPromote<P>())
	      action.simpleMove(pos,toPos,PtypeFuns<T>::promotePtype,true,P);
	    toPos=toPos+offset;
	    p1=state.getPieceAt(toPos);
	  }
	  if (!noCapture && p1.canMoveOn<P>() && toPos.canPromote<P>()){
	    action.unknownMove(pos,toPos,p1,PtypeFuns<T>::promotePtype,true,P);
	  }
	}
	// short move
	template <class State,class Action>
	static void generateIfValid(const State& state,Piece p, Action& action, Int2Type<false>,Int2Type<true>){
	  Position pos=p.position();
	  const Offset offset=DirectionPlayerTraits<Dir,P>::offset();
	  Position toPos=pos+offset;
	  Piece p1=state.getPieceAt(toPos);
	  if (p1.isEmpty()){
	    action.simpleMove(pos,toPos,PtypeFuns<T>::promotePtype,true,P);
	  }
	  else if (!noCapture && p1.canMoveOn<P>()){
	    action.unknownMove(pos,toPos,p1,PtypeFuns<T>::promotePtype,true,P);
	  }
	}
	template <class State,class Action>
	static void generateIfValid(const State&, Piece, Action&, Int2Type<true>,Int2Type<false>){
	}
	template <class State,class Action>
	static void generateIfValid(const State&, Piece, Action&, Int2Type<false>,Int2Type<false>){
	}
      public:
	template<class State,class Action>
	static void generate(State const& state,Piece p,Action& action){
	  generateIfValid(state,p,action,
			  Int2Type<DirectionTraits<Dir>::isLong>(),
			  Int2Type<(PtypeTraits<T>::moveMask 
				    & DirectionTraits<Dir>::mask) !=0>());
	}
      };
      /**
       * ꤷpromoteǽǤϤʤ
       */
      template<Player P,Ptype T,bool noCapture>
      class MayPromote{
      public:
	template<class State,class Action>
	static void generate(State const& state,Piece p,Action& action){
	  MayPromoteDir<P,T,noCapture,UL>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,U>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,UR>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,UUL>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,UUR>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,LONG_UL>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,LONG_U>::generate(state,p,action);
	  MayPromoteDir<P,T,noCapture,LONG_UR>::generate(state,p,action);
	}
      };

      template<typename State,typename Action,Player P,Ptype T,bool noCapture>
      struct EachOnBoard
      {
	const State& state;
	Action& action;
	EachOnBoard(const State& state,Action& action):state(state),action(action){}
	void operator()(Piece p){
	  if (!p.isPromotedNotKingGold() && 
	     PtypePlayerTraits<T,P>::mayPromote(p.position())){
	    if (p.position().template canPromote<P>()){
	      AlwaysPromote<P,T,noCapture>::generate(state,p,action);
	    }
	    else{
	      MayPromote<P,T,noCapture>::generate(state,p,action);
	    }
	  }
	}
      };
    }

    template<Player P,class State,class Action,bool noCapture>
    template<Ptype T>
    void Promote<P,State,Action,noCapture>::
    generateMovesPtype(const State& state, Action& action){
      typedef promote::EachOnBoard<State,Action,P,T,noCapture> each_t;
      each_t eachOnBoard(state,action);
      // forEachOnBoardMayPromote®ӽǤϤ
      state.template forEachOnBoard<P,T,each_t>(eachOnBoard);
    }

    template<Player P,class State,class Action,bool noCapture>
    void Promote<P,State,Action,noCapture>::
    generateMoves(const State& state, Action& action){
      // promoteβͤι⤤Ƥߤ
      // PAWN600-100=500
      generateMovesPtype<PAWN>(state,action);
      // ROOK1300-950=350
      generateMovesPtype<ROOK>(state,action);
      // BISHOP1150-800=350
      generateMovesPtype<BISHOP>(state,action);
      // LANCE600-400=200
      generateMovesPtype<LANCE>(state,action);
      // KNIGHT600-400=200
      generateMovesPtype<KNIGHT>(state,action);
      // SILVER600-550=50
      generateMovesPtype<SILVER>(state,action);
      // GOLD,KINGpromoteʤΤǽ
    }
  }
}
#endif /* _MOVE_GENERATOR_PROMOTE_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
