/* position.h
 */
#ifndef _POSITION_H
#define _POSITION_H

#include "osl/misc/loki.h"
#include "osl/offset.h"

namespace osl 
{
  /**
   * ɸ.
   *        ̤Υǥå
   * X, Y1-9ϰϤɽ 
   * Xϱ롥YϾ夫
   * ʤ0
   * <pre>
   * (A0)  ......................... (00)
   * (A1)  ......................... (01)
   * (A2) 92 82 72 62 52 42 32 22 12 (02)
   * (A3) 93 83 73 63 53 43 33 23 13 (03)
   * (A4) 94 84 74 64 54 44 34 24 14 (04)
   * (A5) 95 85 75 65 55 45 35 25 15 (05)
   * (A6) 96 86 76 66 56 46 36 26 16 (06)
   * (A7) 97 87 77 67 57 47 37 27 17 (07)
   * (A8) 98 88 78 68 58 48 38 28 18 (08)
   * (A9) 99 89 79 69 59 49 39 29 19 (09)
   * (AA) 9A 8A 7A 6A 5A 4A 3A 2A 1A (0A)
   * (AB) ...........................(0B)
   * (AC) ...........................(0C)
   * (AD) ...........................(0D)
   * (AE) ...........................(0E)
   * (AF) ...........................(0F) 
   * </pre>
   */
  class Position;
  bool operator==(Position l, Position r);

  class Position
  {
    unsigned int position;
    explicit Position(int p) : position(p)
    {
    }
  public:
    static const Position makeDirect(int value) { return Position(value); }
    unsigned int uintValue() const { return position; }
    enum {
      PIECE_STAND=0,
      MIN=0,
      SIZE=0x100
    };
    Position() : position(PIECE_STAND)
    {
    }
    static const Position STAND() { return Position(PIECE_STAND); }
    Position(int x, int y) : position((x*Offset::BOARD_HEIGHT)+y+1)
    {
      assert(position < SIZE);
    }
    static const Position nth(unsigned int i) { return Position(i+MIN); }
    /**
     * ȤƤXɸ֤. Positionɽ˰¸ʤ
     */
    int x() const { return position >> 4; }
    /**
     * ȤƤYɸ֤. Positionɽ˰¸ʤ
     */
    int y() const { return (position&0xf)-1; }
    unsigned int index() const { return position - MIN; }
    static unsigned int indexMax() { return SIZE - MIN; }
    int indexForOffset32() const { return position + (position&0xf0); }

    bool isPieceStand() const { return position == PIECE_STAND; }
    bool isOnBoardSlow() const;
    /**
     * ̾ɽɤȽꡥ
     * 1<=x() && x()<=9 && 1<=y() && y()<=9
     * Positionɽ˰¸롥
     */
    bool isOnBoard() const { 
      unsigned int x=position-0x12;
      x=(x&0x1ff)+(x&0x1f0);
      return ((0x718-x)&0x610)==0x610;
    }
    bool isValid() const;


    const Position positionForBlackSlow(Player player) const;

    const Position positionForBlack(Int2Type<BLACK>) const{
      assert(isOnBoard());
      Position ret=*this;
      return ret;
    }

    const Position positionForBlack(Int2Type<WHITE>) const{
      assert(isOnBoard());
      Position ret=makeDirect(Position(9,9).uintValue()+Position(1,1).uintValue()-uintValue());
      return ret;
    }
    /** 
     * ξ̤ä֤.
     * PIECE_STANDξϰʤ
     */
    template<Player P>
    const Position positionForBlack() const{
      return positionForBlack(Int2Type<P>());
    }
    const Position positionForBlack(Player player) const{
      if(player==BLACK)
	return positionForBlack<BLACK>();
      else
	return positionForBlack<WHITE>();
    }

    const Position rotate180() const 
    {
      return positionForBlack<WHITE>();
    }

    static const Position onBoardMax(){ return Position(9,9); }
    static const Position onBoardMin(){ return Position(1,1); }
  
    /**
     * posONBOARD_MINONBOARD_MAXδ֤ˤ
     */
    bool isOnBoardRegion() const {
      return static_cast<unsigned int>(index()-onBoardMin().index()) 
	<= static_cast<unsigned int>(onBoardMax().index()-onBoardMin().index());
    }

    Position& operator++() {
      position += 1;
      return *this;
    }

    static int reverseX(int x) { return 10-x; }
    static int reverseY(int y) { return 10-y; }
  private:
    bool canPromote(Int2Type<BLACK>) const
    {
      assert(isOnBoard());
      return y() <= 3;
    }
    bool canPromote(Int2Type<WHITE>) const
    {
      assert(isOnBoard());
      return y() >= 7;
    }
  public:
    template <Player P>
    bool canPromote() const { return canPromote(Int2Type<P>()); }
    bool canPromote(Player player) const 
    {
      if (player==BLACK) 
	return canPromote<BLACK>();
      else 
	return canPromote<WHITE>();
    }

    Position& operator+=(Offset offset) {
      position += offset.intValue();
      return *this;
    }
    Position& operator-=(Offset offset) {
      position -= offset.intValue();
      return *this;
    }
    const Position operator+(Offset offset) const {
      Position result(*this);
      return result+=offset;
    }
    const Position operator-(Offset offset) const {
      Position result(*this);
      return result-=offset;
    }
    const Offset operator-(Position other) const {
      return Offset::makeDirect(position - other.position);
    }
  };

  inline bool operator==(Position l, Position r)
  {
    return l.uintValue() == r.uintValue();
  }
  inline bool operator!=(Position l, Position r)
  {
    return ! (l == r);
  }
  std::ostream& operator<<(std::ostream&, Position);

} // namespace osl

#endif /* _POSITION_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
