/* featureSet.h
 */
#ifndef _FEATURESET_H
#define _FEATURESET_H

#include "osl/rating/range.h"
#include "osl/rating/ratedMoveVector.h"
#include "osl/state/numEffectState.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/container/moveVector.h"
#include "osl/stl/vector.h"
#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/noncopyable.hpp>
#include <string>
#include <iosfwd>

namespace osl
{
  namespace rating
  {
    class Group;
    class Feature;
    class RatingEnv;
    class CheckmateIfCapture;
    class CaptureGroup;
    class FeatureSet : boost::noncopyable
    {
      // range  GroupΥСˤʵ
      boost::ptr_vector<Group> groups;
      vector<Feature*> features;	// acquaintance
      vector<range_t> ranges;
      vector<double> weights;
      vector<int> weightslog10;
      /** makeRatḛgroup */
      int capture_group, checkmate_if_capture_group;
      vector<char> normal_groups;
    public:
      FeatureSet();
      virtual ~FeatureSet();
      bool tryLoad(const std::string& input_directory);
      void setWeight(size_t feature_id, const double& value);

      const Group& group(size_t group_id) const { return groups[group_id]; }
      const Feature& feature(size_t feature_id) const { return *features[feature_id]; }
      const range_t& range(size_t group) const { return ranges[group]; }
      const double& weight(size_t feature_id) const { return weights[feature_id]; }
      size_t groupSize() const { return groups.size(); }
      size_t featureSize() const { return features.size(); }
      const RatedMove makeRate(const NumEffectState& state, 
			       const RatingEnv& env, Move move) const;
      const std::string annotate(const NumEffectState& state, 
				 const RatingEnv& env, Move move) const;
      void generateRating(const NumEffectState& state, const RatingEnv& env,
			  RatedMoveVector& out) const;
      void generateLogProb(const NumEffectState& state, const RatingEnv& env,
			   int limit, MoveLogProbVector& out) const;
      int logProbTakeBack(const NumEffectState& state, const RatingEnv& env, Move) const;
      int logProbSeePlus(const NumEffectState& state, const RatingEnv& env, Move) const;
      int logProbKingEscape(const NumEffectState& state, const RatingEnv& env, Move) const;

      void showGroup(std::ostream&, size_t group_id) const;
      void save(const std::string& output_directory, size_t group_id) const;
    protected:
      void add(Feature *f);
      void add(Group *g);
      void add(CaptureGroup *g);
      void add(CheckmateIfCapture *f);
      void addCommon(Group *g);
      void addFinished();
    private:
      const range_t makeRange(size_t group) const;
      int rating(const NumEffectState& state, 
		 const RatingEnv& env, Move move, size_t group_id) const;
    };
    
    class StandardFeatureSet : public FeatureSet
    {
    public:
      explicit StandardFeatureSet(bool allow_load_failure=false);
    };

    /** Τ*/
    class CaptureSet : public FeatureSet
    {
    public:
      explicit CaptureSet(bool allow_load_failure=false);
    };
    /** Τ*/
    class TacticalSet : public FeatureSet
    {
    public:
      explicit TacticalSet(bool allow_load_failure=false);
    };
  }
  extern rating::StandardFeatureSet Standard_Feature_Set;
}

#endif /* _FEATURESET_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
