/* captureGroup.h
 */
#ifndef _CAPTUREGROUP_H
#define _CAPTUREGROUP_H

#include "osl/rating/group.h"
#include "osl/rating/feature/capture.h"

namespace osl
{
  namespace rating
  {
    struct CaptureGroup : public Group
    {
      vector<int> see_range;
      CaptureGroup();
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showAll(os, name_width, range, weights);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	int see = PieceEval::computeDiffAfterMoveForRP(state, move);
	see = see*100/128;
	for (size_t i=0; i<see_range.size()-1; ++i) {
	  if (see < see_range[i+1]) {
	    return i;
	  }
	}
	assert(0);
	abort();
      }
    };

    struct ShadowEffectGroup : public Group
    {
      ShadowEffectGroup() : Group("ShadowEffect")
      {
	push_back(new ShadowEffect1());
	push_back(new ShadowEffect2());
      }
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showAll(os, name_width, range, weights);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv&) const
      {
	return ShadowEffect::count2(state, move.to(), move.player()) -1;
      }
    };

  }
}


#endif /* _CAPTUREGROUP_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
