/* escape.h
 */
#ifndef GROUP_ESCAPE_H
#define GROUP_ESCAPE_H

#include "osl/rating/group.h"
#include "osl/rating/feature/escape.h"

namespace osl
{
  namespace rating
  {
    struct FromEffectGroup : public Group
    {
      FromEffectGroup() : Group("FromEffect")
      {
	for (int a=0; a<3; ++a)
	  for (int d=0; d<3; ++d)
	    push_back(new FromEffect(a, d));
      }
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showAll(os, name_width, range, weights);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	if (move.isDrop())
	  return -1;
	const int index = CountEffect2::index(state, move.from());
	return index;    
      }
    };

    struct PtypeAttackedGroup : public Group
    {
      PtypeAttackedGroup() : Group("PtypeAttacked")
      {
	for (int s=PTYPE_PIECE_MIN; s<=PTYPE_MAX; ++s) {
	  for (int a=PTYPE_MIN; a<=PTYPE_MAX; ++a) {
	    push_back(new PtypeAttacked(static_cast<Ptype>(s), static_cast<Ptype>(a)));
	  }
	}
      }
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showTopN(os, name_width, range, weights, 3);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	if (move.isDrop())
	  return -1;
	const Ptype self = move.oldPtype();
	const Ptype attack = EffectUtil::cheapestPiece(alt(move.player()), state, move.from()).ptype();
	const int index = (self-PTYPE_PIECE_MIN)*(PTYPE_MAX+1-PTYPE_MIN)+attack;
	return index;    
      }
    };
  }
}

#endif /* GROUP_ESCAPE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
