/* positionGroup.h
 */
#ifndef _POSITIONGROUP_H
#define _POSITIONGROUP_H

#include "osl/rating/group.h"
#include "osl/rating/feature/position.h"

namespace osl
{
  namespace rating
  {
    struct RelativeKingXGroup : public Group
    {
      bool attack;
      RelativeKingXGroup(bool a);
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showTopN(os, name_width, range, weights, 3);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	int index = DropPtype::UNIT*RelativeKingX::makeX(attack, state, move);
	index += DropPtype::index(move);
	return index;    
      }  
    };

    struct RelativeKingYGroup : public Group
    {
      bool attack;
      RelativeKingYGroup(bool a) : Group(a ? "KingRelAY" : "KingRelDY"), attack(a)
      {
	for (int y=-8; y<9; ++y) {
	  for (int s=PTYPE_PIECE_MIN; s<=PTYPE_BASIC_MIN; ++s) { // we cannot drop KING 
	    push_back(new RelativeKingY(y, a, static_cast<Ptype>(s), false));
	  }
	  for (int s=PTYPE_BASIC_MIN+1; s<=PTYPE_MAX; ++s) {
	    push_back(new RelativeKingY(y, a, static_cast<Ptype>(s), false));
	    push_back(new RelativeKingY(y, a, static_cast<Ptype>(s), true));
	  }
	}
      }
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showTopN(os, name_width, range, weights, 3);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	int index = DropPtype::UNIT * (RelativeKingY::makeY(attack, state, move)+8);
	index += DropPtype::index(move);
	return index;
      }  
    };

    struct PositionXGroup : public Group
    {
      PositionXGroup();
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showTopN(os, name_width, range, weights, 3);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	int index = DropPtype::UNIT*(PositionX::makeX(move)-1);
	index += DropPtype::index(move);
	return index;    
      }
    };

    struct PositionYGroup : public Group
    {
      PositionYGroup();
      void show(std::ostream& os, int name_width, const range_t& range, 
		const vector<double>& weights) const
      {
	showTopN(os, name_width, range, weights, 3);
      }
      int findMatch(const NumEffectState& state, Move move, const RatingEnv& env) const
      {
	int index = DropPtype::UNIT*(PositionY::makeY(move)-1);
	index += DropPtype::index(move);
	return index;    
      }
    };
  }
}


#endif /* _POSITIONGROUP_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
