/* alphaBeta2.h
 */
#ifndef _ALPHA_BETA2_H
#define _ALPHA_BETA2_H

#include "osl/search/realizationProbability.h"
#include "osl/search/searchBase.h"
#include "osl/search/searchState2.h"
#include "osl/search/searchRecorder.h"
#include "osl/search/passCounter.h"
#include "osl/search/killerMoveTable.h"
#include "osl/search/hasTimer.h"
#include "osl/checkmate/dualCheckmateSearcher.h"
#include "osl/eval/evalTraits.h"
#include "osl/eval/progressEval.h"
#include "osl/misc/realTime.h"
#include "osl/container/moveStack.h"
#include "osl/stat/average.h"
#include "osl/oslConfig.h"
#include <boost/scoped_array.hpp>
#include <iosfwd>

namespace osl
{
  namespace search
  {
    class SimpleHashRecord;
    class SimpleHashTable;
    class MoveGenerator;

    /**
     * AlphaBeta2Tree ΥǡСdefault copy constructor copyǽʤΡ
     */
    struct AlphaBeta2Common
    {
      static const int leaf_limit = 300;

      enum { MaxDepth = SearchState2Core::MaxDepth };
      eval::ProgressEval eval;
      PassCounter pass_count;
      misc::RealTime timer;
      enum MoveType { INITIAL, HASH=INITIAL, TACTICAL, KILLER, PASS, ALL, FINISH };
      /** ߤοǤκ, nextMove()  */
      CArray<MoveType, MaxDepth> move_type;
      typedef FixedCapacityVector<Move,4> killer_t;
      CArray<killer_t, MaxDepth> killers;
      uint64_t blocking;	// obsolete
      const MoveVector *root_ignore_moves; // acquaintance
      
      explicit AlphaBeta2Common(const NumEffectState& s) 
	: eval(s), timer(60), blocking(0), root_ignore_moves(0)
      {
      }
    };

    class AlphaBeta2Parallel;
    /**
     * "tree" of AlphaBeta2, copied by split
     */
    class AlphaBeta2Tree
      : public SearchBase<eval::ProgressEval,SimpleHashTable,CountRecorder,RealizationProbability>,
	public SearchState2, public HasTimer, protected AlphaBeta2Common
    {
    public:
      typedef eval::ProgressEval eval_t;
      enum { MaxDepth = SearchState2Core::MaxDepth };
    protected:
      /** Żõޤ᤿Ρɿ */
      size_t node_count;
      FixedCapacityVector<MoveGenerator*, MaxDepth> generators;
      stat::Average mpn, mpn_cut, alpha_update, last_alpha_update;
      stat::Average ext, ext_limit;
      boost::shared_ptr<AlphaBeta2Parallel> shared;
    protected:
      static CArray<int, MaxDepth> depth_node_count;
      AlphaBeta2Tree(const HashEffectState& s, checkmate_t& checker,
		     SimpleHashTable *t, CountRecorder&);
      // share parallel data for split
      AlphaBeta2Tree(const AlphaBeta2Tree& src, AlphaBeta2Parallel*);
      // do not share parallel data for no-split
      AlphaBeta2Tree(const AlphaBeta2Tree& src);
      ~AlphaBeta2Tree();

    private:
      // timer ط
      /**
       * ͤˤäƤõ
       */
      volatile int stop_by_alarm;
      bool has_stop_alarm;
      volatile int& stopByAlarm();
      const volatile int& stopByAlarm() const;
    public:
      const AlarmSwitch alarmSwitch() { return AlarmSwitch(&stopByAlarm()); }
      void setTimeOut(int seconds);
      void resetTimeOut();
      bool timerAvailable() const { return stopByAlarm() == Alarm::WATCHING; }
    private:
      void throwStop();
    public:
      struct BetaCut {};
      bool stopping() const 
      {
	return stopByAlarm() == Alarm::TIMEOUT || stop;
      }
      void testStop() { 
	if (stopping())
	  throwStop();
      }
    public:
      class Window
      {
	CArray<int,2> values;
      public:
	explicit Window(int a=0) { values.fill(a); }
	Window(int a, int b) 
	{
	  values[0] = a;
	  values[1] = b;
	}
	Window(Player P, int a=0, int b=0) 
	{
	  alpha(P) = a;
	  beta(P) = b;
	}
	int& alpha(Player P) { 
	  return values[playerToIndex(P)]; 
	}
	int& beta(Player P) { 
	  return values[playerToIndex(alt(P))]; 
	}

	int alpha(Player P) const { 
	  return values[playerToIndex(P)]; 
	}
	int beta(Player P) const { 
	  return values[playerToIndex(alt(P))]; 
	}
	bool isConsistent() const { 
	  return eval::notLessThan(BLACK, beta(BLACK), alpha(BLACK));
	}
	bool null() const { return values[0] == values[1]; }
      };

      size_t nodeCount() const { return node_count; }
      void showPV(std::ostream&, int, Move) const;

      template <Player P>
      const MoveLogProb nextMove();
    protected:
      template <Player P> class NextMove;
      friend class NextMove<BLACK>;
      friend class NextMove<WHITE>;
      template <Player P> class NextQMove;
      friend class NextQMove<BLACK>;
      friend class NextQMove<WHITE>;

      /**
       * alphaBetaSearch (move)
       * - makeMove(move)
       * - => alphaBetaSearchAfterMove
       * -- search extension etc.
       * -- => searchAllMoves => alphaBetaSearch (child move)
       * - unmakeMove(move)
       */
      template <Player P>
      int alphaBetaSearch(const MoveLogProb& move, Window window,
			  bool in_pv);
      template <Player P>
      int alphaBetaSearchAfterMove(const MoveLogProb& move, 
				   Window window, bool in_pv);
      template <Player P> int quiesce(Window);
      template <Player P> int quiesceExp(Window);

      template <Player P>
      int searchAllMoves(SimpleHashRecord*, Window w);
      template <Player P>
      int searchAllMoves(Move m, int limit_consumption, 
			 SimpleHashRecord*, Window w);

      /** ǵͤߤɤ */
      template <Player P>
      bool tryCheckmate(SimpleHashRecord *record, bool in_pv, Move& checkmate_move);
      /** 餱ʻˤ˵ͤߤɤ */
      template <Player P>
      bool tryCheckmateAgain(SimpleHashRecord *record, Move& checkmate_move,
			     int node_count,
			     int best_value);

      /** ̵ͤͭǧ */
      template <Player P>
      void testThreatmate(SimpleHashRecord *record, bool in_pv);

      /** alphaͤޤä¾μĴ٤ */
      template <Player P>
      void examineMovesRoot(const MoveLogProbVector&, size_t, Window,
			    MoveLogProb&, int&);

      template <Player P> int quiesceRoot(Window, int depth_left, Move& best_move, DualThreatmateState);
      template <Player P> int quiesce(Window, int depth_left, DualThreatmateState);
      template <Player P>
      bool quiesceWithMove(Move, Window, int, Move&, int&, const DualThreatmateState&);

      void updateCheckmateCount();
      bool tryPass(SimpleHashRecord *record, Player P) const;
      MoveGenerator& makeGenerator();
      static MoveGenerator *alloc();
      static void dealloc(MoveGenerator *);
#ifdef OSL_SMP
    public:
      friend class AlphaBeta2Parallel;
      struct NodeProperty;
      template <Player P> struct SearchJob;
      struct SearchJobData;
      struct Shared;
      friend class Shared;
      friend class SearchJob<BLACK>;
      friend class SearchJob<WHITE>;
    protected:
      template <Player P>
      void examineMovesRootPar(const MoveLogProbVector&, size_t, Window,
			       MoveLogProb&, int&);
      void examineMovesRootPar(int tree_id);
      template <Player P>
      void testMoveRoot(int tree_id, const MoveLogProb&);

      template <Player P>
      bool examineMovesOther(Window& w, MoveLogProb& best_move, int& best_value, 
			     int& tried_moves, int& alpha_update, int& last_alpha_update);
      void examineMovesOther(int tree_id);
      template <Player P>
      bool testMoveOther(int tree_id, const MoveLogProb&, size_t index, bool in_pv);
#endif
    };

    /**
     * AlphaBeta νľ
     */
    class AlphaBeta2
      : public AlphaBeta2Tree
    {
    public:
      AlphaBeta2(const HashEffectState& s, checkmate_t& checker,
		 SimpleHashTable *t, CountRecorder&);
      // do *not* share checkmate_holder to clear table after makemove
      AlphaBeta2(const AlphaBeta2& src);
      ~AlphaBeta2();

      /** 
       * entrance of alpha beta window search.
       *
       * see http://www.logos.t.u-tokyo.ac.jp/~gekisashi/algorithm/re_search.html.
       * rootDepth, curLimit are initialized here.
       */
      template <Player P>
      int alphaBetaSearchRoot(Window window, MoveLogProb& best_move,
			      int limit);
      static const Window fullWindow(Player P)
      {
	return Window(P, winThreshold(alt(P)), winThreshold(P));
      }
      int alphaBetaSearchRoot(Window window, MoveLogProb& best_move, int limit)
      {
	const Player P = state().getTurn();
	if (P == BLACK)
	  return alphaBetaSearchRoot<BLACK>(window, best_move, limit);
	else
	  return alphaBetaSearchRoot<WHITE>(window, best_move, limit);
      }
      int alphaBetaSearchRoot(MoveLogProb& best_move, int limit)
      {
	Window full_window = fullWindow(state().getTurn());
	return alphaBetaSearchRoot(full_window, best_move, limit);
      }
    
      /**
       * entrance of alpha beta iterative search.
       */
      Move computeBestMoveIteratively(int limit,
				      const int step=100, 
				      const int initialLimit=600,
				      int nodeLimit=1600000,
				      const misc::RealTime& timer
				      =misc::RealTime(60));
      bool isReasonableMove(Move move, int pawn_sacrifice=1);
      void setRootIgnoreMoves(const MoveVector *rim) { root_ignore_moves = rim; }

      static void showNodeDepth(std::ostream&);
      static void clearNodeDepth();
    public:
      // only for test
      MoveType currentMoveType() const { return move_type[curDepth()]; }
      const MoveLogProb nextMoveOfTurn() 
      {
	if (state().getTurn() == BLACK)
	  return nextMove<BLACK>();
	else
	  return nextMove<WHITE>();
      }
      void setRoot(int limit);
      void makeMove(Move);
    }; // class AlphaBeta2

  } // namespace search
  using search::AlphaBeta2;
} // namespace osl

#endif /* _ALPHA_BETA2_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
