#ifndef _MTDF_H
#define _MTDF_H

#include "osl/search/searchTable.h"
#include "osl/search/searchBase.h"
#include "osl/search/searchFramework.h"
#include "osl/search/nullWindowSearch.h"
#include "osl/search/searchMoveSorter.h"
#include "osl/search/hasTimer.h"
#include "osl/checkmate/dualCheckmateSearcher.h"
#include "osl/container/moveStack.h"
#include "osl/eval/concept.h"
#include "osl/misc/realTime.h"

namespace osl
{
  namespace search
  {
    class SimpleHashTable;
    class SearchMove;
    class BigramKillerMove;
    class MtdfParShared;
    /**
     * MTDF
     *
     * @param MoveGenerator  CategoryList 
     *   㤨õʤ FWMoveGenerator 
     *   ʣ generator  CategoryList<A,CategoryList<B,C> > ʤɤȥꥹȤǽ
     * @param Eval PieceEval or PiecePairEval
     * @param Probabilities fullwidth ξ FullWidthProbability Ȥ
     * @param Recorder 㤨 NullMtdfRecorder
     *
     * null window õ AlphaBeta Ǥ
     * ˤroot ΡնΥ᥽åɤ
     */
    template<typename Eval, typename MoveGenerator,
	     typename Table=SimpleHashTable,
	     typename Recorder=CountRecorder, 
	     typename Probabilities=RealizationProbability>
    class MTDF 
      : public SearchBase<Eval,Table,Recorder,Probabilities>,
	public HasTimer
    {
      BOOST_CLASS_REQUIRE(Eval, osl::eval, Concept);
      typedef SearchBase<Eval,Table,Recorder,Probabilities> base_t;
    public:
      typedef MTDF<Eval,MoveGenerator,Table,Recorder,Probabilities> mtdf_t;
      typedef NullWindowSearch<Eval,MoveGenerator,Table,Recorder,Probabilities> tester_t;
      typedef typename base_t::eval_t eval_t;
      typedef misc::RealTime RealTime;
      typedef DualCheckmateSearcher<> checkmate_t;
#ifdef OSL_SMP
      boost::shared_ptr<CheckmateSearcherHolder<checkmate_t> > checkmate_holder;
#endif
    private:
      int threshold;
      int lower_bound, upper_bound;
      checkmate_t *checkmate_searcher;
      tester_t memory_tester;
      RealTime timer;
      const MoveVector *root_ignore_moves;
    public:
      explicit MTDF(const HashEffectState& s, 
		    DualCheckmateSearcher<>& c,
		    Table& t, Recorder& r);
      ~MTDF();

      /**
       * threshold νͤ
       */
      void setInitialGuess(int guess);
      /** threshold νͤ */
      void setInitialGuess();
#ifdef OSL_SMP
      void setCheckmateSearcher(checkmate_t *c, const Worker *w);
#endif
      bool isReasonableMove(Move move, int pawn_sacrifice=1);

      // õ
      /** last_move ѤϤ餫 history ˤƤ */
      void setHistory(const MoveStack& history)
      {
	memory_tester.setHistory(history);
      }
      void setRootIgnoreMoves(const MoveVector *rim) { root_ignore_moves = rim; }
      /**
       * ̾ƤФ᥽å
       * @param limit 1400Ǻ14ޤɤ
       */
      const Move computeBestMove(int limit, const RealTime& timer=RealTime(60));
      /**
       * computeBestMove  iterative deepening version
       * @param limit controls search depth
       * @param nodeLimit maximum number of nodes
       * @param step iteration step. 
       */
      const Move computeBestMoveIteratively(int limit,
					    const int step=100, 
					    const int initialLimit=400,
					    int nodeLimit=1600000,
					    const misc::RealTime& timer
					    =misc::RealTime(60));

      const AlarmSwitch alarmSwitch();
      const BigramKillerMove& bigramKillerMove();
      void setBigramKillerMove(const BigramKillerMove& killers);
    private:
      // ȥåץ٥礭ʴؿ
      /** ᥤõ */
      template<Player P>
      const Move computeBestMoveOfPlayer(int limit);
      /** ٤٤Ƥμį᤿ */
      template<Player P>
      const Move retryByLowerF(SimpleHashRecord *record, int limit, 
			       SearchMoveSorter<P>& lower_moves, SearchMoveSorter<P>& higher_moves);
      /** 
       * F ۤʣμǺ
       *
       * ͤι⤤˸¤١夫2ܰʹߤõ
       * 1ɤβ+delta ۤʤȤǧ롣
       * ۤʤ1ܤμ꤬ꡢۤΤʤ¤ľƤľ
       *
       * @param lower_moves F ⤤֤õΤǡߤ threshold ۤǽν
       * @param higher_moves ȤF ۤν
       */
      template<Player P>
      const Move selectBestMoveByHigherF(SimpleHashRecord *record, int limit, 
					 SearchMoveSorter<P>& lower_moves, SearchMoveSorter<P>& higher_moves);
      template <Player P>
      void triageMovesSpeculatively(SearchMoveSorter<P>& higher_moves, int limit,
				    int jump);
    public:
      /**
       * ȥåץ٥  moves μĴ٤ 
       * lower_moves  higher_moves ƥʥ󥹤
       * ޤ threshold Ŭڤѹ
       *
       * @param current_value ߤκ betterThan<P>(threshold, current_value) 
       * @param lower_moves threshold ꤬ʤäȤƤμɤ¸
       * @param higher_moves threshold Τߤɤ¸
       */
      template<Player P>
      void examineMoves(const char *category_name, int& current_value, 
			int limit, const SearchMoveVector& moves, 
			SearchMoveSorter<P>& lower_moves, 
			SearchMoveSorter<P>& higher_moves);
#ifdef OSL_SMP
      template<Player P>
      void examineMovesParBlock(const boost::shared_ptr<MtdfParShared>&,
				int limit, size_t first, size_t last);
      template<Player P>
      void examineMovesPar(int& current_value, 
			   int limit, const SearchMoveVector& moves, 
			   SearchMoveSorter<P>& lower_moves, 
			   SearchMoveSorter<P>& higher_moves);
#endif
      /** 
       * 줿 moves  out ˽
       */
      void gatherMoves(const MoveLogProbVector& moves,
		       SearchMoveSet& out);
    private:
      /**
       * move õmax_value (ǽʤ)
       * @return cut Ǥ뤫
       * @param val  move  val
       * @param max_value max(val,max_value) ˹
       */
      template<Player P>
      bool testMove(const SearchMove& move, int& val, int& max_value, int limit);
    public:
      const DualCheckmateSearcher<> *
      checkmateSearcher() const { return checkmate_searcher; }
      const HistoryTable& historyTable() const { 
	return memory_tester.historyTable();
      }
    }; // class MTDF
  } // namespace search
  using search::MTDF;
} // namespace osl

#endif /* _MTDF_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
