/* quiescenceWindow.tcc
 */
#ifndef _QUIESCENCEWINDOW_TCC
#define _QUIESCENCEWINDOW_TCC
#include "osl/search/quiescenceWindow.h"

namespace osl
{
  namespace search
  {
    struct NullWindowToQWindow
    {
      /**
       * 1ʬ
       */
      static const std::pair<int,int>
      halfPawn(int current, int pawn_value2)
      {
	return std::make_pair(current - pawn_value2/4,
			      current + pawn_value2/4);
      }
      /**
       * root 󤤾root˴󤻤
       */
      static const std::pair<int,int>
      extendToCenter(Player turn, int current, 
		     int pawn_value2, int root_value)
      {
	const int delta = eval::delta(turn);
	const int extend_range = pawn_value2 * 4;
	int alpha = current - pawn_value2/8;
	int beta  = current + pawn_value2/8;

	if (eval::betterThan(turn, current, root_value))
	{
	  alpha = eval::min(turn, root_value+extend_range-delta, alpha);
	}
	else
	{
	  beta = eval::max(turn, root_value-extend_range+delta, beta);
	}
	return std::make_pair(alpha, beta);
      }
    };
  } // namespace search
} // namespace osl

template <class Eval> 
const std::pair<int,int> 
osl::search::QuiescenceWindow<osl::search::NullWindow>::
make(Player turn, NullWindow window, Eval
#ifdef EXPERIMENTAL_WINDOW_SIZE
     eval
#endif
  )
{
  const int pawn_value2 = Eval::captureValue(newPtypeO(alt(turn),PAWN));
  const int current = window.beta();
#ifdef EXPERIMENTAL_WINDOW_SIZE
  const int root_value = eval.value();
  const std::pair<int,int> alpha_beta
    = NullWindowToQWindow::extendToCenter(turn, current, pawn_value2, 
					  root_value);
#else
  const std::pair<int,int> alpha_beta
    = NullWindowToQWindow::halfPawn(current, pawn_value2);
#endif
  const int alpha = eval::max(turn, FixedEval::winThreshold(alt(turn)), 
			      alpha_beta.first);
  const int beta = eval::min(turn, FixedEval::winThreshold(turn), 
			     alpha_beta.second);
  return std::make_pair(alpha, beta);
}

#endif /* _QUIESCENCEWINDOW_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

