/* SearchFramework.h
 */
#ifndef _SEARCH_FRAMEWORK_H
#define _SEARCH_FRAMEWORK_H

#include "osl/search/realizationProbability.h"
#include "osl/search/searchBase.h"
#include "osl/search/searchState.h"
#include "osl/search/searchWindow.h"
#include "osl/search/passCounter.h"
#include "osl/eval/evalTraits.h"
#include "osl/eval/concept.h"

#ifdef OSL_SMP
#  include "osl/search/checkmateSearcherHolder.h"
#  include <boost/shared_ptr.hpp>
#endif

namespace osl
{
  namespace container
  {
    class MoveStack;
    class SearchMoveSet;
    class MoveLogProbVector;
  }
  namespace search
  {
    using namespace container;

    class SimpleHashRecord;
    class SimpleHashTable;
    class SearchMoveVector;
    /**
     * ¸ΨѤ õι.
     *
     * ֤ state, ߤ limit, root Ǥ depth, limit, ġ
     * 
     * null window search  alpha beta search Ѥ
     * ̤ι¤:
     * - searchAllMoves:
     *   - Ƥλؼäɾ򤹤롥
     *   - ƥ examineMoves ƤӽФ
     * - examineMoves: ƻؼ˰Ϳ줿 policy.searchWithMove Ƥ֡
     * - policy ϥإѥ饹ǡΤΥ᥽å fooSearch Ƥ֡
     *   fooSearch ϥåȽ뤫ɤ֤
     * - fooSearch  searchWithMove ͳ ApplyMove::doUndoMove Ԥʤ
     *   ˥إѤͳ fooSearchAfterMove ƤФ롥
     *   fooSearchAfterMove ϺƵŪ searchAllMoves Ƥӡɾ֤ͤ
     */
    template<typename Eval, typename MoveGenerator,
	     typename Table, typename Recorder, 
	     typename Probabilities=RealizationProbability>
    class SearchFramework
      : public SearchBase<Eval,Table,Recorder,Probabilities>,
	public SearchState
    {
      BOOST_CLASS_REQUIRE(Eval, osl::eval, Concept);
      typedef SearchBase<Eval,Table,Recorder,Probabilities> base_t;
    public:
      typedef Eval eval_t;
      typedef MoveGenerator MoveGenerator_t; // ̾ΤĤɤ?
      typedef Probabilities Probabilities_t;
    protected:
      eval_t eval;
    private:
      PassCounter pass_count;
#ifdef OSL_SMP
      typedef CheckmateSearcherHolder<checkmate_t> holder_t;
      boost::shared_ptr<holder_t> checkmate_holder;
#endif
      size_t node_count;
    public:
      explicit SearchFramework(const HashEffectState& s, checkmate_t& checker,
			       Table *t, Recorder& r);
      SearchFramework(const SearchFramework&);
      ~SearchFramework();

      bool abort(Move) const;
      size_t nodeCount() const { return node_count; }
    protected:
      /**
       * move ؤɾͤ
       *
       * move Ф doUndoMove Ԥʤ Helper Υɤ¹Ԥ롣
       * @move ؼ. 
       *   CAVEAT: &move åˤ󲽤Ȥ򵯤
       *   SimpleHashRecord ˤΤȤ褦ˤɬפ롣
       * @param P 
       * @param Helper ˼¹Ԥ륳
       * @param baseVal ɾͤ򹹿Ȥߤʤƺõ򤹤
       *   (null window search ξ)
       * @return ɾ
       */
      template<Player P, class Helper>
      int searchWithMove(const SearchMove& move, Helper& helper);
    public:
      const eval_t& getEval() const { return eval; }
      template<Player P, class Policy>
      int quiescenceValue(const Policy&);
      template<Player P>
      int quiescenceValue(int alpha, int beta, int depth=8 /* QSearchTraits::MaxDepth */);
      /**
       * move Żõδʤؼ꤫ɤ򸡾.
       * @param pawn_sacrifice »ޤǤϵ
       */
      bool isReasonableMove(Move move, int pawn_sacrifice);
      // private:
      /**
       * (private)ؼäɾ
       * 
       * friend Ĺˤʤ뤿publicˤƤ뤬̤֤˰¸
       * ¼private
       * @param maxval P μǤɾ
       * @param P 
       * @param Policy examineMoves ǤɤΥ᥽åɤƤ֤ݲΡ
       * @param record ߤζ̤record (0 ʤɬפ˱allocate)
       * @param best_move  ȿ§λʤɤǤʤ
       * @return ɾ
       */
      template<Player P, class Policy>
      int searchAllMoves(SimpleHashRecord *& record,
			 const Policy&, SearchMove& best_move);

      /**
       * (private) moves ơ̤ bestmove  max_value .
       *
       * @param moves β¿ξĤΥƥ꤬ä
       * @param tried ˤμɤɤΤ
       * @param helper ƻؼ˼¹Ԥ륳
       *   null window search ξ nullWindowSearch ƤФ
       * @param isKingEscape 򤱤
       * @param P turn
       * @return Υƥɬפ뤫
       */
      template<Player P, class Helper, bool isKingEscape>
      bool examineMoves(const char *category_name, const MoveLogProbVector& moves,
			const Helper&, SearchMove& best_move, 
			int& max_value, SearchMoveSet& tried);
      /**
       * (private) moves ơ̤ bestmove  max_value .
       * recordˤؼꡥ
       */
      template<Player P, class Helper>
      bool examineMoves(const char *category_name, const SearchMoveVector& moves,
			const Helper&, SearchMove& best_move, int& max_value);
      template<Player P, class Helper>
      bool examineOneMove(const SearchMove& move,
			  const Helper&, SearchMove& best_move, int& max_value);
#ifdef OSL_SMP
      /**
       * moves indexʹߤ
       * @return try_next.
       */
      template<Player P, class Helper>
      bool examineMovesPar(boost::shared_ptr<SearchMoveVector> moves,
			   size_t index,
			   const Helper&, SearchMove& best_move, int& max_value);
      void shareCheckmateHolder(boost::shared_ptr<holder_t>&);
#endif
    }; // class SearchFramework
  } // namespace search
} // namespace osl

#endif /* _SEARCH_FRAMEWORK_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
