/* IterativeLinearSolver.h
 */

#ifndef _STAT_ITERATIVELINEARSOLVER_H
#define _STAT_ITERATIVELINEARSOLVER_H

#include <cstddef>

namespace std
{
  template<class T> class valarray;
} // namespace std

namespace osl
{
  namespace stat
  {
    class Preconditioner;
    class MatrixMultiplier;

    /**
     * IterativeLinearSolver implements variants of conjugate gradient methods
     */
    class IterativeLinearSolver
    {
    public:
      typedef std::valarray<double> valarray_t;
    private:
      const MatrixMultiplier *m_A;
      const Preconditioner *m_M;
      size_t m_dim;
      size_t m_max_iteration;
      double m_tolerance;
    public:
      // CREATORS
      IterativeLinearSolver(const MatrixMultiplier& A, const Preconditioner *M,
			    size_t max_iteration=0, // default: dimensition of A
			    double tolerance=0.001);
      ~IterativeLinearSolver();
      // MANIPULATORS
      /** 
       * matrix A must be 'positive definite': symmetric and full rank
       * @return non-zero in case of error (1: not-converged, 2: preconditioner error or others)
       */
      int solve_by_CG(const valarray_t& b, valarray_t& x,
		      int *iterated=0, double *tol_achieved=0, bool verbose=true);
      int solve_by_BiCGSTAB(const valarray_t& b, valarray_t& x,
			    int *iterated=0, double *tol_achieved=0, bool verbose=true);
      // ACCESSORS
    };

  } // namespace stat
} // namespace osl

#endif /* _STAT_ITERATIVELINEARSOLVER_H */

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
