/* PawnMaskState.h
 */
#ifndef _PAWN_MASK_STATE_H
#define _PAWN_MASK_STATE_H

#include "osl/container/bitXmask.h"
#include "osl/misc/carray.h"
#include "osl/apply_move/doUndoMoveLockForward.h"
#include "osl/apply_move/applyDoUndoXMove.h"
#include "osl/state/simpleState.h"
#include "osl/direction.h"
#include "osl/ptype.h"
#include "osl/piece.h"
#include "osl/move.h"
#include "osl/player.h"
#include "osl/handicap.h"

#include <iosfwd>

namespace osl
{
namespace state
{
  class PawnMaskState;
  std::ostream& operator<<(std::ostream& os,const PawnMaskState& state);
  bool operator==(const PawnMaskState& st1,const PawnMaskState& st2);


  /**
   * PawnMask State
   */
  class PawnMaskState : public SimpleState
  {
  public:
    /** Ѿ饹 ĤФ˻Ѥ */
    typedef PawnMaskState pawn_mask_state_t;
    typedef SimpleState simple_state_t;
    typedef SimpleState effect_state_t;
    static const bool hasPawnMask=true;
  private:
    friend std::ostream& operator<<(std::ostream& os,const PawnMaskState& state);
    friend bool operator==(const PawnMaskState& st1,const PawnMaskState& st2);
    typedef PawnMaskState state_t;
    CArray<BitXmask,2> pawnMask;
    
    friend class apply_move::ApplyDoUndoSimpleMove<BLACK,PawnMaskState>;
    friend class apply_move::ApplyDoUndoCaptureMove<BLACK,PawnMaskState>;
    friend class apply_move::ApplyDoUndoDropMove<BLACK,PawnMaskState>;
    friend class apply_move::ApplyDoUndoSimpleMove<WHITE,PawnMaskState>;
    friend class apply_move::ApplyDoUndoCaptureMove<WHITE,PawnMaskState>;
    friend class apply_move::ApplyDoUndoDropMove<WHITE,PawnMaskState>;
    friend class apply_move::DoUndoMoveLockSimple<PawnMaskState>;
    friend class apply_move::DoUndoMoveLockCapture<PawnMaskState>;
    friend class apply_move::DoUndoMoveLockDrop<PawnMaskState>;
  public:
    // ˴ؤ
    explicit PawnMaskState();
    explicit PawnMaskState(SimpleState const& st);
    explicit PawnMaskState(Handicap h);
    // publicѾˤϡvirtual destructor롥
    virtual ~PawnMaskState();
    
  private:
    /** ̤ξ֤˽ */
    void init();
  public:
    /** ϥǥ˱֤˽ */
    void init(Handicap h);
    /** pawnMaskƷ׻ */
    void initPawnMask();

    bool isPawnMaskSet(Player player, int x) const
    {
      return pawnMask[playerToIndex(player)].isSet(x);
    }

    template<Player P>
    bool isPawnMaskSet(int x)const {return isPawnMaskSet(P,x); }

    void clearPawn(Player pl,Position pos){
      pawnMask[playerToIndex(pl)].clear(pos);
    }
    void setPawn(Player pl,Position pos){
      pawnMask[playerToIndex(pl)].set(pos);
    }

    void setPiece(Player player,Position pos,Ptype ptype);
    void setPieceAll(Player player);

    // check
    bool isConsistent(bool showError=true) const;
    // move methods
    void doSimpleMove(Position from, Position to, int promoteMask);
    void doDropMove(Position to,Ptype ptype);
    void doCaptureMove(Position from, Position to, Piece target,int promoteMask);
  };


} // namespace state
  using state::PawnMaskState;

  namespace apply_move
  {
  template<Player P>
  struct ApplyDoUndoSimpleMove<P,PawnMaskState>
  {
    static void prologue(PawnMaskState& s, 
			 Position from, Position to, int promoteMask,
			 Piece& oldPiece, int& num)
    {
      oldPiece=s.getPieceOnBoard(from);
      Piece newPiece=oldPiece.promoteWithMask(promoteMask);
      newPiece+=(to-from);
      num=oldPiece.number();
      s.setPieceOf(num,newPiece);
      s.setBoard(to,newPiece);
      s.setBoard(from,Piece::EMPTY());
    }
    static void epilogue(PawnMaskState& s, Position from, Position to,
			 Piece oldPiece, int num)
    {
      // ʬȤθ򳰤
      s.setPieceOf(num,oldPiece);
      s.setBoard(from,oldPiece);
      s.setBoard(to,Piece::EMPTY());
    }
    template <typename F>
    static void doUndoSimpleMove(PawnMaskState& s, 
				 Position from, Position to, int promoteMask,F& func);
  };
  
  template<Player P>
  template <typename F>
  void ApplyDoUndoSimpleMove<P,PawnMaskState>::
  doUndoSimpleMove(PawnMaskState& s, 
		   Position from, Position to, int promoteMask,F& func)
  {
    Piece oldPiece;
    int num;
    prologue(s, from, to, promoteMask, oldPiece, num);
    if (promoteMask!=0 && num < PtypeTraits<PAWN>::indexLimit)
    {
      s.clearPawn(P,from);
      s.changeTurn();
      func(to);
      s.changeTurn();
      s.setPawn(P,from);
    }
    else{
      s.changeTurn();
      func(to);
      s.changeTurn();
    }
    epilogue(s, from, to, oldPiece, num);
  }

  template<Player P>
  struct ApplyDoUndoDropMove<P,PawnMaskState>
  {
    typedef PawnMaskState state_t;

    static void prologue(PawnMaskState& s, Ptype ptype, Position to,
			 Piece& oldPiece, int& num, int& numIndex, int& numLow)
    {
      numIndex=Ptype_Table.getIndex(ptype);
      const mask_t ownMochigoma=
	s.standMask(P).getMask(numIndex) & Ptype_Table.getMaskLow(ptype);
      assert(ownMochigoma.any() || (s.dump(), 0));
      numLow=ownMochigoma.bsf();
      num=numLow|(numIndex<<5);
      oldPiece=s.getPieceOf(num);
      Piece p=oldPiece;
      p+=to-Position::STAND();

      s.setPieceOf(num,p);
      s.setBoard(to,p);
      s.standMask(P).xorMask(numIndex,PieceMask::numToMask(numLow));
    }
    static void epilogue(PawnMaskState& s, Position to,
			 Piece oldPiece, int num,
			 int numIndex, int numLow)
    {
      s.standMask(P).xorMask(numIndex,PieceMask::numToMask(numLow));
      s.setBoard(to,Piece::EMPTY());
      s.setPieceOf(num,oldPiece);
    }
    template <typename F>
    static void doUndoDropMove(PawnMaskState& s, 
			       Position to, Ptype ptype,F& func);
  };
  
  
  template<Player P>
  template <typename F>
  void ApplyDoUndoDropMove<P,PawnMaskState>::
  doUndoDropMove(PawnMaskState& s, 
		 Position to, Ptype ptype,F& func)
  {
    Piece oldPiece;
    int num, numIndex, numLow;
    prologue(s, ptype, to, oldPiece, num, numIndex, numLow);

    if (ptype==PAWN)
    {
      s.setPawn(P,to);
      s.changeTurn();
      func(to);
      s.changeTurn();
      s.clearPawn(P,to);
    }
    else
    {
      s.changeTurn();
      func(to);
      s.changeTurn();
    }
    epilogue(s, to, oldPiece, num, numIndex, numLow);
  }

  
  template<Player P>
  struct ApplyDoUndoCaptureMove<P,PawnMaskState>
  {
    typedef PawnMaskState state_t;
    template <typename F>
    static void doUndoCaptureMove(state_t& s,
				  Position from,Position to, Piece p1, int promoteMask,F& func);

    static
    void prologue(PawnMaskState& s, 
		  Position from, Position to, Piece target, int promoteMask,
		  Ptype& capturePtype, Piece& oldPiece, int& num0, 
		  int& num1, int& num1Index, mask_t& num1Mask)
    {
      capturePtype=target.ptype();
      num1=target.number();
      num1Index=PieceMask::numToIndex(num1);
      num1Mask=PieceMask::numToMask(num1);

      s.standMask(P).xorMask(num1Index,num1Mask);
      oldPiece=s.getPieceOnBoard(from);
      Piece newPiece=oldPiece.promoteWithMask(promoteMask);
      newPiece+=(to-from);
      num0=oldPiece.number();
      s.setPieceOf(num0,newPiece);
      s.setPieceOf(num1,target.captured());
      s.setBoard(to,newPiece);
      s.setBoard(from,Piece::EMPTY());
    }
    static 
    void epilogue(PawnMaskState& s, Position from, Position to, Piece target, 
		  Piece oldPiece, int num0, 
		  int num1, int num1Index, mask_t num1Mask)
    {
      s.standMask(P).xorMask(num1Index,num1Mask);
      s.setPieceOf(num0,oldPiece);
      s.setPieceOf(num1,target);
      s.setBoard(from,oldPiece);
      s.setBoard(to,target);
    }
  };
  
  template<Player P>
  template <typename F>
  void ApplyDoUndoCaptureMove<P,PawnMaskState>::
  doUndoCaptureMove(state_t& s,
		    Position from,Position to, Piece target, int promoteMask,F& func)
  {
    Piece oldPiece;
    int num0, num1;
    int num1Index;
    mask_t num1Mask;
    Ptype capturePtype;
    prologue(s, from, to, target, promoteMask,
	     capturePtype, oldPiece, num0, num1, num1Index, num1Mask);

    s.changeTurn();
    if (capturePtype==PAWN){
      s.clearPawn(PlayerTraits<P>::opponent,to);
      if (promoteMask!=0 && num0<PtypeTraits<PAWN>::indexLimit){
	s.clearPawn(P,from);
	func(to);
	s.setPawn(P,from);
      }
      else{
	func(to);
      }
      s.setPawn(PlayerTraits<P>::opponent,to);
    }
    else if (promoteMask!=0 && num0<PtypeTraits<PAWN>::indexLimit){
      s.clearPawn(P,from);
      func(to);
      s.setPawn(P,from);
    }
    else{
      func(to);
    }
    s.changeTurn();

    epilogue(s, from, to, target, oldPiece, num0, num1, num1Index, num1Mask);
  }

  template<> class DoUndoMoveLockSimple<PawnMaskState>;
  template<> class DoUndoMoveLockDrop<PawnMaskState>;
  template<> class DoUndoMoveLockCapture<PawnMaskState>;

  } // namespace apply_move
} // namespace osl

#endif /* _PAWN_MASK_STATE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
