/* simpleState.tcc
 */
#ifndef _SIMPLE_STATE_TCC
#define _SIMPLE_STATE_TCC

#include "osl/state/simpleState.h"

namespace osl
{
  namespace simple_state_detail
  {
    template <bool checkDone> struct CheckDoneIf;
    template <> struct CheckDoneIf<true>
    {
      template <class Action>
      static bool done(Action& a) { return a.done(); }
    };
    template <> struct CheckDoneIf<false>
    {
      template <class Action>
      static bool done(Action&) { return false; }
    };
  } // simple_state_detail namespace
}

template <class Action>
void osl::SimpleState::forEachEffectOfPiece(Piece piece,Action & action) const 
{
  Position piecePosition = piece.position();
  switch (piece.ptypeO()) {
  case NEW_PTYPEO(WHITE,PAWN): forEachEffectOfPiece<WHITE,PAWN,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,LANCE): forEachEffectOfPiece<WHITE,LANCE,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,KNIGHT): forEachEffectOfPiece<WHITE,KNIGHT,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,SILVER): forEachEffectOfPiece<WHITE,SILVER,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,PPAWN): forEachEffectOfPiece<WHITE,PPAWN,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,PLANCE): forEachEffectOfPiece<WHITE,PLANCE,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,PKNIGHT): forEachEffectOfPiece<WHITE,PKNIGHT,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,PSILVER): forEachEffectOfPiece<WHITE,PSILVER,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,GOLD): forEachEffectOfPiece<WHITE,GOLD,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,BISHOP): forEachEffectOfPiece<WHITE,BISHOP,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,PBISHOP): forEachEffectOfPiece<WHITE,PBISHOP,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,ROOK): forEachEffectOfPiece<WHITE,ROOK,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,PROOK): forEachEffectOfPiece<WHITE,PROOK,Action>(piecePosition,action); break;
  case NEW_PTYPEO(WHITE,KING): forEachEffectOfPiece<WHITE,KING,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PAWN): forEachEffectOfPiece<BLACK,PAWN,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,LANCE): forEachEffectOfPiece<BLACK,LANCE,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,KNIGHT): forEachEffectOfPiece<BLACK,KNIGHT,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,SILVER): forEachEffectOfPiece<BLACK,SILVER,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PPAWN): forEachEffectOfPiece<BLACK,PPAWN,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PLANCE): forEachEffectOfPiece<BLACK,PLANCE,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PKNIGHT): forEachEffectOfPiece<BLACK,PKNIGHT,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PSILVER): forEachEffectOfPiece<BLACK,PSILVER,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,GOLD): forEachEffectOfPiece<BLACK,GOLD,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,BISHOP): forEachEffectOfPiece<BLACK,BISHOP,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PBISHOP): forEachEffectOfPiece<BLACK,PBISHOP,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,ROOK): forEachEffectOfPiece<BLACK,ROOK,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,PROOK): forEachEffectOfPiece<BLACK,PROOK,Action>(piecePosition,action); break;
  case NEW_PTYPEO(BLACK,KING): forEachEffectOfPiece<BLACK,KING,Action>(piecePosition,action); break;
  default: assert(0);
  }
}

template <osl::Player P, osl::Ptype Type, class Action>
void osl::SimpleState::forEachEffectPtype(Position pos,Action & action) const
{
  forEachEffectPtypeDirection<P,Type,UL,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,UR,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,L,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,R,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,DL,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,D,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,DR,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,UUL,Action>(pos,action);
  forEachEffectPtypeDirection<P,Type,UUR,Action>(pos,action);
  // U
  if (Type!=LANCE && Type!=ROOK) {
    forEachEffectPtypeDirection<P,Type,U,Action>(pos,action);
  }
  else{
    Position pos1=pos-DirectionPlayerTraits<U,P>::offset();
    Piece p=getPieceAt(pos1);
    if (p.isEmpty()) {
      pos1-=DirectionPlayerTraits<U,P>::offset();
      while((p=getPieceAt(pos1)).isEmpty()) {
	pos1-=DirectionPlayerTraits<U,P>::offset();
      }
      PtypeO ptypeo=p.ptypeO();
      int moveMask=Ptype_Table.getMoveMask(getPtype(ptypeo))&
	DirectionTraits<LONG_U>::mask;
      int playerMask= (~(static_cast<signed int>(ptypeo)>>31)) ^
	PlayerTraits<P>::mask;
      if ((moveMask&playerMask)!=0 && unpromote(getPtype(ptypeo))==Type ) {
	action.template doAction<P>(p,pos);
      }
    }
    else{
      PtypeO ptypeo=p.ptypeO();
      int moveMask=Ptype_Table.getMoveMask(getPtype(ptypeo))&
	(DirectionTraits<U>::mask|DirectionTraits<LONG_U>::mask);
      int playerMask= (~(static_cast<signed int>(ptypeo)>>31)) ^
	PlayerTraits<P>::mask;
      if ((moveMask&playerMask)!=0 && unpromote(getPtype(ptypeo))==Type) {
	action.template doAction<P>(p,pos);
      }
    }
  }
  if (Type==BISHOP) {
    for (int num=36;num<=37;num++) {
      Piece p=getPieceOf(num);
      if (p.isOnBoardByOwner<P>()) {
	Position pos1=p.position();
	EffectContent effect=Ptype_Table.getEffectNotLongU(p.ptypeO(),pos,pos1);
	if (effect.hasEffect()) {
	  Offset offset=effect.offset();
	  Position pos2=pos1+offset;
	  do{
	    if (! getPieceAt(pos2).isEmpty()) goto retryBIshop;
	    pos2+=offset;
	  } while(pos2!=pos);
	  action.template doAction<P>(p,pos);
	}
      retryBIshop:;
      }
    }
  }
  else if (Type==ROOK) {
    for (int num=38;num<=39;num++) {
      Piece p=getPieceOf(num);
      if (p.isOnBoardByOwner<P>()) {
	Position pos1=p.position();
	EffectContent effect=Ptype_Table.getEffectNotLongU(p.ptypeO(),pos,pos1);
	if (effect.hasEffect()) {
	  Offset offset=effect.offset();
	  Position pos2=pos1+offset;
	  do{
	    if (! getPieceAt(pos2).isEmpty()) goto retryRook;
	    pos2+=offset;
	  } while(pos2!=pos);
	  action.template doAction<P>(p,pos);
	}
      retryRook:;
      }
    }
  }
}


template <osl::Player P, class Action,bool checkDone, class PositionCheck>
inline
void osl::SimpleState::
forEachEffectWithCheck(Position pos,Action & action,PositionCheck const& positionCheck) const 
{
  for (int i=0;i<U;i++) {
    Direction dir=static_cast<Direction>(i);
    Position pos1=pos-Board_Table.template getOffset<P>(dir);
    Piece p=getPieceAt(pos1);
    PtypeO ptypeo=p.ptypeO();
    if (Ptype_Table.getShortMoveMask(P,ptypeo,dir)!=0) {
      action.template doAction<P>(p,pos);
      if (simple_state_detail::CheckDoneIf<checkDone>::done(action)) return;
    }
  }
  for (int i=U+1;i<=SHORT_DIRECTION_MAX;i++) {
    Direction dir=static_cast<Direction>(i);
    Position pos1=pos-Board_Table.template getOffset<P>(dir);
    Piece p=getPieceAt(pos1);
    PtypeO ptypeo=p.ptypeO();
    if (Ptype_Table.getShortMoveMask(P,ptypeo,dir)!=0) {
      action.template doAction<P>(p,pos);
      if (simple_state_detail::CheckDoneIf<checkDone>::done(action)) return;
    }
  }
  // U
  {
    Position pos1=pos-DirectionPlayerTraits<U,P>::offset();
    Piece p=getPieceAt(pos1);
    if (p.isEmpty() || positionCheck(pos1)) {
      pos1-=DirectionPlayerTraits<U,P>::offset();
      while((p=getPieceAt(pos1), p.isEmpty()) || positionCheck(pos1)) {
	pos1-=DirectionPlayerTraits<U,P>::offset();
      }
      PtypeO ptypeo=p.ptypeO();
      int moveMask=Ptype_Table.getMoveMask(getPtype(ptypeo))&
	DirectionTraits<LONG_U>::mask;
      int playerMask= (~(static_cast<signed int>(ptypeo)>>31)) ^
	PlayerTraits<P>::mask;
      if ((moveMask&playerMask)!=0) {
	action.template doAction<P>(p,pos);
	if (simple_state_detail::CheckDoneIf<checkDone>::done(action)) return;
      }
    }
    else{
      PtypeO ptypeo=p.ptypeO();
      int moveMask=Ptype_Table.getMoveMask(getPtype(ptypeo))&
	(DirectionTraits<U>::mask|DirectionTraits<LONG_U>::mask);
      int playerMask= (~(static_cast<signed int>(ptypeo)>>31)) ^
	PlayerTraits<P>::mask;
      if ((moveMask&playerMask)!=0) {
	action.template doAction<P>(p,pos);
	if (simple_state_detail::CheckDoneIf<checkDone>::done(action)) return;
      }
    }
  }
  for (int num=36;num<=39;num++) {
    Piece p=getPieceOf(num);
    if (p.isOnBoardByOwner<P>()) {
      Position pos1=p.position();
      EffectContent effect=Ptype_Table.getEffectNotLongU(p.ptypeO(),pos1,pos);
      if (effect.hasEffect()) {
	Offset offset=effect.offset();
	Position pos2=pos1+offset;
	do{
	  if (! getPieceAt(pos2).isEmpty() && !positionCheck(pos2)) goto retry;
	  pos2+=offset;
	} while(pos2!=pos);
	action.template doAction<P>(p,pos);
	if (simple_state_detail::CheckDoneIf<checkDone>::done(action)) return;
      }
    retry:;
    }
  }
}

template <osl::Player P, osl::Ptype Type, class Action>
void osl::SimpleState::
forEachEffectOfPiece(Position piecePosition,Action & action) const 
{
  forEachEffectOfPieceDir<P,Type,Action,UL>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,U>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,UR>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,L>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,R>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,DL>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,D>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,DR>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,UUL>(piecePosition,action);
  forEachEffectOfPieceDir<P,Type,Action,UUR>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_UL>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_U>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_UR>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_L>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_R>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_DL>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_D>(piecePosition,action);
  forEachEffectOfPieceLongDir<P,Type,Action,LONG_DR>(piecePosition,action);
}

#endif /* _SIMPLE_STATE_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
