/* brinkmateState.cc
 */
#include "osl/brinkmate/brinkmateState.h"

osl::brinkmate::
BrinkmateState::BrinkmateState()
  : m_state(SimpleState(HIRATE)), checkmator(0)
{
  defense_check.push(0);
}

void osl::brinkmate::
BrinkmateState::init(const HashEffectState& s)
{
  m_state = s;
  path = PathEncoding(s.getTurn());
}

void osl::brinkmate::
BrinkmateState::init(checkmate_t& c)
{
  checkmator = &c;
}

void osl::brinkmate::
BrinkmateState::push(Move todo, bool is_defense_check)
{
  move_stack.push(m_state, todo);
  moves.push(todo);
  path.pushMove(todo);
  defense_check.push(defense_check.top()+is_defense_check);
}

void osl::brinkmate::
BrinkmateState::pop()
{
  const Move last_move = moves.lastMove();
  move_stack.pop();
  moves.pop();
  path.popMove(last_move);
  defense_check.pop();
}

bool osl::brinkmate::
BrinkmateState::isEffectiveDefense(Move move)
{
  using namespace checkmate;
  const Player attacker = alt(move.player());
  const checkmate_t::table_t& table = checkmator->getTable(attacker);
  const CheckHashRecord *record 
    = table.find(m_state.getHash().newHashWithMove(Move::PASS(alt(attacker))));

  push(move, false);
  bool checkmate = false;
  // oracle ǵͤफ
  checkmate = isWinningState(0);

  // ̤8˵Ѳoracle ŬѤʤΤǼư
  if ((! checkmate) && (move.ptype() != KING)
      && record && record->proofDisproof().isCheckmateSuccess())
  {
    if (attacker == BLACK)
    {
      Move best_move;
      checkmate::ProofOracleAttack<BLACK> oracle(record);
      OracleProverLight<BLACK> prover;
      checkmate = prover.proofWin(m_state, oracle, best_move);
    }
    else
    {
      Move best_move;
      ProofOracleAttack<WHITE> oracle(record);
      OracleProverLight<WHITE> prover;
      checkmate = prover.proofWin(m_state, oracle, best_move);
    }
  }

  // ͤߤϤ뤫
  if (! checkmate)
  {
    checkmate = isWinningState(1);
  }
  pop();

  return ! checkmate;
}

const osl::checkmate::ProofDisproof osl::brinkmate::
BrinkmateState::testThreatmateState(int node)
{
  if ((depth() / 2) == 0)
    node *= 20;
  else if ((depth() / 2) == 1)
    node *= 2;

  m_state.changeTurn();

  const HashKey& threatmate_hash = m_state.getHash();
  const PathEncoding threatmate_path(m_state.getTurn());

  AttackOracleAges age;
  Move threatmate_move;
  const bool threatmate0
    = checkmator->isWinningState(0, m_state, threatmate_hash, 
				threatmate_path, threatmate_move, age);
  const bool threatmate 
    = (threatmate0 
       || (checkmator->isWinningState(node, m_state, threatmate_hash, 
				     threatmate_path, threatmate_move, age)));
  const checkmate_t::table_t& table = checkmator->getTable(m_state.getTurn());
  const CheckHashRecord *record = table.find(m_state.getHash());
  assert(threatmate || record);
  
  m_state.changeTurn();

  if (threatmate)
    return ProofDisproof::Checkmate();
  return record->proofDisproof();
}

const osl::checkmate::ProofDisproof osl::brinkmate::
BrinkmateState::testWinningState(int node)
{
  if ((depth() / 2) == 0)
    node *= 8;
  else if ((depth() / 2) == 1)
    node *= 2;
  
  const HashKey& hash = m_state.getHash();
  
  AttackOracleAges age;
  Move checkmate_move;
  const bool checkmate0
    = checkmator->isWinningState(0, m_state, hash, path, checkmate_move, age);
  const bool checkmate 
    = (checkmate0 
       || (checkmator->isWinningState(node, m_state, hash, 
				     path, checkmate_move, age)));

  const checkmate_t::table_t& table = checkmator->getTable(m_state.getTurn());
  const CheckHashRecord *record = table.find(m_state.getHash());
  assert(checkmate || (node == 0) || record);
  if (checkmate)
    return ProofDisproof::Checkmate();
  if (node == 0)
    return ProofDisproof::Unknown();
  return record->proofDisproof();
}

bool osl::brinkmate::
BrinkmateState::isThreatmateState(int node)
{
  return testThreatmateState(node).isCheckmateSuccess();
}

bool osl::brinkmate::
BrinkmateState::isWinningState(int node)
{
  return testWinningState(node).isCheckmateSuccess();
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
