/* facade.c
 */
#include "osl/c/facade.h"
#include "osl/checkmate/dualCheckmateSearcher.h"
#include "osl/game_playing/alphaBetaPlayer.h"
#include "osl/game_playing/gameState.h"
#include "osl/search/simpleHashTable.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/record/csaString.h"
#include "osl/record/csa.h"
#include <string>

extern "C" 
int checkmate_attack(const char *state_str, int limit, char *move)
{
  osl::DualCheckmateSearcher<> checkmate;
  osl::Move checkmate_move;
  osl::NumEffectState state(osl::CsaString(state_str).getInitialState());
  osl::HashKey key = osl::HashKey::calcHash(state);
  osl::PathEncoding pe(state.getTurn());
  const bool win = checkmate.isWinningState(limit, state, key, pe,
					    checkmate_move);
  if (win) {
    const std::string checkmate_move_str = 
      osl::record::csa::show(checkmate_move);
    sprintf(move, "%s", checkmate_move_str.c_str());
  }
  return win;
}

extern "C" 
int checkmate_escape(const char *state_str, int limit)
{
  osl::DualCheckmateSearcher<> checkmate;
  osl::Move checkmate_move;
  osl::NumEffectState state(osl::CsaString(state_str).getInitialState());
  osl::HashKey key = osl::HashKey::calcHash(state);
  osl::PathEncoding pe(state.getTurn());
  const bool escape = checkmate.isLosingState(limit, state, key, pe);
  return escape;
}
  
extern "C" 
int search(const char *state_str, int seconds, int verbose, char *move)
{
  osl::game_playing::AlphaBeta2Player player;
  player.setNextIterationCoefficient(1.7);
  player.setTableLimit(3000000, 0);
  player.setVerbose(verbose);
  player.setDepthLimit(2000, 400, 200);

  osl::game_playing::GameState state(osl::CsaString(state_str).getInitialState());
  osl::Move best_move = player.searchWithSecondsForThisMove(state, seconds).move;

  const std::string best_move_str = osl::record::csa::show(best_move);
  sprintf(move, "%s", best_move_str.c_str());

  const osl::SimpleHashTable *table = player.table();
  const osl::HashKey& key = state.state().getHash();

  const osl::SimpleHashRecord *record = table->find(key);
  int value = record ? record->lowerBound() : 0;
  return value;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
