/* breakThreatmate.cc
 */
#include "osl/category/breakThreatmate.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/move_generator/addEffect_.h"
#include "osl/move_generator/capture_.h"
#include "osl/move_generator/pieceOnBoard.h"
#include "osl/move_action/store.h"
#include "osl/boardTable.h"
#include "osl/ptypeTable.h"

void osl::category::
BreakThreatmate::generateBreakDrop(int limit,
				   const NumEffectState& state, Position to,
				   int default_prob,
				   MoveLogProbVector& out)
{
  const Player Turn = state.getTurn();
  default_prob = std::max(default_prob, 150);
  if (state.hasPieceOnStand(Turn, PAWN))
  {
    if (! state.isPawnMaskSet(Turn, to.x())
	&& Ptype_Table.canDropTo(Turn, PAWN, to)
	&& limit >= default_prob-100)
      out.push_back(MoveLogProb(Move(to, PAWN, Turn), default_prob-100));
  }
  if (state.hasPieceOnStand(Turn, LANCE)
      && Ptype_Table.canDropTo(Turn, LANCE, to)
      && limit >= default_prob-50)
    out.push_back(MoveLogProb(Move(to, LANCE, Turn), default_prob-50));
  if (default_prob > limit)
    return;

  if (state.hasPieceOnStand(Turn, KNIGHT)
      && Ptype_Table.canDropTo(Turn, KNIGHT, to))
    out.push_back(MoveLogProb(Move(to, KNIGHT, Turn), default_prob));
  if (state.hasPieceOnStand(Turn, SILVER))
    out.push_back(MoveLogProb(Move(to, SILVER, Turn), default_prob));
  if (state.hasPieceOnStand(Turn, GOLD))
    out.push_back(MoveLogProb(Move(to, GOLD, Turn), default_prob));
  if (state.hasPieceOnStand(Turn, BISHOP))
    out.push_back(MoveLogProb(Move(to, BISHOP, Turn), default_prob));
  if (state.hasPieceOnStand(Turn, ROOK))
    out.push_back(MoveLogProb(Move(to, ROOK, Turn), default_prob));
}

// yoshiki's suggestion  50,  100, Τ 400 
void osl::category::BreakThreatmate::
generateBreak(int limit, const NumEffectState& state, Move threatmate_move,
	      MoveLogProbVector& out)
{
  assert(threatmate_move.isNormal());
  const Player Turn = state.getTurn();
  
  MoveVector all_moves;
  move_action::Store store(all_moves);
  assert(threatmate_move.isNormal());
  const Position target = threatmate_move.to();

  move_generator::GenerateAddEffect<false>::generate<NumEffectState>
    (Turn, state, target, store);
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
  {
    const Ptype ptype = p->ptype();
    if (ptype == KING)
      continue;			// KING_WALK will generate
    if (! p->isDrop())
    {
      if (isMajor(ptype)
	  && state.hasEffectByPiece(state.getPieceOnBoard(p->from()), target))
	continue;			// already have effect
    }
    const Position to = p->to();
    const int me = state.countEffect(Turn, to) + (p->isDrop() ? 1 : 0);
    const int op = state.countEffect(alt(Turn), to);
    int prob = (p->isDrop() ? 100 : 100); // delay drop
    if (p->capturePtype() != PTYPE_EMPTY)
    {
      prob -= 50;
    }
    else
    { 
      if (isMajor(ptype)
	  || ((ptype == GOLD || ptype == SILVER)
	      && (to.x() == 1 || to.x() == 9)))
      {
	prob += 50;
      }
      if (! ((me >= 2) || (op == 0)))
      {
	prob += 300;
      }
    }
    if (prob <= limit)
      out.push_back(MoveLogProb(*p, prob));
  }
  if (threatmate_move.isDrop())
  {
    const int drop_prob = (state.hasEffectBy(alt(Turn), target) ? 400 : 100);
    generateBreakDrop(limit, state, target, drop_prob, out);
  }
  else
  {
    // not drop
    const Position from = threatmate_move.from();
    const Offset offset
      = Board_Table.getShortOffsetNotKnight(Offset32(target, from));
    if (! offset.zero())
    {
      for (Position to = from + offset; to != target; to += offset) 
      {
	assert(to.isOnBoard());
	assert(state.getPieceOnBoard(to) == Piece::EMPTY());
	const int drop_prob = (state.hasEffectBy(Turn, to) ? 100 : 400);
	generateBreakDrop(limit, state, to, drop_prob, out);

	const int move_prob = (state.hasMultipleEffectBy(Turn, to) ? 100 : 400);
	if (move_prob > limit)
	  continue;
	all_moves.clear();
	move_generator::GenerateCapture::generate
	  (Turn, state, to, store);
	for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
	     ++p)
	{
	  out.push_back(MoveLogProb(*p, move_prob));
	}
      }
    }
  }
  
  // king walk
  const Piece my_king = state.getKingPiece(Turn);
  const int king_prob = 100;
  if (king_prob <= limit)
  {
    all_moves.clear();
    GeneratePieceOnBoard::generate(Turn, state, my_king, all_moves);
    for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
    {
      if (state.hasEffectBy(alt(Turn), p->to()))
	continue;
      out.push_back(MoveLogProb(*p, king_prob));
    }
  }
  
  // open king road
  const Position center = my_king.position();
  generateOpenRoad(limit, state, center + DirectionTraits<U>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<UL>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<UR>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<L>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<R>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<D>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<DL>::blackOffset(), out);
  generateOpenRoad(limit, state, center + DirectionTraits<DR>::blackOffset(), out);
}

void osl::category::BreakThreatmate::
generateOpenRoad(int limit, const NumEffectState& state, 
		 Position from, MoveLogProbVector& out)
{
  const Piece target = state.getPieceAt(from);
  if (! target.isPiece())
    return;
  const Player Turn = state.getTurn();
  if (target.owner() != Turn)
    return;

  const int capture_prob = 50;
  const int default_prob = 100;
  const int sacrifice_prob = 400;
  if (limit < capture_prob)
    return;

  MoveVector moves;
  GeneratePieceOnBoard::generate(Turn, state, target, moves);

  for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p)
  {
    const bool capture = (p->capturePtype() != PTYPE_EMPTY);
    const bool sacrifice = state.hasEffectBy(alt(Turn), p->to());
    const int prob = capture ? capture_prob
      : (sacrifice ? sacrifice_prob : default_prob);
    if (prob <= limit)
      out.push_back(MoveLogProb(*p, prob));
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
