/* capture.cc
 */
#include "osl/category/capture.h"
#include "osl/category/probTableDefine.h"
#include "osl/move_generator/capture_.h"
#include "osl/move_generator/additionalLance.h"
#include "osl/move_action/store.h"
#include "osl/state/simpleState.tcc"
#include <string>

namespace osl
{
  namespace category
  {
    DEF_CATEGORY_TABLE_IP(PieceValueTable, ContinueCapture);
    template <osl::Ptype PTYPE> 
    DEF_CATEGORY_TABLE_IP(PieceValueTable, Capture<PTYPE>);
    DEF_CATEGORY_TABLE(FixedValueTable, TrapRook);
  }
}


template <osl::Ptype PTYPE>
void osl::category::Capture<PTYPE>::
generate(const CategoryEnv& env, MoveVector& out)
{
  const Player turn = env.state->getTurn();
  move_action::Store a(out);
  for (int i=PtypeTraits<PTYPE>::indexMin; 
       i < PtypeTraits<PTYPE>::indexLimit; i++)
  {
    const Piece target = env.state->getPieceOf(i);
    if (target.owner() == alt(env.state->getTurn()))
    {
      move_generator::GenerateCapture::generate
	(turn, *env.state, target.position(), a);
    }
  }

  if (env.state->hasPieceOnStand(turn,LANCE))
  {
    // Ƕ꤬ϡɲĤ

    // NOTE: MoveVector push_backƤiteratorinvalidate
    // ʤȤͤ˰¸
    MoveVector::const_iterator original_end = out.end();
    for (MoveVector::const_iterator p=out.begin(); p!=original_end; ++p)
    {
      const Position from = p->from();
      if ((p->oldPtype() == PAWN)
	  && (env.state->hasEffectBy(alt(turn), p->to())))
      {
	if (turn == BLACK)
	  move_generator::AdditionalLance<BLACK>::generate(*env.state, from, out);
	else
	  move_generator::AdditionalLance<WHITE>::generate(*env.state, from, out);
      }
    }
  }
}

template <osl::Ptype PTYPE>
const char * osl::category::Capture<PTYPE>::getName()
{
  static std::string name = std::string("Capture-")
    + PtypeTraits<PTYPE>::name();
  return name.c_str(); 
}

template struct osl::category::Capture<osl::ROOK>;
template struct osl::category::Capture<osl::BISHOP>;
template struct osl::category::Capture<osl::GOLD>;
template struct osl::category::Capture<osl::SILVER>;
template struct osl::category::Capture<osl::KNIGHT>;
template struct osl::category::Capture<osl::LANCE>;
template struct osl::category::Capture<osl::PAWN>;


struct osl::category::ContinueCapture::PositionAction
{
  const CategoryEnv::effect_state_t *state;
  Position attackFrom;
  Ptype attackerPtype;
  MoveVector *out;
  PositionAction(const CategoryEnv::effect_state_t *s, 
		 Position from, Ptype ptype,
		 MoveVector *o)
    : state(s), attackFrom(from), attackerPtype(ptype), out(o)
  {
  }
  template<Player P>
  void doAction(Piece, Position pos)
  {
    const Piece target = state->getPieceAt(pos);
    if (target.isPiece() && (target.owner() == alt(state->getTurn())))
    {
      assert(target.position()==pos);
      if (isBasic(attackerPtype) && canPromote(attackerPtype)
	  && (pos.canPromote(state->getTurn())
	      || attackFrom.canPromote(state->getTurn())))
      {
	// promotion move
	const Move m1(attackFrom, pos, promote(attackerPtype),
		      target.ptype(), true, state->getTurn());
	out->push_back(m1);
      }
      if ((! isBasic(attackerPtype))
	  || Ptype_Table.canDropTo(state->getTurn(), attackerPtype, pos))
      {
	// no promotion move
	const Move m2(attackFrom, pos, attackerPtype,
		      target.ptype(), false, state->getTurn());
	out->push_back(m2);
      }
    }
  }
};
void osl::category::
ContinueCapture::generate(const CategoryEnv& env, MoveVector& out)
{
  if (! isEffective(env))
    return;
  const Move lastLastMove=env.history->lastMove(2);
  const Position attackFrom = lastLastMove.to();
  const Piece attacker = env.state->getPieceOnBoard(attackFrom);
  assert(attacker.isPiece());
  if (attacker.owner() != env.state->getTurn())
    return;
	
  PositionAction a(env.state, attackFrom, attacker.ptype(), &out);
  env.state->forEachEffectOfPiece(attacker,a);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
