/* defense.cc
 */
#include "osl/category/defense.h"
#include "osl/category/categoryEnv.h"
#include "osl/category/probTableDefine.h"
#include "osl/eval/pieceEval.h"
#include "osl/effect_util/effectUtil.h"
#include "osl/effect_util/effectUtil.tcc"
#include "osl/container/moveStack.h"
#include "osl/state/simpleState.tcc"

namespace osl
{
  namespace category
  {
DEF_CATEGORY_TABLE_IP(PtypePtypeProbTable, AddSupport);
DEF_CATEGORY_TABLE_IP(PieceValueTable, AntiPromotion);
DEF_CATEGORY_TABLE_IP(PieceValueTable, AddEffectToLastEffect);
DEF_CATEGORY_TABLE_IP(PieceValueTable, BlockLong);
  }
}

struct osl::category::AddSupport::Test
{
  const CategoryEnv::effect_state_t *state;
  Player turn;

  Piece result;
  bool supported;
  int value;
  Test(const CategoryEnv::effect_state_t *s, Player t)
    : state(s), turn(t),
      result(Piece::EMPTY()), supported(1), value(0)
  {
  }
  /**
   * @param pos move Ĥposition
   */
  void operator()(Position pos)
  {
    const Piece target = state->getPieceAt(pos);
    assert(target.isPiece());
    if ((target.owner() == turn)
	&& state->hasEffectBy(alt(turn), pos))
    {
      update(pos, target);
    }
  }
  /**
   * ʣζĤ:
   * - ʬʤͥ
   * - ͤ⤤ͥ
   */
  void update(Position pos, Piece target)
  {
    const bool supported = state->hasEffectBy(turn, pos);
    if ((! this->supported) && supported)
      return;
    const int targetVal = eval::Ptype_Eval_Table.value(target.ptype());
    if (((! supported) && this->supported)
	|| (targetVal > value))
    {
      result = target;
      this->supported = supported;
      value = targetVal;
    }
  }
};

/**
 * move ŨΤ̣ζĤ
 * fixme: open attack ˤʤȤʤ
 */
std::pair<bool,size_t> osl::category::
AddSupport::getIndexIf(const CategoryEnv& env, Move move, int pieceValue)
{
  const Player turn = move.player();
  assert(env.state->getTurn() == turn);
  Test selector(env.state, turn);
  EffectUtil::forEachEffectPieceOfMove(*env.state, move, selector);
  if (! selector.result.isPiece())
    return std::make_pair(false,0u);
    
  const Piece attacker 
    = EffectUtil::cheapestPiece(alt(turn), *env.state, 
				selector.result.position());
  const size_t index
    = probTable.getIndex(selector.result.ptype(), selector.supported, 
			 attacker.ptype(), pieceValue);
  return std::make_pair(true, index);
}

struct osl::category::AddEffectToLastEffect::CheckEffect
{
  bool *result;
  const PawnMaskState& state;
  Move move;
  CheckEffect(bool *r, const PawnMaskState& s, Move m)
    : result(r), state(s), move(m)
  {
  }
  template <Player P>
  void doAction(Piece /*piece*/, Position lastAttack)
  {
    if (! lastAttack.isOnBoard())
      return;
    const Piece attacked = state.getPieceOnBoard(lastAttack);
    if (((attacked == Piece::EMPTY()) // Ũζޤ⤢ޤѤʤΤǸɵ
	 || (attacked.owner() == state.getTurn()))
	&& state.hasEffectFromTo(move.ptypeO(), move.to(), lastAttack))
      *result = true;
  }
};
bool osl::category::AddEffectToLastEffect::
isInCategory(const CategoryEnv& env, Move move)
{
  const Move last_move=env.history->lastMove();
  if (! last_move.isNormal()) 
    return false;
  const Position lastTo = last_move.to();
  if (lastTo==move.to())
    return false;		// TakeBack Ͻ
  bool result = false;
  CheckEffect action(&result, *env.state, move);
  env.state->forEachEffectOfPiece(env.state->getPieceOnBoard(lastTo), action);
  return result;
}

struct osl::category::AntiPromotion::Test
{
  const CategoryEnv::effect_state_t& state;
  const Player opponent;
  bool result;

  Test(const CategoryEnv::effect_state_t& s, Player o)
    : state(s), opponent(o), result(false)
  {
  }
  void operator()(Position focus)
  {
    // focus ؤɤ
    if (! focus.canPromote(opponent))
      return;
      
    if ((state.getPieceOnBoard(focus) == Piece::EMPTY())
	&& (! state.hasEffectBy(alt(opponent), focus)))
    {
      const Piece promoter = 
	EffectUtil::promotablePiece(opponent, state, focus); 
      if (promoter.isPiece())
      {
	assert(canPromote(promoter.ptype()));
	result = true;
      }
    }
  }
};

bool osl::category::AntiPromotion::
isInCategory(const CategoryEnv& env, Move move)
{
  const Player opponent = alt(env.state->getTurn());
  int ymin, ymax;
  if (opponent == BLACK)
  {
    ymin = 1;
    ymax = 4;
  }
  else 
  {
    ymin = 7;
    ymax = 10;
  }
  const Ptype ptype = unpromote(move.ptype());
  const Position to = move.to();
  if ((ptype != BISHOP) && (ptype != ROOK))
  {
    const int toY = to.y();
    if (opponent == BLACK)
    {
      if (toY > ymax)		// 5-9
	return false;
    }
    else
    {
      if (toY < ymin-1)		// 1-5
	return false;
    }
  }

  Test checker(*env.state, opponent);
  EffectUtil::forEachEffectPositionOfMove(*env.state, move, checker);
  return false;
};

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
