/* escape.cc
 */
#include "osl/category/escape.h"
#include "osl/category/probTable.tcc"
#include "osl/category/moveEvaluation.h"
#include "osl/category/probTableDefine.h"
#include "osl/move_action/store.h"
#include "osl/move_generator/escape_.h"
#include "osl/move_classifier/moveAdaptor.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/state/simpleState.tcc"
#include <string>



struct osl::category::EscapeCommon::Generator
{
  const CategoryEnv::effect_state_t *state;
  const PtypeValueTable *table;
  int limit;
  MoveLogProbVector *out;
  int *indices;
  Generator(const CategoryEnv::effect_state_t *s, const PtypeValueTable *t,
	    int l, MoveLogProbVector *o, int *io)
    : state(s), table(t), limit(l), out(o), indices(io)
  {
  }
  /** @param p ƨ */
  void operator()(Piece p)
  {
    const Player myself = state->getTurn();
    assert(p.owner() == myself);
    if (state->hasEffectBy(alt(myself),p.position()))
    {
      MoveVector moves;
      move_action::Store store(moves);
      const Player opponent = alt(myself);
      const Piece attacker = 
	EffectUtil::cheapestPiece(opponent, *state, p.position());
      if(myself==BLACK)
	move_generator::Escape<BLACK,CategoryEnv::effect_state_t,move_action::Store>::
	  generateMoves(*state, p, attacker, store);
      else
	move_generator::Escape<WHITE,CategoryEnv::effect_state_t,move_action::Store>::
	  generateMoves(*state, p, attacker, store);
      for (size_t i=0; i<moves.size(); ++i)
      {
	const int diff = MoveEvaluation::evalLight(*state, moves[i]);
	const bool supported 
	  = state->hasEffectBy(myself, p.position());
	const size_t index = 
	  table->getIndex(supported, attacker.ptype(), diff);
	const int prob = (*table)[index];
	if (prob <= limit)
	{
	  out->push_back(moves[i], prob);
	  if (indices)
	    *indices++ = index;
	}
      }
    }
  }
};



template <osl::Ptype PTYPE>
void osl::category::Escape<PTYPE>::
generate(const CategoryEnv& env, MoveLogProbVector& out, int *indices)
{
  EscapeCommon::Generator gen(env.state, &probTable, env.limit, &out, indices);
  if (env.state->getTurn() == BLACK)
    (*env.state).template forEachOnBoard<BLACK,PTYPE,EscapeCommon::Generator>
      (gen);
  else
    (*env.state).template forEachOnBoard<WHITE,PTYPE,EscapeCommon::Generator>
      (gen);
}

template <osl::Ptype PTYPE>
void osl::category::Escape<PTYPE>::
generate(const CategoryEnv& env, MoveLogProbVector& out)
{
  generate(env, out, 0);
}

template <osl::Ptype PTYPE>
const char * osl::category::Escape<PTYPE>::getName()
{
  static std::string name = std::string("Escape-")
    + PtypeTraits<PTYPE>::name();
  return name.c_str(); 
}


struct osl::category::ImmediateEscape::PositionAction
{
  const CategoryEnv::effect_state_t *state;
  Position attackFrom;
  int limit;
  MoveLogProbVector *out;
  int *indexOut;
  PositionAction(const CategoryEnv::effect_state_t *s, Position from,
		 int l, MoveLogProbVector *o, int *iout)
    : state(s), attackFrom(from), limit(l), out(o), indexOut(iout)
  {
  }
  template<Player P>
  void doAction(Piece attacker, Position pos);
};

template<osl::Player P>
void osl::category::ImmediateEscape::PositionAction::
doAction(Piece attacker, Position pos)
{
  assert(P==alt(state->getTurn()));
  const Piece p = state->getPieceAt(pos);	   // פγ⤷ʤ
  if (p.isPiece() && (p.owner() == state->getTurn()))
  {
    MoveVector moves;
    move_action::Store store(moves);
    move_generator::Escape<PlayerTraits<P>::opponent,CategoryEnv::effect_state_t,move_action::Store>::
      generateMoves(*state, p,state->getPieceAt(attackFrom), store);
    for (size_t i=0; i<moves.size(); ++i)
    {
      const int diff = MoveEvaluation::evalLight(*state, moves[i]);
      const bool supported 
	= state->hasEffectBy(state->getTurn(), attackFrom);
      const size_t index = 
	probTable.getIndex(p.ptype(), supported, attacker.ptype(), diff);
      const int prob = probTable[index];
      if (prob <= limit)
      {
	out->push_back(moves[i], prob);
	if (indexOut)		// FIXME: ٤ɤޤ
	  *indexOut++ = index;
      }
    }
  }
}

void osl::category::ImmediateEscape::
generate(const CategoryEnv& env, MoveLogProbVector& out, int *indices)
{
  const Move last_move=env.history->lastMove();
  if (! last_move.isNormal()) 
    return;
  const Position pos = last_move.to();
  const Piece attacker = env.state->getPieceOnBoard(pos);
  assert(attacker.isPiece());
  assert(attacker.owner() == alt(env.state->getTurn()));

  PositionAction a(env.state, pos, env.limit, &out, indices);
  env.state->forEachEffectOfPiece(attacker,a);
}

void osl::category::ImmediateEscape::
generate(const CategoryEnv& env, MoveLogProbVector& out)
{
  generate(env, out, 0);
}
namespace osl
{
  namespace category
  {
    DEF_CATEGORY_TABLE_IP(PtypePtypeProbTable, ImmediateEscape);
    template <Ptype PTYPE>
    DEF_CATEGORY_TABLE_IP(PtypeValueTable, Escape<PTYPE>);
  }
}
template struct osl::category::Escape<osl::ROOK>;
template struct osl::category::Escape<osl::BISHOP>;
template struct osl::category::Escape<osl::GOLD>;
template struct osl::category::Escape<osl::SILVER>;
template struct osl::category::Escape<osl::KNIGHT>;
template struct osl::category::Escape<osl::LANCE>;
template struct osl::category::Escape<osl::PAWN>;

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
