/* checkmateIfCapture.cc
 */
#include "osl/checkmate/checkmateIfCapture.h"
#include "osl/checkmate/fixedDepthSearcher.h"
#include "osl/effect_util/effectUtil.h"
#include "osl/move_generator/capture_.h"
#include "osl/move_action/store.h"
#include "osl/apply_move/applyMove.h"

struct osl::checkmate::CheckmateIfCapture::CallDefense
{
  NumEffectState *state;
  int depth;
  bool result;
  void operator()(Position last_to)
  {
    result = cannotCapture(*state, last_to, depth);
  }
};


bool osl::checkmate::
CheckmateIfCapture::effectiveAttack(NumEffectState& state, Move move, int depth)
{
  CallDefense defense = { &state, depth };
  ApplyMoveOfTurn::doUndoMove(state, move, defense);
  return defense.result;
}

bool osl::checkmate::
CheckmateIfCapture::cannotCapture(NumEffectState& state, 
				  Position last_to, int depth)
{
  if (EffectUtil::isKingInCheck(alt(state.getTurn()), state))
    return false;		// μ꤬

  using namespace move_generator;
  using namespace move_action;
  MoveVector moves;		// may contain unsafe move
  Store store(moves);
  GenerateCapture::generate(state.getTurn(), state, last_to, store);
  if (moves.empty())
    return false;

  FixedDepthSearcher searcher(state);
  for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p)
  {
    const bool checkmate
      = searcher.hasEscapeByMoveOfTurn(*p, depth).isCheckmateSuccess();
    if (! checkmate)
      return false;
  }

  return true;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
