/* king8Info.cc
 */
#include "osl/checkmate/king8Info.h"
#include "osl/effect_util/additionalEffect.h"
#include "osl/effect_util/pin.h"
#include <bitset>
#include <iostream>

std::ostream& osl::checkmate::operator<<(std::ostream& os, King8Info info)
{
  typedef std::bitset<8> bs_t;
  os << bs_t(info.moveCandidate()) << " " 
     << bs_t(info.libertyCandidate()) << " " 
     << bs_t(info.liberty()) << " " 
     << bs_t(info.dropCandidate());
  return os;
}

namespace osl
{
  namespace
  {
/**
 * Pζ̤pinƤʳ.
 * @param state - (alt(P)μ֤Ȥϸ¤ʤ)
 * @param target - Pζ̤ΰ
 * @param pos - ̾(Pζ̤Ĺΰ֤ˤȤϸ¤ʤ)
 * @param pinned - pinƤPζmask
 */
template<Player P>
bool hasEnoughEffect(NumEffectState const& state,Position target,Position pos,PieceMask pinned)
{
  assert(state.getKingPosition(P)==target);
  assert(pos.isOnBoard());
  PieceMask pieceMask=state.getEffect(pos)&state.getOnBoardMask(P);
  pieceMask.reset(KingTraits<P>::index);
  if(pieceMask.none()) return false;
  PieceMask pieceMask1=pieceMask&~pinned;
  if(pieceMask1.any()) return true;
  PieceMask pieceMask2=pieceMask&pinned;
  while(pieceMask2.any()){
    int num=pieceMask2.takeOneBit();
    Piece p=state.getPieceOf(num);
    assert(p.isOnBoardByOwner(P));
    Position pos1=p.position();
    if(Board_Table.getShortOffset(Offset32(pos1,target))==
       Board_Table.getShortOffset(Offset32(pos,target))) return true;
  }
  return false;
}
  }
}

template<osl::Player P,osl::Direction Dir>
unsigned int osl::checkmate::
King8Info::hasEffectMask(NumEffectState const& state,Position target,PieceMask pinned)
{
  const Player altP=PlayerTraits<P>::opponent;
  Position pos=target-DirectionPlayerTraits<Dir,P>::offset();
  Piece p=state.getPieceAt(pos);
  if(p.canMoveOn<altP>()){ // ʬζ𤫶
    if(!state.hasEffectBy(P,pos)) // ʬʤ
      return 0x10100u<<static_cast<int>(Dir);
    else if(p.isEmpty() &&
	    !hasEnoughEffect<altP>(state,target,pos,pinned)){ // ʤ
      // ֤ζΤ褦ˡɲʤȼ夤
      if(state.countEffect(P,pos)>=2 ||
	 effect_util::AdditionalEffect::hasEffect(state,pos,P))
	return 0x1010001u<<static_cast<int>(Dir);
      else
	return 0x10001u<<static_cast<int>(Dir);
    }
    else{
      return 0x10000u<<static_cast<int>(Dir);
    }
  }
  else if(!p.isEdge()){
    if(hasEnoughEffect<altP>(state,target,pos,pinned)) return 0;
    if(state.countEffect(P,pos)>=2 ||
       effect_util::AdditionalEffect::hasEffect(state,pos,P))
      return 0x1000000u<<static_cast<int>(Dir);
    else
      return 0;
  }
  return 0u;
}

template<osl::Player P>
const osl::checkmate::King8Info 
#ifdef __GNUC__
__attribute__ ((noinline))
#endif
osl::checkmate::King8Info::make(NumEffectState const& state,Position target,PieceMask pinned)
{
  unsigned int canMoveMask=hasEffectMask<P,UR>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,R>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,DR>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,U>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,D>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,UL>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,L>(state,target,pinned);
  canMoveMask|=hasEffectMask<P,DL>(state,target,pinned);
  return King8Info(canMoveMask);
}

template <osl::Player P>
const osl::checkmate::King8Info 
#ifdef __GNUC__
__attribute__ ((noinline))
#endif
osl::checkmate::
King8Info::make(NumEffectState const& state, Position target)
{
  const Player altP=PlayerTraits<P>::opponent;
  PieceMask pins = effect_util::Pin::make(state,target,altP);
  return make<P>(state,target,pins);
}

const osl::checkmate::King8Info osl::checkmate::
King8Info::make(Player attack, NumEffectState const& state)
{
  const Position king=state.getKingPosition(alt(attack));
  if (attack == BLACK)
    return make<BLACK>(state, king);
  else
    return make<WHITE>(state, king);
}

const osl::checkmate::King8Info osl::checkmate::
King8Info::makeWithPin(Player attack, NumEffectState const& state,
		       const PieceMask& pins)
{
  const Position king=state.getKingPosition(alt(attack));
  if (attack == BLACK)
    return make<BLACK>(state, king, pins);
  else
    return make<WHITE>(state, king, pins);
}


/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

