#include "osl/effect_util/additionalEffect.h"
#include "osl/effect_util/additionalOrShadow.h"
#include "osl/effect_util/effectUtil.h"

template <int count_max>
int osl::effect_util::
AdditionalEffect::count(const NumEffectState& state, Position target, 
			Player attack)
{
  PieceVector direct_pieces;
  EffectUtil::findEffect(attack, state, target, direct_pieces);
  return AdditionalOrShadow::count<count_max>
    (direct_pieces, state, target, attack);
}

bool osl::effect_util::
AdditionalEffect::hasEffect(const NumEffectState& state, Position target, 
			    Player attack)
{
  return count<1>(state, target, attack);
}

int osl::effect_util::
AdditionalEffect::count2(const NumEffectState& state, Position target, 
			    Player attack)
{
  return count<2>(state, target, attack);
}

void osl::effect_util::
AdditionalEffect::find(const NumEffectState& state, Position target, 
		       const PieceVector& direct_effects,
		       PieceVector& black, PieceVector& white)
{
  for (PieceVector::const_iterator p=direct_effects.begin();
       p!=direct_effects.end(); ++p)
  {
    const Position from = p->position();
    const Offset32 diff32 = Offset32(from, target);
    const Offset step = Board_Table.getShortOffsetNotKnight(diff32);
    if (step.zero())
      continue;
    // 8ξ
    Piece candidate=state.nextPiece(from, step);
    if (! candidate.isPiece())
      continue;
    const Offset32 diff_reverse = Offset32(target,candidate.position());
    for (; candidate.isPiece(); 
	 candidate=state.nextPiece(candidate.position(), step))
    {
      const EffectContent effect 
	= Ptype_Table.getEffect(candidate.ptypeO(), diff_reverse);
      if (! effect.hasEffect())
	break;
      if (candidate.owner() == BLACK)
	black.push_back(candidate);
      else
	white.push_back(candidate);
    }
  }
  
}

void osl::effect_util::
AdditionalEffect::find(const NumEffectState& state, Position target, 
		     PieceVector& black, PieceVector& white)
{
  PieceVector direct_pieces;
  EffectUtil::findEffect(BLACK, state, target, direct_pieces);
  find(state, target, direct_pieces, black, white);
  
  direct_pieces.clear();
  EffectUtil::findEffect(WHITE, state, target, direct_pieces);
  find(state, target, direct_pieces, black, white);
}

void osl::effect_util::
AdditionalEffect::count(const NumEffectState& state, Position target,
		      int& black, int& white)
{
  PieceVector black_pieces, white_pieces;
  find(state, target, black_pieces, white_pieces);
  black = black_pieces.size();
  white = white_pieces.size();
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
