/* mtdfPlayer.cc
 */
#include "osl/game_playing/mtdfPlayer.h"
#include "osl/game_playing/searchPlayer.tcc"
#include "osl/search/mtdf.h"
#include "osl/search/simpleHashTable.h"
#include "osl/eval/progressEval.h"
#include "osl/eval/pieceEval.h"
#include "osl/category/standardMoveGenerator.h"

/* ------------------------------------------------------------------------- */

osl::game_playing::
PlainMtdfPlayer::PlainMtdfPlayer()
{
}

osl::game_playing::
PlainMtdfPlayer::~PlainMtdfPlayer()
{
}

osl::game_playing::ComputerPlayer* osl::game_playing::
PlainMtdfPlayer::clone() const 
{
  return cloneIt(*this);
}

const osl::game_playing::MoveWithComment osl::game_playing::
PlainMtdfPlayer::searchWithSecondsForThisMove(const GameState& gs, int seconds)
{
  typedef MTDF<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> mtdf_t;

  const int consumed = setUpTable(gs, 0);
  if (seconds > 0)
    seconds = std::max(1, seconds-consumed);
  searcher.reset();
  searcher.reset
    (new mtdf_t(gs.state(), *checkmate_ptr, *table_ptr, *recorder_ptr));
  return SearchPlayer::search<mtdf_t>(gs, seconds);
}

bool osl::game_playing::
PlainMtdfPlayer::isReasonableMove(const GameState& gs,
				  Move move, int pawn_sacrifice)
{
  typedef MTDF<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> mtdf_t;
  setUpTable(gs, 0);
  mtdf_t searcher(gs.state(), *checkmate_ptr, *table_ptr, *recorder_ptr);  
  return isReasonableMoveBySearch(searcher, move, pawn_sacrifice);
}

/* ------------------------------------------------------------------------- */

osl::game_playing::
ProgressMtdfPlayer::ProgressMtdfPlayer()
{
}

osl::game_playing::
ProgressMtdfPlayer::~ProgressMtdfPlayer()
{
}

osl::game_playing::ComputerPlayer* osl::game_playing::
ProgressMtdfPlayer::clone() const 
{
  return cloneIt(*this);
}

const osl::game_playing::MoveWithComment osl::game_playing::
ProgressMtdfPlayer::searchWithSecondsForThisMove(const GameState& gs, int seconds)
{
  typedef MTDF<eval::ProgressEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> mtdf_t;

  const int consumed = setUpTable(gs, 0);
  if (seconds > 0)
    seconds = std::max(1, seconds-consumed);
  searcher.reset();
  searcher.reset
    (new mtdf_t(gs.state(), *checkmate_ptr, *table_ptr, *recorder_ptr));
  return SearchPlayer::search<mtdf_t>(gs, seconds);
}

bool osl::game_playing::
ProgressMtdfPlayer::isReasonableMove(const GameState& gs,
				     Move move, int pawn_sacrifice)
{
  typedef MTDF<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> mtdf_t;
  setUpTable(gs, 0);
  mtdf_t searcher(gs.state(), *checkmate_ptr, *table_ptr, *recorder_ptr);  
  return isReasonableMoveBySearch(searcher, move, pawn_sacrifice);
}


/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
