/* tableStack.cc
 */
#include "osl/game_playing/tableStack.h"
#include "osl/search/simpleHashTable.h"
#include "osl/hash/hashKey.h"
#include "osl/record/csa.h"
#include "osl/misc/nonBlockDelete.h"
#include <boost/scoped_ptr.hpp>
#include <list>
#include <iostream>

struct osl::game_playing::TableStack::TableWithRoot
{
  boost::scoped_ptr<SimpleHashTable> table;
  HashKey root;
};

struct osl::game_playing::TableStack::Stack
  : public std::list<TableWithRoot*>
{
};

osl::game_playing::
TableStack::TableStack()
  : stack(new Stack())
{
}

osl::game_playing::
TableStack::TableStack(const TableStack& copy)
  : stack(new Stack())
{
  for (Stack::const_iterator p=copy.stack->begin(); 
       p!=copy.stack->end(); ++p)
  {
    stack->push_back(new TableWithRoot());
    TableWithRoot& store = *(stack->back());
    store.table.reset(new SimpleHashTable(100000,0,false));
    store.root = (*p)->root;
    try 
    {
      (*p)->table->migrate(store.root, migration_depth, 
			   migration_limit, *store.table);
    }
    catch(TableFull&)
    {
    }
  }
}

osl::game_playing::
TableStack::~TableStack()
{
  clear();
}

void osl::game_playing::
TableStack::clear()
{
  while (! stack->empty())
  {
    TableWithRoot *top = stack->front();
    stack->pop_front();
    NonBlockDelete::planDelete(top);
  }
}

void osl::game_playing::
TableStack::pushMove(Move selected, const HashKey& root, 
		     const SimpleHashTable& table)
{
  stack->push_back(new TableWithRoot());
  TableWithRoot& store = *(stack->back());
  store.table.reset(new SimpleHashTable(100000,0,table.verboseLevel()));
  store.root = root.newHashWithMove(selected);
  try 
  {
    table.migrate(store.root, migration_depth, 
		  migration_limit, *store.table);
  }
  catch(TableFull&)
  {
  }
  if (table.verboseLevel() > 1 && store.table->size())
    std::cerr << "preserved " << store.table->size() << " entries for "
	      << record::csa::show(selected)
	      << " stack " << stack->size() << "\n";
  
  if (stack->size() > max_size)
  {
    TableWithRoot *top = stack->front();
    stack->pop_front();
    NonBlockDelete::planDelete(top);
  }
  assert(stack->size() <= max_size);
  assert(! stack->empty());
}

void osl::game_playing::
TableStack::pushMove(Move selected)
{
  if (stack->empty())
    return;
  pushMove(selected, stack->back()->root, *stack->back()->table);
}

void osl::game_playing::
TableStack::popMove()
{
  if (! stack->empty())
    stack->pop_back();
}

void osl::game_playing::
TableStack::migrate(SimpleHashTable& table) const
{
  if (stack->empty())
    return;
  assert(stack->back()->table.get());
  try 
  {
    stack->back()->table->migrate(stack->back()->root, migration_depth, 
				  migration_limit, table);
  }
  catch(TableFull&)
  {
  }
#ifndef NDEBUG
  if (table.verboseLevel() > 1 && table.size())
  {
    char ctime_buf[64];
    time_t t = time(0);
    std::cerr << "consistency " << ctime_r(&t, ctime_buf) << std::flush;
  }
  assert(table.isConsistent());
#endif
  if (table.verboseLevel() > 1 && table.size())
    std::cerr << "migrated " << table.size() << " entries\n";
}


// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
