/* alarm.cc
 */
#include "osl/misc/alarm.h"
#include "osl/misc/lightMutex.h"
#include <stdexcept>
#include <iostream>
#include <cassert>
#include <signal.h>
#include <unistd.h>

volatile int * volatile osl::misc::Alarm::notify = 0;
static osl::LightMutex alarm_mutex;

void osl::misc::
Alarm::catchSignal(int 
#ifndef NDEBUG
	signum
#endif
  )
{
  assert(SIGALRM == signum);
  if (notify)
    *notify = TIMEOUT;
  notify = 0;
#ifdef DEBUG
  const time_t now = time(0);
  char ctime_buf[64];
  std::cerr << "\asignal alarm " << ctime_r(&now, ctime_buf);
#endif
}

bool osl::misc::
Alarm::set(int timeout, AlarmSwitch notify)
{
  assert(notify.notify);
  assert(timeout > 0);
  if (timeout <= 0)
    throw std::range_error("negative or zero seconds in Alarm::set");
  LightMutex::scoped_lock lk(alarm_mutex);
  if (Alarm::notify)
  {
    std::cerr << "Alarm reject " << notify.notify << " (" << Alarm::notify << ")" << "\n";
    *notify.notify = UNAVAILABLE;
    return false;
  }
  if (*notify.notify == TIMEOUT)
  {
    std::cerr << "Alarm::set leave already timeout flag\n";
    return false;
  }
  Alarm::notify = notify.notify;
  signal(SIGALRM, catchSignal);
  alarm(timeout);
  *notify.notify = WATCHING;
  return true;
}

void osl::misc::
Alarm::reset(AlarmSwitch notify)
{
  LightMutex::scoped_lock lk(alarm_mutex);
  if (Alarm::notify && notify.notify != Alarm::notify)
  {
    std::cerr << "warning: Alarm::reset inconsistency " 
	      << const_cast<int*>(notify.notify) << " " << const_cast<int*>(Alarm::notify) << "\n";
    return;
  }
  Alarm::notify = 0;
  alarm(0);
}


/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
