/* effect5x3.cc
 */
#include "osl/progress/effect5x3.h"
#include "osl/progress/effect5x3Table.h"
#include "osl/pieceStand.h"

int osl::progress::
Effect5x3::makeProgressAll(Player defense, const NumEffectState& state,
			   Position king)
{  
  return makeProgressArea(alt(defense), state, king) 
    + makeProgressStand(alt(defense), state);
}

int osl::progress::
Effect5x3::makeProgressArea(Player attack, const NumEffectState& state,
			   Position king)
{  
  const Position center = Centering5x3::adjustCenter(king);

  const int min_x = center.x() - 2;
  const int min_y = center.y() - 1;

  // 
  int sum_effect = 0;

  for (int dx=0; dx<5; ++dx)
    {
      for (int dy=0; dy<3; ++dy)
	{
	  const Position target(min_x+dx,min_y+dy);
	  sum_effect += state.countEffect(attack, target) *
	    Effect5x3_Table.getAttackEffect(attack,king,dx,dy);
	}
    }
  return sum_effect / 2;
}

int osl::progress::
Effect5x3::makeProgressAreaBonus(Player attack, const NumEffectState& state,
				 Position king)
{  
  const Position center = Centering5x3::adjustCenter(king);

  const int min_x = center.x() - 2;
  const int min_y = center.y() - 1;

  // 
  int sum_effect = 0;

  for (int dy = 0; dy < 3; ++dy)
  {
    const Position target(king.x(), min_y + dy);
    const int effect =
      std::min(state.countEffect(attack, target),
	       state.countEffect(alt(attack), target) +
	       state.getPieceAt(target).isOnBoardByOwner(attack) ? 1 : 2);
    sum_effect += effect *
      Effect5x3_Table.getAttackEffect(attack, king, 0, dy);
  }
  for (int x = king.x() - 1; x >= min_x; --x)
  {
    int y_count = 0;
    for (int dy = 0; dy < 3; ++dy)
    {
      const Position target(x, min_y + dy);
      const int effect =
	std::min(state.countEffect(attack, target),
		 state.countEffect(alt(attack), target) +
		 state.getPieceAt(target).isOnBoardByOwner(attack) ? 1 : 2);
      sum_effect += effect *
	Effect5x3_Table.getAttackEffect(attack, king, king.x() - x, dy);
      if (effect > 0)
	y_count++;
    }
    if (y_count == 3)
    {
      for (int dy = 0; dy < 3; ++dy)
      {
	sum_effect += Effect5x3_Table.getAttackEffect(attack, king, king.x() - x, dy);
      }
      break;
    }
  }
  for (int x = king.x() + 1; x < min_x + 5; ++x)
  {
    int y_count = 0;
    for (int dy = 0; dy < 3; ++dy)
    {
      const Position target(x, min_y + dy);
      const int effect =
	std::min(state.countEffect(attack, target),
		 state.countEffect(alt(attack), target) +
		 state.getPieceAt(target).isOnBoardByOwner(attack) ? 1 : 2);
      sum_effect += effect *
	Effect5x3_Table.getAttackEffect(attack, king, x - king.x(), dy);
      if (effect > 0)
	y_count++;
    }
    if (y_count == 3)
    {
      for (int dy = 0; dy < 3; ++dy)
      {
	sum_effect += Effect5x3_Table.getAttackEffect(attack, king, x - king.x(), dy);
      }
      break;
    }
  }
  return sum_effect / 2;
}

int osl::progress::
Effect5x3::makeProgressStand(Player attack, const NumEffectState& state)
{  
  // 
  int sum_pieces = 0;
  sum_pieces += state.countPiecesOnStand<PAWN>(attack)*Effect5x3Table::StandPAWN;
  sum_pieces += state.countPiecesOnStand<LANCE>(attack)*Effect5x3Table::StandLANCE;
  sum_pieces += state.countPiecesOnStand<KNIGHT>(attack)*Effect5x3Table::StandKNIGHT;
  sum_pieces += state.countPiecesOnStand<SILVER>(attack)*Effect5x3Table::StandSILVER;
  sum_pieces += state.countPiecesOnStand<GOLD>(attack)*Effect5x3Table::StandGOLD;
  sum_pieces += state.countPiecesOnStand<BISHOP>(attack)*Effect5x3Table::StandBISHOP;
  sum_pieces += state.countPiecesOnStand<ROOK>(attack)*Effect5x3Table::StandROOK;
  return sum_pieces;
}

void osl::progress::
Effect5x3::updateStand(int& old_stand, Move last_move)
{
  if (last_move.isDrop()) {
    const Ptype ptype = last_move.ptype();
    old_stand -= Effect5x3_Table.piecesOnStand(ptype);
    return;
  }
  const Ptype ptype = last_move.capturePtype();
  if (ptype == PTYPE_EMPTY) {
    return;
  }
  old_stand += Effect5x3_Table.piecesOnStand(unpromote(ptype));
}

void osl::progress::
Effect5x3::update(const NumEffectState& new_state, Move last_move)
{
  const Player pl = last_move.player();
  updateStand(stand_progresses[playerToIndex(alt(pl))], last_move);
  progresses[playerToIndex(WHITE)]=makeProgressArea(BLACK,new_state,new_state.getKingPosition<WHITE>());
  progresses[playerToIndex(BLACK)]=makeProgressArea(WHITE,new_state,new_state.getKingPosition<BLACK>());
  progress_bonuses[playerToIndex(WHITE)]=makeProgressAreaBonus(BLACK,new_state,new_state.getKingPosition<WHITE>());
  progress_bonuses[playerToIndex(BLACK)]=makeProgressAreaBonus(WHITE,new_state,new_state.getKingPosition<BLACK>());
  progresses[playerToIndex(BLACK)]+=stand_progresses[playerToIndex(BLACK)];
  progresses[playerToIndex(WHITE)]+=stand_progresses[playerToIndex(WHITE)];
}

osl::progress::Effect5x3 osl::progress::
Effect5x3::expect(const NumEffectState& state, Move move) const
{
  Effect5x3 new_progress = *this;
  if (move.capturePtype() != PTYPE_EMPTY) {
    const Player alt_pl = alt(move.player());
    int old = stand_progresses[playerToIndex(alt_pl)];
    updateStand(new_progress.stand_progresses[playerToIndex(alt_pl)], move);
    new_progress.progresses[playerToIndex(alt_pl)] += new_progress.stand_progresses[playerToIndex(alt_pl)] - old;
  }
  return new_progress;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
