/* group.cc
 */
#include "osl/rating/group.h"
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <cmath>

osl::rating::Group::~Group()
{
}

int osl::rating::Group::findMatch(const NumEffectState& state, Move m, const RatingEnv& env) const
{
  for (size_t j=0; j<size(); ++j) {
    if ((*this)[j].match(state, m, env)) 
      return j;
  }
  return -1;
}

void osl::rating::Group::saveResult(const std::string& directory, const range_t& range, 
				    const vector<double>& weights) const
{
  {
    boost::filesystem::path dir(directory);
    boost::filesystem::create_directory(dir);
  }
  
  std::string filename = directory + "/" + group_name + ".txt";
  std::ofstream os(filename.c_str());
  for (int i=range.first; i<range.second; ++i)
    os << std::setprecision(8) << weights[i] << "\n";
}

bool osl::rating::Group::load(const std::string& directory, const range_t& range, 
			      vector<double>& weights) const
{  
  std::string filename = directory + "/" + group_name + ".txt";
  std::ifstream is(filename.c_str());
  for (int i=range.first; i<range.second; ++i)
    is >> weights[i];
  return is;
}

void osl::rating::Group::show(std::ostream& os, int name_width, const range_t& range, 
			      const vector<double>& weights) const
{
  for (size_t f=0; f<size(); ++f) {
    os << std::setw(name_width) 
       << (*this)[f].name() 
       << "  " << 400*log10(weights[f+range.first]) << "\n";
  }
}

void osl::rating::Group::showAll(std::ostream& os, int name_width, const range_t& range, 
				 const vector<double>& weights) const
{
  showMinMax(os, name_width, range, weights);
  for (size_t i=0; i<size(); ++i) {
    os << " " << (*this)[i].name() << " " << 400*log10(weights[i+range.first]);
  }
  os << "\n";
}
void osl::rating::Group::showMinMax(std::ostream& os, int name_width, const range_t& range, 
				    const vector<double>& weights) const 
{
  double min = 10000000000.0, max = -min;
  for (size_t i=0; i<size(); ++i) {
    min = std::min(min, 400*log10(weights[i+range.first]));
    max = std::max(max, 400*log10(weights[i+range.first]));
  }
  os << std::setw(name_width) 
     << group_name
     << "  [" << min << " -- " << max << "] ";
}

void osl::rating::Group::showTopN(std::ostream& os, int name_width, const range_t& range, 
				  const vector<double>& weights, int n) const
{
  if ((int)weights.size() <= n*2)
    return showAll(os, name_width, range, weights);
  showMinMax(os, name_width, range, weights);
  vector<double> w;
  w.reserve(size());
  for (int i=range.first; i<range.second; ++i)
    w.push_back(weights[i]);
  std::sort(w.begin(), w.end());
  for (int i=0; i<n; ++i) {
    double value = w[size()-1-i];
    int j=range.first;
    for (; j<range.second; ++j)
      if (weights[j] == value)
	break;
    os << " " << (*this)[j-range.first].name() << " " << 400*log10(value);
  }
  os << " ... ";
  for (int i=0; i<n; ++i) {
    double value = w[n-1-i];
    int j=range.first;
    for (; j<range.second; ++j)
      if (weights[j] == value)
	break;
    os << " " << (*this)[j-range.first].name() << " " << 400*log10(value);
  }  
  os << "\n";
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
