#include <iostream>
#include <algorithm>
#include "osl/record/record.h"
#include "osl/record/compactBoard.h"

namespace osl
{
  namespace record
  {

    struct opiece_sort
    { 
      bool operator()(const OPiece& l, const OPiece& r)
      {
	// need to special case pieces on stand
	if (l.getPosition() == Position::STAND() || r.getPosition() == Position::STAND())
	  {
	    if (l.getPosition() == Position::STAND() && r.getPosition() != Position::STAND())
	      return true;
	    else if (l.getPosition() != Position::STAND() && r.getPosition() == Position::STAND())
	      return false;
	    else
	      {
		if (l.getOwner() != r.getOwner())
		  return l.getOwner() == WHITE;
		return l.getPtype() < r.getPtype();
	      }
	  }
	else
	  {
	    if (l.getPosition().x() < r.getPosition().x())
	      return true;
	    else if (l.getPosition().x() > r.getPosition().x())
	      return false;
	    else
	      {
		if (l.getPosition().y() <= r.getPosition().y())
		  return true;
		else
		  return false;
	      }
	  }
      }
    };

    CompactBoard::CompactBoard(const SimpleState& state)
    {
      pieces.reserve(40);
      for (int i = 0; i < 40; i++)
      {
	pieces.push_back(OPiece(state.getPieceOf(i)));
      }
      std::sort(pieces.begin(), pieces.end(), opiece_sort());
      turn = state.getTurn();
    }

    SimpleState
    CompactBoard::getState()
    {
      assert(pieces.size() == 40);

      SimpleState state;
      state.init();

      for (unsigned int i = 0; i < pieces.size(); i++)
      {
	const OPiece& p = pieces[i];
	state.setPiece(p.getOwner(), p.getPosition(), p.getPtype());
      }
      state.setTurn(turn);

      return state;
    }

    std::ostream& operator<<(std::ostream& os, const CompactBoard& c)
    {
      assert(c.pieces.size() == 40);

      for (unsigned int i = 0; i < c.pieces.size(); i++)
      {
	writeInt(os, static_cast<int>(c.pieces[i]));
      }
      writeInt(os, static_cast<int>(c.turn));
      return os;
    }

    std::istream& operator>>(std::istream& is, CompactBoard& c)
    {
      assert(c.pieces.size() == 0);

      for (unsigned int i = 0; i < 40; i++)
      {
	c.pieces.push_back(OPiece(readInt(is)));
      }
      c.turn = static_cast<Player>(readInt(is));
      return is;
    }
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
