/* usi.cc
 */
#include "osl/record/usi.h"
#include "osl/record/psn.h"
#include "osl/state/simpleState.h"
#include "osl/apply_move/applyMove.h"
#include <iostream>
#include <sstream>
#include <cctype>

const std::string osl::record::usi::
show(Move m)
{
  return psn::show(m);
}

const osl::Move osl::record::usi::
strToMove(const std::string& str, const SimpleState& s)
{
  return psn::strToMove(str, s);
}

osl::PtypeO osl::record::usi::
charToPtypeO(char c)
{
  const Ptype ptype = psn::charToPtype(toupper(c));
  const Player pl = isupper(c) ? BLACK : WHITE;
  return newPtypeO(pl, ptype);
}

void osl::record::usi::parseBoard(const std::string& word, SimpleState& state)
{
  state.init();
  int x=9, y=1;
  for (size_t i=0; i<word.size(); ++i) {
    const char c = word[i];
    if (isalpha(c)) {
      const PtypeO ptypeo = charToPtypeO(c);
      state.setPiece(getOwner(ptypeo), Position(x,y), getPtype(ptypeo));
      --x;
    } else if (c == '/') {
      assert(x == 0);
      x = 9;
      ++y;
    } else if (isdigit(c)) {
      const int n = c - '0';
      x -= n;
      assert(x >= 0);
    } else {
      std::cerr << "usi: unknown input " << c << "\n";
    }
  }
}

void osl::record::usi::parse(const std::string& line, SimpleState& state)
{
  std::istringstream is(line);
  std::string word;
  is >> word;
  if (word == "startpos") 
    state.init(HIRATE);
  else {
    assert(word == "sfen");
    is >> word;
    parseBoard(word, state);
    is >> word;
    assert(word == "b" || word == "w");
    state.setTurn((word == "b") ? BLACK : WHITE);
    is >> word;
    if (word != "-") {
      int prefix = 1;
      for (size_t i=0; i<word.size(); ++i) {
	if (isalpha(word[i])) {
	  PtypeO ptypeo = charToPtypeO(word[i]);
	  for (int j=0; j<prefix; ++j)
	    state.setPiece(getOwner(ptypeo), Position::STAND(), getPtype(ptypeo));
	  prefix = 1;
	}
	else {
	  assert(isdigit(word[i]));
	  prefix = word[i] - '0';
	}
      }
    }
    int move_number;
    is >> move_number;		// ignore
    assert(is);
  }
  if (! (is >> word))
    return;
  assert(word == "moves");
  while (is >> word) {
    Move m = strToMove(word, state);
    ApplyMoveOfTurn::doMove(state, m);
  }
} 

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
