/* searchMoveSorter.cc
 */
#include "osl/search/searchMoveSorter.h"
#include "osl/record/csa.h"
#include <iostream>

template <osl::Player P>
osl::search::
SearchMoveSorter<P>::SearchMoveSorter()
{
}
  
template <osl::Player P>
osl::search::
SearchMoveSorter<P>::~SearchMoveSorter()
{
}

template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::insert(const pair_t& data)
{
  assert(data.second->getLogProb() < 2000);
  assert(data.second->getMove().isValid());
  assert(! data.second->getMove().isPass());
  base_t::insert(data);
}

template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::insert(typename base_t::iterator hint, const pair_t& data)
{
  base_t::insert(hint, data);
}

template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::insert(typename base_t::iterator first,
			    typename base_t::iterator last)
{
  base_t::insert(first, last);
}
  
template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::erase(typename base_t::iterator first, typename base_t::iterator last)
{
  base_t::erase(first, last);
}
  
template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::clear()
{
  base_t::clear();
}

template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::dump(size_t show_elements) const
{
  if ((show_elements == 0)
      || (show_elements > size()))
    show_elements = size();
    
  size_t i=0;
  for (typename base_t::const_iterator p=begin(); i<show_elements; ++p, ++i)
  {
    assert(p != end());
    std::cerr << '+' << i << "\t" << p->first << "\t"
	      << record::csa::show(p->second->getMove());
    std::cerr << " " << p->second->getLogProb() 
#ifndef NDEBUG
	      << "\t" << p->second->record << " " << p->second 
#endif
	      << "\n";
    assert(p->second->getMove().isValid());
  }
}

template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::summary(size_t show_elements) const
{
  if ((show_elements == 0)
      || (show_elements > size()))
    show_elements = size();
    
  size_t i=0;
  for (typename base_t::const_iterator p=begin(); i<show_elements; ++p, ++i)
  {
    assert(p != end());
    if (p == begin())
      std::cerr << p->first << " ";
    else
      std::cerr << "/";
    std::cerr << record::csa::show(p->second->getMove());
    assert(p->second->getMove().isValid());
  }
  std::cerr << std::endl;
}

template <osl::Player P>
void osl::search::
SearchMoveSorter<P>::swap(SearchMoveSorter& o)
{
  base_t::swap(o);
}

template <osl::Player P>
bool osl::search::
SearchMoveSorter<P>::isValidAll() const
{
  int i=0;
  for (typename base_t::const_iterator p=begin(); p!=end(); ++p,++i)
  {
    if (! p->second->getMove().isValid())
    {
      std::cerr << "E " << i << "\t" << p->first << "\t"
		<< record::csa::show(p->second->getMove());
      std::cerr << " " << p->second->getLogProb() 
		<< " " << p->second->record<< "\n";
      return false;
    }
  }
  return true;
}
namespace osl
{
  namespace search
  {
    // explicit template instantiation
    template class SearchMoveSorter<BLACK>;
    template class SearchMoveSorter<WHITE>;
  } // namespace search
} // namespace osl

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
