/* searchState.cc
 */
#include "osl/search/searchState.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/search/sacrificeCheck.h"
#include "osl/record/csa.h"
#include "osl/misc/realTime.h"
#include <iostream>

osl::search::
SearchStateShared::SearchStateShared()
{
#ifdef OSL_SMP_DEBUG
  std::cerr << ">> SearchStateShared " << this << "\n";
#endif
}

osl::search::
SearchStateShared::~SearchStateShared()
{
#ifdef OSL_SMP_DEBUG
  std::cerr << "<< SearchStateShared " << this << "\n";
#endif
}


osl::search::
SearchStateCore::SearchStateCore(const NumEffectState& s, checkmate_t& c)
  : current_state(s), checkmate_searcher(&c), worker_cache(0),
    current_path(s.getTurn()), root_depth(0), 
    stop_flag(Alarm::UNAVAILABLE)
{
  setState(s);			// initialize shared
  assert(hasLastRecord());
}
osl::search::
SearchStateCore::~SearchStateCore()
{
}

void osl::search::
SearchStateCore::setTimeOut(int seconds)
{
  if (seconds <= 0)
  {
    if (seconds < 0)
      std::cerr << "warning: setTimeOut negative " << seconds << "\n";
    seconds = 1;
  }
  const bool success = Alarm::set(seconds, AlarmSwitch(&stop_flag));
  if (! success)
  {
    std::cerr << "warning: search without alarm\n";
  }
}
void osl::search::
SearchStateCore::resetTimeOut()
{
  Alarm::reset(AlarmSwitch(&stop_flag));
}

void osl::search::
SearchStateCore::throwTimeOut()
{
  resetTimeOut();
  throw misc::NoMoreTime();
}

void osl::search::
SearchStateCore::setState(const NumEffectState& s)
{
  if (&current_state != &s)
    current_state = s;
  current_path = PathEncoding(s.getTurn(), move_history.size());
  killer_moves.clear();
  shared.reset(new SearchStateShared());
  repetition_counter.clear();
  const HashKey key = HashKey::calcHash(s);
  repetition_counter.push(key, s);
  move_history.clear();
  record_stack.clear();
  root_depth = 0;
}

void osl::search::
SearchStateCore::setHistory(const MoveStack& h)
{
  move_history = h;
  current_path = PathEncoding(current_path.turn(), h.size());
  root_depth = history().size();
}

void osl::search::
SearchStateCore::setBigramKillerMove(const BigramKillerMove& killers)
{
  shared.reset(new SearchStateShared());
  shared->bigram_killers = killers;
}

bool osl::search::
SearchStateCore::abort(Move best_move) const
{
  std::cerr << state();
  history().dump();
  const SimpleHashRecord *record = record_stack.lastRecord();
  std::cerr << "best move " << record::csa::show(best_move)
	    << "\n";
  std::cerr << "record " << record << "\n";
  if (record)
  {
    record->dump(std::cerr);
  }
  record_stack.dump();
  repetition_counter.history().dump();
  return false;
}

/* ------------------------------------------------------------------------- */

osl::search::
SearchState::SearchState(const NumEffectState& s, checkmate_t& c)
  : SearchStateCore(s, c), root_limit(0), cur_limit(0)
{
}

osl::search::
SearchState::~SearchState()
{
}

void osl::search::
SearchState::setState(const NumEffectState& s)
{
  SearchStateCore::setState(s);
  root_limit = cur_limit = 0;
}

int osl::search::
SearchState::countSacrificeCheck2(int history_max) const
{
  return SacrificeCheck::count2(recordHistory(), history(), history_max);
}

bool osl::search::
SearchState::abort(Move best_move) const
{
  std::cerr << "cur limit " << cur_limit
	    << " root limit " << root_limit << "\n";
  SearchStateCore::abort(best_move);
  return false;
}

void osl::search::
SearchState::checkPointSearchAllMoves()
{
  // debug code can be written here
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
