/* searchState2.cc
 */
#include "osl/search/searchState2.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/search/sacrificeCheck.h"
#include "osl/record/csa.h"
#include "osl/misc/realTime.h"
#include <iostream>

/* ------------------------------------------------------------------------- */
osl::search::
RecordStack2::RecordStack2()
{
  clear();
}

void osl::search::
RecordStack2::clear()
{
  data.clear();
  data.push_back(0);
}

void osl::search::
RecordStack2::dump() const 
{
  std::cerr << "RecordStack\n";
  for (size_t i=0; i<data.size(); ++i) {
    std::cerr << data[i];
    std::cerr << std::endl;
  }
}

/* ------------------------------------------------------------------------- */

osl::search::
SearchState2Shared::SearchState2Shared()
{
}

osl::search::
SearchState2Shared::~SearchState2Shared()
{
}


osl::CArray<int, osl::search::SearchState2Core::MaxDepth> 
osl::search::SearchState2Core::depth_node_count_quiesce;

osl::search::
SearchState2Core::SearchState2Core(const NumEffectState& s, checkmate_t& c)
  : current_state(s), checkmate_searcher(&c), 
    current_path(s.getTurn()), root_depth(0), 
    stop(false)
{
  setState(s);			// initialize shared
  assert(hasLastRecord());
}
osl::search::
SearchState2Core::~SearchState2Core()
{
}

void osl::search::
SearchState2Core::setState(const NumEffectState& s)
{
  if (&current_state != &s)
    current_state = s;
  current_path = PathEncoding(s.getTurn(), move_history.size());
  shared.reset(new SearchState2Shared());
  repetition_counter.clear();
  const HashKey key = HashKey::calcHash(s);
  repetition_counter.push(key, s);
  move_history.clear();
  record_stack.clear();
  root_depth = 0;
}

void osl::search::
SearchState2Core::setHistory(const MoveStack& h)
{
  move_history = h;
  current_path = PathEncoding(current_path.turn(), h.size());
  root_depth = history().size();
}

void osl::search::
SearchState2Core::setBigramKillerMove(const BigramKillerMove& killers)
{
  shared.reset(new SearchState2Shared());
  shared->bigram_killers = killers;
}

bool osl::search::
SearchState2Core::abort() const
{
  return abort(Move());
}

bool osl::search::
SearchState2Core::abort(Move best_move) const
{
  std::cerr << state();
  history().dump();
  const SimpleHashRecord *record = record_stack.lastRecord();
  std::cerr << "best move " << record::csa::show(best_move)
	    << "\n";
  std::cerr << "record " << record << "\n";
  if (record)
  {
    record->dump(std::cerr);
  }
  record_stack.dump();
  repetition_counter.history().dump();
  return false;
}

void osl::search::SearchState2Core::makePV(MoveVector& parent, Move m, MoveVector& pv)
{
  parent.clear();
  parent.push_back(m);
  parent.push_back(pv.begin(), pv.end());
}

/* ------------------------------------------------------------------------- */

osl::search::
SearchState2::SearchState2(const NumEffectState& s, checkmate_t& c)
  : SearchState2Core(s, c), root_limit(0), cur_limit(0)
{
}

osl::search::
SearchState2::~SearchState2()
{
}

void osl::search::
SearchState2::setState(const NumEffectState& s)
{
  SearchState2Core::setState(s);
  root_limit = cur_limit = 0;
}

int osl::search::
SearchState2::countSacrificeCheck2(int history_max) const
{
  return SacrificeCheck::count2(recordHistory(), history(), history_max);
}

bool osl::search::
SearchState2::abort(Move best_move) const
{
  std::cerr << "cur limit " << cur_limit
	    << " root limit " << root_limit << "\n";
  SearchState2Core::abort(best_move);
  return false;
}

void osl::search::
SearchState2::checkPointSearchAllMoves()
{
  // debug code can be written here
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
