/* TwoDimensionalStatistics.cc
 */
#include "osl/stat/twoDimensionalStatistics.h"
#include "osl/stat/average.h"
#include <deque>
#include <cmath>

class osl::stat::TwoDimensionalStatistics::Data 
{
  typedef std::deque<std::pair<double,double> > deque_t;
  deque_t m_deque;
  Average m_x, m_y;
public:
  // MANIPULATORS
  void add(const double& x, const double& y) 
  {
    m_deque.push_back(std::make_pair(x,y));
    m_x.add(x);
    m_y.add(y);
  }
  void merge(const Data& r)
  {
    m_x.merge(r.m_x);
    m_y.merge(r.m_y);
    m_deque.insert(m_deque.end(), r.m_deque.begin(), r.m_deque.end());
  }
  void clear()
  {
    m_deque.clear();
    m_x.clear();
    m_y.clear();
  }
  // ACCESSORS
  double x(size_t i) const { return m_deque.at(i).first; }
  double y(size_t i) const { return m_deque.at(i).second; }
  double averageX() const { return m_x.getAverage(); }
  double averageY() const { return m_y.getAverage(); }
  size_t size() const { return m_deque.size(); }
};

// CREATORS
osl::stat::TwoDimensionalStatistics::
TwoDimensionalStatistics() : m_data(new Data())
{
}

osl::stat::TwoDimensionalStatistics::
~TwoDimensionalStatistics()
{
}

// MANIPULATORS
void osl::stat::TwoDimensionalStatistics::
addInstance(const double& x, const double& y)
{
  m_data->add(x,y);
}

void osl::stat::TwoDimensionalStatistics::
merge(const TwoDimensionalStatistics& r)
{
  m_data->merge(*r.m_data);
}

void osl::stat::TwoDimensionalStatistics::
clear()
{
  m_data->clear();
}


// ACCESSORS
size_t osl::stat::TwoDimensionalStatistics::
size() const
{
  return m_data->size();
}

double osl::stat::TwoDimensionalStatistics::
averageX() const
{
  return m_data->averageX();
}

double osl::stat::TwoDimensionalStatistics::
averageY() const
{
  return m_data->averageY();
}

double osl::stat::TwoDimensionalStatistics::
meanSquaredErrors() const
{
  // TODO: make it incremental
  Average squared_error;
  for (size_t i=0; i<size(); ++i)
  {
    const double error = m_data->x(i) - m_data->y(i);
    squared_error.add(error*error);
  }
  return squared_error.getAverage();
}

double osl::stat::TwoDimensionalStatistics::
meanSquaredErrorsAdjustConstant() const
{
  // TODO: make it incremental
  Average squared_error;
  for (size_t i=0; i<size(); ++i)
  {
    const double error = (m_data->x(i)-averageX()) - (m_data->y(i)-averageY());
    squared_error.add(error*error);
  }
  return squared_error.getAverage();
}

double osl::stat::TwoDimensionalStatistics::
correlation() const
{
  /** variance */
  Average xy, x, y;
  for (size_t i=0; i<size(); ++i)
  {
    const double xdiff = m_data->x(i)-averageX();
    const double ydiff = m_data->y(i)-averageY();
    xy.add(xdiff*ydiff);
    x.add(xdiff*xdiff);
    y.add(ydiff*ydiff);
  }
  return xy.getAverage() / sqrt(x.getAverage()*y.getAverage());
}



/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
