/* simpleState.cc
 */

#include "osl/state/simpleState.h"
#include "osl/state/simpleState.tcc"
#include "osl/effect_action/checkMultiple.h"
#include "osl/effect_action/getPiece.h"
#include "osl/apply_move/doUndoMoveLock.h"
#include "osl/apply_move/doUndoMoveLock.tcc"
#include "osl/apply_move/applyMove.h"
#include "osl/record/csa.h"
#include "osl/pieceTable.h"
#include "osl/pieceStand.h"
#include <iostream>
#include <stdexcept>

namespace osl 
{
  template bool SimpleState:: 
  hasEffectByWithRemove<BLACK>(Position, Position) const;
  template bool SimpleState:: 
  hasEffectByWithRemove<WHITE>(Position, Position) const;
  template bool SimpleState:: 
  hasEffectNotBy<BLACK>(Piece, Position) const;
  template bool SimpleState:: 
  hasEffectNotBy<WHITE>(Piece, Position) const;
  template const Piece SimpleState::
  hasEffectPiece<BLACK>(Position) const;
  template const Piece SimpleState::
  hasEffectPiece<WHITE>(Position) const;
  /**
   * 餫ϿPositionȰפemptyȸʤ functor
   */
  class PositionIsEmpty{
    Position position;
  public:
    explicit PositionIsEmpty(Position position) : position(position) {}
    bool operator()(Position pos) const{ return position==pos; }  
  };
  
}

osl::SimpleState::SimpleState() {
  init();
}

osl::SimpleState::SimpleState(Handicap h) {
  init(h);
}

const osl::Piece osl::SimpleState::hasEffectPiece(Player pl,Position target) const
{
  if (pl==BLACK) return hasEffectPiece<BLACK>(target);
  else return hasEffectPiece<WHITE>(target);
}
template <osl::Player P>
const osl::Piece osl::SimpleState::hasEffectPiece(Position target) const
{
  assert(target.isOnBoard());
  Piece ret=Piece::EMPTY();
  typedef effect_action::GetPiece<SimpleState> action_t;
  action_t getPieceAction(ret);
  forEachEffectWithCheck<P,action_t,true,NotEmpty>(target,getPieceAction,NotEmpty());
  return ret;
}

template <osl::Player P>
bool osl::SimpleState::hasMultipleEffectBy(Position target) const 
{
  Piece attackerPiece=Piece::EMPTY();
  if (!hasEffectBy<P>(target,attackerPiece)) 
    return false;
  return attackerPiece==Piece::EMPTY();
}
bool osl::SimpleState::hasMultipleEffectBy(Player player,Position target) const 
{
  if(player==BLACK)
    return hasMultipleEffectBy<BLACK>(target);
  else
    return hasMultipleEffectBy<WHITE>(target);
}

template <osl::Player P>
bool osl::SimpleState::hasEffectBy(Position target,Piece& attackerPiece) const 
{
  attackerPiece=Piece::EMPTY();
  typedef effect_action::CheckMultiple<SimpleState> action_t;
  action_t checkMultipleAction(attackerPiece);
  forEachEffectWithCheck<P,action_t,true,NotEmpty>(target,checkMultipleAction,NotEmpty());
  switch(checkMultipleAction.count)
  {
  case 0: return false;
  case 1: return true;
  case 2: attackerPiece=Piece::EMPTY(); return true;
  default: 
    ;
  }
  assert(0);
  return true;		// to prevent useless warning
}

template <osl::Player P>
bool osl::SimpleState::
hasEffectByWithRemove(Position target,Position removed) const
{
  Piece ret=Piece::EMPTY();
  typedef effect_action::GetPiece<SimpleState> action_t;
  action_t getPieceAction(ret);
  forEachEffectWithCheck<P,action_t,true,PositionIsEmpty>(target,getPieceAction,PositionIsEmpty(removed));
  return ret!=Piece::EMPTY();
}

template <osl::Player P>
bool osl::SimpleState::hasEffectNotBy(Piece piece,Position target) const 
{
  Piece ret=Piece::EMPTY();
  typedef effect_action::GetPiece<SimpleState> action_t;
  action_t getPieceAction(ret);
  forEachEffectNotBy<P,action_t>(target,piece,getPieceAction);
  return ret!=Piece::EMPTY();
}

void osl::SimpleState::init() {
  turn=BLACK;
  for (int ipos=0;ipos<Position::SIZE;ipos++) {
    setBoard(Position::nth(ipos),Piece::EDGE());
  }
  for (int y=1;y<=9;y++)
    for (int x=9;x>0;x--) {
      setBoard(Position(x,y),Piece::EMPTY());
    }
  //  promoteMask.clearAll();
  stand_mask[playerToIndex(BLACK)].resetAll();
  stand_mask[playerToIndex(WHITE)].resetAll();
  used_mask.resetAll();
}
  

void osl::SimpleState::init(Handicap h) {
  init();
  if (h != HIRATE) {
    std::cerr << "unsupported handicap\n";
    throw std::runtime_error("unsupported handicap");
  }
  // 
  for (int x=9;x>0;x--) {
    setPiece(BLACK,Position(x,7),PAWN);
    setPiece(WHITE,Position(x,3),PAWN);
  }
  // 
  setPiece(BLACK,Position(1,9),LANCE);
  setPiece(BLACK,Position(9,9),LANCE);
  setPiece(WHITE,Position(1,1),LANCE);
  setPiece(WHITE,Position(9,1),LANCE);
  //
  setPiece(BLACK,Position(2,9),KNIGHT);
  setPiece(BLACK,Position(8,9),KNIGHT);
  setPiece(WHITE,Position(2,1),KNIGHT);
  setPiece(WHITE,Position(8,1),KNIGHT);
  //
  setPiece(BLACK,Position(3,9),SILVER);
  setPiece(BLACK,Position(7,9),SILVER);
  setPiece(WHITE,Position(3,1),SILVER);
  setPiece(WHITE,Position(7,1),SILVER);
  //
  setPiece(BLACK,Position(4,9),GOLD);
  setPiece(BLACK,Position(6,9),GOLD);
  setPiece(WHITE,Position(4,1),GOLD);
  setPiece(WHITE,Position(6,1),GOLD);
  //
  setPiece(BLACK,Position(5,9),KING);
  setPiece(WHITE,Position(5,1),KING);
  //
  setPiece(BLACK,Position(8,8),BISHOP);
  setPiece(WHITE,Position(2,2),BISHOP);
  //
  setPiece(BLACK,Position(2,8),ROOK);
  setPiece(WHITE,Position(8,2),ROOK);
}
  

osl::SimpleState::~SimpleState() {}

void osl::SimpleState::setPiece(Player player,Position pos,Ptype ptype) {
  int num;
  for (num=0;num<40;num++) {
    if (!used_mask.test(num) && Piece_Table.getPtypeOf(num)==unpromote(ptype)
	&& (ptype!=KING || 
	    num==PtypeTraits<KING>::indexMin+playerToIndex(player))) {
      used_mask.set(num);
	
      Piece p(player,ptype,num,pos);
      setPieceOf(num,p);
      setBoard(pos,p);
      if (pos.isPieceStand())
	stand_mask[playerToIndex(player)].set(num);
      else{
      }
      return;
    }
  }
  std::cerr << "osl::SimpleState::setPiece! maybe too many pieces " 
	    << ptype << " " << pos << " " << player << "\n";
  abort();
}

void osl::SimpleState::setPieceAll(Player player) {
  for (int num=0;num<40;num++) {
    if (!used_mask.test(num)) {
      used_mask.set(num);
      stand_mask[playerToIndex(player)].set(num);
      Player pplayer = player;
      /* Ҷ̤ʤΤ */
      if (num==PtypeTraits<KING>::indexMin+playerToIndex(alt(player)))
      {
	pplayer=alt(player);
      }
      Piece p(pplayer,Piece_Table.getPtypeOf(num),num,Position::STAND());
      setPieceOf(num,p);
    }
  }
}
  
// check

bool osl::SimpleState::isConsistent(bool show_error) const
{
  // boardǤconsistency
  for (int y=1;y<=9;y++)
  {
    for (int x=9;x>=1;x--)
    {
      const Position pos(x,y);
      const Piece p0=getPieceAt(pos);
      if (p0.isPiece())
      {
	if (p0.position()!=pos)
	{
	  if (show_error) {
	    std::cerr << p0 << " must be put at " << pos << std::endl;
	  }
	  return false;
	}
	int num=p0.number();
	if (! PieceTable::validNumber(num) || !used_mask.test(num)) {
	  if (show_error) std::cerr << "NotUsed, num=" << num << std::endl;
	  return false;
	}
	Piece p1=getPieceOf(num);
	if (p0!=p1) {
	  if (show_error) std::cerr << "board[" << pos << "]!=" 
				    << "piece[" << num << "]" << std::endl;
	  return false;
	}
      }
    }
  }
  // piecesconsistency
  for (int num0=0; num0<Piece::SIZE; num0++)
  {
    if (isOnBoard(num0))
    {
      Piece p0=getPieceOf(num0);
      Ptype ptype=p0.ptype();
      if (unpromote(ptype)!=Piece_Table.getPtypeOf(num0)) {
	if (show_error) std::cerr << "ptype of piece[" << num0 << "]=" 
				  << ptype << std::endl;
	return false;
      }
      if (!p0.isOnBoard()) {
	if (show_error) std::cerr << "mochigoma[" << num0 << "]=true" << std::endl;
	return false;
      }
      Position pos=p0.position();
      if (!pos.isOnBoard()) {
	if (show_error) std::cerr << "position " << pos << " is not onboard" << std::endl;
	return false;
      }
      Piece p1=getPieceAt(pos);
      int num1=p1.number();
      if (num0 !=num1) {
	if (show_error) std::cerr << "pieces[" << num0 << "]=" << p0 << ",board[" << pos << "] is " << p1 << std::endl;
	return false;
      }
    }
    else
    {
      Piece p0=getPieceOf(num0);
      Ptype ptype=p0.ptype();
      if (p0.number()!=num0) {
	if (show_error) 
	  std::cerr << "pieces[" << num0 << "] (" 
		    << Piece_Table.getPtypeOf(num0)  <<  ") ="
		    << p0 << std::endl;
	return false;
	  
      }
      if (ptype!=Piece_Table.getPtypeOf(num0)) {
	if (show_error) std::cerr << "ptype of piece[" << num0 << "]=" 
				  << ptype << std::endl;
	return false;
      }
      if (! p0.position().isPieceStand()) {
	if (show_error) std::cerr << p0 << " must be offboard" << std::endl;
	return false;
      }
    }
  }
  return true;
}

bool osl::SimpleState::isAlmostValidMove(Move move,bool show_error) const
{
  if (show_error)
  {
    const bool valid = isAlmostValidMove<true>(move);
    if (! valid)
      std::cerr << *this << " " << move << std::endl;
    return valid;
  }
  else
    return isAlmostValidMove<false>(move);
}

template <bool show_error>
bool osl::SimpleState::isAlmostValidMove(Move move) const
{
  assert(move.isValid());
  assert(getTurn() == move.player());
  assert(isValidMoveByRule(move, true));

  const Position from=move.from();
  const Position to=move.to();
  const Piece toPiece=getPieceAt(to);
  const Ptype ptype=move.ptype();
  const Player turn = move.player();
    
  if (from.isPieceStand()) // Ǥļ
  {      
    // åȤ
    if (! toPiece.isEmpty()) {
      if (show_error) std::cerr << "drop on to piece : " << move << std::endl;
      return false;
    }
    // ⤽⤽ζäƤ뤫?
    if (! hasPieceOnStand(turn,ptype)) {
      if (show_error) std::cerr << turn << " don't have : " << ptype << std::endl;
      return false;
    }
    // Υå
    if (ptype==PAWN && isPawnMaskSet(turn, to.x())) {
      if (show_error) std::cerr << " Double Pawn : " << move << std::endl;
      return false;
    }
  }
  else
  {
    const Piece from_piece = getPieceAt(from);
    // fromˤΤζ
    if (from_piece.isEmpty() 
	|| (from_piece.owner() != turn))
    {
      if (show_error) 
	std::cerr << " No such piece0 : " << move << std::endl;
      return false;
    }
    // promoteƤpromoteǽ
    if (move.isPromote())
    {
      // fromˤΤζ
      if (from_piece.ptype() != unpromote(move.ptype()))
      {
	if (show_error) 
	  std::cerr << " No such piece1  : " << move << std::endl;
	return false;
      }
      if (from_piece.isPromotedNotKingGold())
      {
	if (show_error) 
	  std::cerr << " can't promote promoted piece : " << move << std::endl;
	return false;
      }
    }
    else
    {
      // fromˤΤζ
      if (from_piece.ptype() != move.ptype())
      {
	if (show_error) 
	  std::cerr << " No such piece2  : " << move << std::endl;
	return false;
      }
    }
    // toˤΤζ𤫶?
    if (!toPiece.isEmpty() && toPiece.owner()==turn) {
      if (show_error) std::cerr << " No move on  : " << move << std::endl;
      return false;
    }
    //  offsetưptype˴ؤvalid?
    EffectContent effect=Ptype_Table.getEffect(from_piece.ptypeO(),from,to);
    if (!effect.hasUnblockableEffect())
    {
      const Offset o=effect.offset();
      if (o.zero()) {
	if (show_error) {
	  std::cerr << " No such move2 : " << move << std::endl;
	}
	return false;
      }
      // Υ줿ưλ˴֤Ƥ뤫?
      for (Position p=from+o;p!=to;p+=o) {
	if (! getPieceAt(p).isEmpty()) {
	  if (show_error) 
	    std::cerr << " Not space to move : " << move << std::endl;
	  return false;
	}
      }
    }
    // capturePtypeפƤ뤫?
    if (toPiece.ptype()!=move.capturePtype()) {
      if (show_error) std::cerr << " Not such capture : " << move 
				<< std::endl << *this;
      return false;
    }
  }

  assert(isValidMoveByRule(move, true));
  return true;
}

bool osl::SimpleState::isValidMoveByRule(Move move,bool show_error) const
{
  assert(move.isNormal());
  const Position from=move.from();
  const Position to=move.to();
  const Ptype ptype=move.ptype();
  const Player turn = move.player();
    
  if (from.isPieceStand()) // Ǥļ
  { 
    // ưʤǤϤʤ?
    if (! Ptype_Table.canDropTo(turn,ptype,to))
    {
      if (show_error) std::cerr << " can't drop to : " << move << std::endl;
      return false;
    }
  }
  else
  {
    const PtypeO old_ptypeo = move.oldPtypeO();
    const EffectContent effect
      = Ptype_Table.getEffect(old_ptypeo, Offset32(to,from));
    //  offsetưptype˴ؤvalid?
    if (!effect.hasUnblockableEffect())
    {
      const Offset o = effect.offset();
      if (o.zero()) {
	if (show_error) {
	  std::cerr << " No such move1 : " << move << std::endl;
	}
	return false;
      }
    }
    // promoteƤpromoteǽ
    if (move.isPromote())
    {
      if (! (canPromote(unpromote(move.ptype()))
	     && (to.canPromote(move.player()) 
		 || from.canPromote(move.player()))))
      {
	if (show_error) 
	  std::cerr << " illegal promote type or position : " << move << std::endl;
	return false;
      }
    }
    // promoteƤʤ˶promoteǤʤ?
    if ((! isPromoted(ptype)
	 && ! Ptype_Table.canDropTo(turn,getPtype(old_ptypeo),to)) 
	&& !move.isPromote()) 
    {
      if (show_error) 
	std::cerr << " must promote to this position : " << move << std::endl;
      return false;
    }
  }
  return true;
}

bool osl::SimpleState::isValidMove(Move move,bool show_error) const
{
  if (getTurn() != move.player()) {
    if (show_error) {
      std::cerr << "invalid player move : " << move << std::endl;
      std::cerr << *this;
    }
    return false;
  }
  if (! isValidMoveByRule(move, show_error))
    return false;
  return isAlmostValidMove(move, show_error);
}
  
bool osl::SimpleState::dump() const
{
  return std::cerr << *this << "\n";
}
  
void osl::SimpleState::doSimpleMove(Position from, Position to, int promoteMask)
{
  Piece oldPiece;
  int num;
  if(getTurn()==BLACK) {
    apply_move::ApplyDoUndoSimpleMove<BLACK,SimpleState>::
      prologue(*this, from, to, promoteMask, oldPiece, num);
  }
  else{
    apply_move::ApplyDoUndoSimpleMove<WHITE,SimpleState>::
      prologue(*this, from, to, promoteMask, oldPiece, num);
  }
}

void osl::SimpleState::doDropMove(Position to,Ptype ptype)
{
  Piece oldPiece;
  int num, numIndex, numLow;
  if(getTurn()==BLACK) {
    apply_move::ApplyDoUndoDropMove<BLACK,SimpleState>::
      prologue(*this, ptype, to, oldPiece, num, numIndex, numLow);
  }
  else{
    apply_move::ApplyDoUndoDropMove<WHITE,SimpleState>::
      prologue(*this, ptype, to, oldPiece, num, numIndex, numLow);
  }
}

void osl::SimpleState::doCaptureMove(Position from, Position to, Piece target,int promoteMask)

{
  Piece oldPiece;
  int num0, num1;
  int num1Index;
  mask_t num1Mask;
  Ptype capturePtype;
  if(getTurn()==BLACK) {
    apply_move::ApplyDoUndoCaptureMove<BLACK,SimpleState>::
      prologue(*this, from, to, target, promoteMask,
	       capturePtype, oldPiece, num0, num1, num1Index, num1Mask);
  }
  else{
    apply_move::ApplyDoUndoCaptureMove<WHITE,SimpleState>::
      prologue(*this, from, to, target, promoteMask,
	       capturePtype, oldPiece, num0, num1, num1Index, num1Mask);
  }
}

/**
 * from ɽ줿Piecenew_ownerλˤ̤.
 */
const osl::SimpleState 
osl::SimpleState::emulateCapture(Piece from, Player new_owner) const {
  osl::SimpleState newState;
  for(int i=0;i<40;i++){
    Piece p=getPieceOf(i);
    if(p==from){
      newState.setPiece(new_owner,Position::STAND(),unpromote(p.ptype()));
    }
    else{
      newState.setPiece(p.owner(),p.position(),p.ptype());
    }
  }
  return newState;
}

/**
 * from to  ptypeλϤ̤.
 */
const osl::SimpleState 
osl::SimpleState::emulateHandPiece(Player from, Player to, Ptype ptype) const {
  assert(hasPieceOnStand(from, ptype));
  assert(from==alt(to));
  osl::SimpleState newState;
  bool done=false;
  for(int i=0;i<40;i++){
    Piece p=getPieceOf(i);
    if(!done &&
       p.owner()==from &&
       !p.isOnBoard() &&
       p.ptype()==ptype){
      newState.setPiece(to,Position::STAND(),ptype);
      done=true;
    }
    else{
      newState.setPiece(p.owner(),p.position(),p.ptype());
    }
  }
  assert(done);
  return newState;
}


bool osl::state::operator==(const SimpleState& st1,const SimpleState& st2)
{
  assert(st1.isConsistent(false));
  assert(st2.isConsistent(false));
  if (st1.turn!=st2.turn) 
    return false;
  for (int y=1;y<=9;y++)
    for (int x=9;x>0;x--) {
      Piece p1=st1.getPieceAt(Position(x,y));
      Piece p2=st2.getPieceAt(Position(x,y));
      if (p1.ptypeO()!=p2.ptypeO()) return false;
    }
  return true;
      
}

namespace osl
{
  namespace 
  {
    void showStand(std::ostream& os, Player player, PieceStand stand)
    {
      using namespace osl::record;
      if (! stand.any())
	return;
      
      os << "P" << csa::show(player);
      for (unsigned int i=0; i<PieceStand::order.size(); ++i)
      {
	const Ptype ptype = PieceStand::order[i];
	for (unsigned int j=0; j<stand.get(ptype); ++j)
	{
	  os << "00" << csa::show(ptype);
	}
      }
      os << "\n";
    }
  } // anonymous namespace
} // namespace osl

std::ostream& osl::state::operator<<(std::ostream& os,const SimpleState& state)
{
  for (int y=1;y<=9;y++) {
    os << 'P' << y;  
    for (int x=9;x>0;x--) {
      csaShow(os,state.getPieceOnBoard(Position(x,y)));
    }
    os << std::endl;
  }
  // ɽ
  const PieceStand black_stand(BLACK, state);
  const PieceStand white_stand(WHITE, state);
  showStand(os, BLACK, black_stand);
  showStand(os, WHITE, white_stand);
  
  os << state.getTurn() << std::endl;
  return os;
}

namespace osl
{
  namespace apply_move
  {
  template <>
  class DoUndoMoveLockSimple<SimpleState> : public DoUndoMoveLockCleanUp
  {
    SimpleState& state;
    Position from, to;
    Piece oldPiece;
    int num;
  public:
    DoUndoMoveLockSimple(SimpleState& s, 
			 Position f, Position t, int promoteMask);
    ~DoUndoMoveLockSimple();
  };

  template <>
  class DoUndoMoveLockDrop<SimpleState> : public DoUndoMoveLockCleanUp
  {
    SimpleState& state;
    Position to;
    Piece oldPiece;
    int num, numIndex, numLow;
  public:
    DoUndoMoveLockDrop(SimpleState& state, 
		       Position to, Ptype ptype);
    ~DoUndoMoveLockDrop();
  };
  
  template <>
  class DoUndoMoveLockCapture<SimpleState> : public DoUndoMoveLockCleanUp
  {
    SimpleState& state;
    Position from, to;
    Piece target;
    Piece oldPiece;
    int num0, num1;
    int num1Index;
    mask_t num1Mask;
    Ptype capturePtype;
  public:
    DoUndoMoveLockCapture(SimpleState& state, 
			  Position from, Position to, Piece target,
			  int promoteMask);
    ~DoUndoMoveLockCapture();
  };


  DoUndoMoveLockSimple<SimpleState>::
  DoUndoMoveLockSimple(SimpleState& s, 
		       Position f, Position t, int promoteMask)
    : state(s), from(f), to(t)
  {
    if(s.getTurn()==BLACK) {
      apply_move::ApplyDoUndoSimpleMove<BLACK,SimpleState>::
	prologue(state, from, to, promoteMask, oldPiece, num);
    }
    else{
      apply_move::ApplyDoUndoSimpleMove<WHITE,SimpleState>::
	prologue(state, from, to, promoteMask, oldPiece, num);
    }
  }

  DoUndoMoveLockSimple<SimpleState>::
  ~DoUndoMoveLockSimple()
  {
    if(state.getTurn()==BLACK) {
      apply_move::ApplyDoUndoSimpleMove<BLACK,SimpleState>::
	epilogue(state, from, to, oldPiece, num);
    }
    else{
      apply_move::ApplyDoUndoSimpleMove<WHITE,SimpleState>::
	epilogue(state, from, to, oldPiece, num);
    }
  }

  DoUndoMoveLockDrop<SimpleState>::
  DoUndoMoveLockDrop(SimpleState& s, Position t, Ptype ptype)
    : state(s), to(t)
  {
    if(state.getTurn()==BLACK) {
      apply_move::ApplyDoUndoDropMove<BLACK,SimpleState>::
	prologue(s, ptype, to, oldPiece, num, numIndex, numLow);
    }
    else{
      apply_move::ApplyDoUndoDropMove<WHITE,SimpleState>::
	prologue(s, ptype, to, oldPiece, num, numIndex, numLow);
    }
  }

  DoUndoMoveLockDrop<SimpleState>::
  ~DoUndoMoveLockDrop()
  {
    if(state.getTurn()==BLACK) {
      apply_move::ApplyDoUndoDropMove<BLACK,SimpleState>::
	epilogue(state, to, oldPiece, num, numIndex, numLow);
    }
    else{
      apply_move::ApplyDoUndoDropMove<WHITE,SimpleState>::
	epilogue(state, to, oldPiece, num, numIndex, numLow);
    }
  }

  DoUndoMoveLockCapture<SimpleState>::
  DoUndoMoveLockCapture(SimpleState& s, 
			Position f, Position t, Piece tg,
			int promoteMask)
    : state(s), from(f), to(t), target(tg)
  {
    if(state.getTurn()==BLACK) {
      apply_move::ApplyDoUndoCaptureMove<BLACK,SimpleState>::
	prologue(s, from, to, target, promoteMask,
		 capturePtype, oldPiece, num0, num1, num1Index, num1Mask);
    }
    else{
      apply_move::ApplyDoUndoCaptureMove<WHITE,SimpleState>::
	prologue(s, from, to, target, promoteMask,
		 capturePtype, oldPiece, num0, num1, num1Index, num1Mask);
    }
  }
  DoUndoMoveLockCapture<SimpleState>::
  ~DoUndoMoveLockCapture()
  {
    if(state.getTurn()==BLACK) {
      apply_move::ApplyDoUndoCaptureMove<BLACK,SimpleState>::
	epilogue(state, from, to, target, oldPiece, num0, num1, num1Index, num1Mask);
    }
    else{
      apply_move::ApplyDoUndoCaptureMove<WHITE,SimpleState>::
	epilogue(state, from, to, target, oldPiece, num0, num1, num1Index, num1Mask);
    }
  }

  template DoUndoMoveLock::DoUndoMoveLock(SimpleState&, Move);
} // namespace apply_move
} // namespace osl




// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
