#include "osl/apply_move/applyMove.h"
#include "osl/category/categoryEnv.h"
#include "osl/category/bigramPosition.h"
#include "osl/container/moveStack.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/state/numEffectState.h"
#include "osl/stl/vector.h"
#include "osl/record/csaString.h"
#include "osl/record/csa.h"
#include "osl/record/record.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>

namespace osl
{
class CategoryBigramTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(CategoryBigramTest);
  CPPUNIT_TEST(testGenerate);
  CPPUNIT_TEST(test5x5);
  CPPUNIT_TEST_SUITE_END();
 public:
  void testGenerate();
  void test5x5();
};
}

CPPUNIT_TEST_SUITE_REGISTRATION(osl::CategoryBigramTest);

void osl::CategoryBigramTest::testGenerate()
{
  osl::record::csa::CsaString csa("P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
				  "P2 * -HI *  *  *  *  * -KA * \n"
				  "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
				  "P4 *  *  *  *  *  *  *  *  * \n"
				  "P5 *  *  *  *  *  *  *  *  * \n"
				  "P6 *  *  *  *  *  *  *  *  * \n"
				  "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
				  "P8 * +KA *  *  *  *  * +HI * \n"
				  "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
				  "+\n"
				  "+7776FU\n"
				  "-3334FU\n"
				  "+6766FU\n"
				  "-8384FU\n");
  osl::state::SimpleState initialState = csa.getInitialState();
  osl::state::NumEffectState state(initialState);
  osl::stl::vector<osl::Move> moves = csa.getRecord().getMoves();
  osl::container::MoveStack history;

  for (size_t i = 0; i < moves.size(); i++)
  {
    osl::ApplyMoveOfTurn::doMove(state, moves[i]);
    history.push(moves[i]);
  }
  osl::container::MoveLogProbVector generatedMoves;
  osl::category::CategoryEnv env(&state,  1000, &history, Progress32(0), 0, 1);
  osl::category::CategoryEnv nEnv(&state, 1000, &history, Progress32(0), 0, 4);
  osl::category::BigramHistory::setupTable(initialState, state, history);

  osl::container::MoveLogProbVector directMoves;;
  osl::category::BigramHistory::generate(env, directMoves);
  directMoves.sortByProbability();

  CPPUNIT_ASSERT(directMoves.size() > 0);
  
  {
    std::string tmpMoves[] = {"+2726FU",
			      "-5354FU"};

    osl::state::NumEffectState tmpState(state);
    osl::container::MoveStack tmpHistory(history);
    // table ˤϤ 2726FU  invalid ˤʤä֤Ǥ
    // Ǥ뤫Ĵ٤
    for (size_t i = 0; i < 2; i++)
    {
      osl::Move move = osl::record::csa::strToMove(tmpMoves[i], tmpState);
      osl::ApplyMoveOfTurn::doMove(tmpState, move);
      tmpHistory.push(move);
    }
    osl::category::CategoryEnv tmpEnv(&tmpState, 1000, &tmpHistory, Progress32(0), 0, 3);

    osl::category::BigramHistory::generate(tmpEnv, generatedMoves);
    generatedMoves.clear();
  }

  osl::category::BigramHistory::generate(nEnv, generatedMoves);
  CPPUNIT_ASSERT(generatedMoves.size() == 0);

  std::string nextMoves[] = {"+7978GI",
			     "-5354FU",
			     "+2868HI",
			     "-7162GI",
			     "+5948OU",
			     "-5142OU",
			     "+4838OU",
			     "-4232OU",
			     "+3828OU",
			     "-8485FU",
			     "+3938GI",
			     "-6253GI"};

  for (size_t i = 0; i < 12; i++)
  {
    osl::Move move = osl::record::csa::strToMove(nextMoves[i], state);
    osl::ApplyMoveOfTurn::doMove(state, move);
    history.push(move);
  }
  osl::category::BigramHistory::setupTable(initialState, state, history);

  osl::container::MoveLogProbVector lastMoves;
  osl::category::BigramHistory::generate(env, lastMoves);

  CPPUNIT_ASSERT(lastMoves.size() > 0);
  for (size_t i = 0; i < lastMoves.size(); i++)
  {
    CPPUNIT_ASSERT(lastMoves[i].getLogProb() <= 500);
  }
}

void osl::CategoryBigramTest::test5x5()
{
  osl::record::csa::CsaString csa("P1-KY-KE *  *  *  * -GI-OU * \n"
				  "P2 *  *  *  *  *  * -KI *  * \n"
				  "P3-FU * -HI * -KA * -KE-FU-FU\n"
				  "P4 *  *  * -FU *  * -FU *  * \n"
				  "P5 * -FU *  * -KI *  *  * -KY\n"
				  "P6+FU *  *  * -FU-FU+FU+GI * \n"
				  "P7 * +FU+FU *  *  * +KE+FU * \n"
				  "P8 *  *  * +HI+KI * +GI *  * \n"
				  "P9+KY+KE *  *  * +KI+OU * -NY\n"
				  "P+00KA\n"
				  "P-00GI00FU00FU00FU00FU00FU\n"
				  "-\n"
				  "-0057GI\n");
  osl::state::SimpleState initialState = csa.getInitialState();
  osl::state::NumEffectState state(initialState);
  osl::stl::vector<osl::Move> moves = csa.getRecord().getMoves();
  osl::container::MoveStack history;

  for (size_t i = 0; i < moves.size(); i++)
  {
    osl::ApplyMoveOfTurn::doMove(state, moves[i]);
    history.push(moves[i]);
  }
  osl::container::MoveLogProbVector generatedMoves;
  osl::category::CategoryEnv env(&state, 1000, &history, Progress32(0), 0, 1);
  osl::category::BigramDefense::generate(env, generatedMoves);
  CPPUNIT_ASSERT_EQUAL((size_t)1, generatedMoves.size());
}
