/* breakThreatmate.t.cc
 */
#include "osl/category/breakThreatmate.h"
#include "osl/state/numEffectState.h"
#include "osl/record/csaString.h"
#include "osl/container/moveLogProbVector.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

namespace osl
{
  namespace category
  {
    class BreakThreatmateTest : public CppUnit::TestFixture 
    {
      CPPUNIT_TEST_SUITE(BreakThreatmateTest);
      CPPUNIT_TEST(testGenerate);
      CPPUNIT_TEST(testDropValidity);
      CPPUNIT_TEST_SUITE_END();
    public:
      void testGenerate();
      void testDropValidity();
    };
  }
}

CPPUNIT_TEST_SUITE_REGISTRATION(osl::category::BreakThreatmateTest);

void osl::category::BreakThreatmateTest::testDropValidity()
{
  {
    NumEffectState state(CsaString(
			   "P1+OU *  *  *  *  * -KI-KE-OU\n"
			   "P2+FU-FU-GI *  *  * -KI-GI-KY\n"
			   "P3 *  *  *  *  *  * -FU-FU-FU\n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  *  *  *  *  *  *  *  * \n"
			   "P+00KE00KY00FU\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    MoveLogProbVector moves;
    BreakThreatmate::generateBreak(500, state, Move(Position(8,1), ROOK, WHITE), 
				   moves);
    CPPUNIT_ASSERT(! moves.empty());    
    CPPUNIT_ASSERT(moves.size() == 3); // 73KE, 93KE, 82OU
  }
}

void osl::category::BreakThreatmateTest::testGenerate()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * -KI-KE-OU\n"
			   "P2 *  *  *  *  *  * -KI-GI-KY\n"
			   "P3 *  *  *  *  *  * -FU-FU-FU\n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7-FU *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9+OU *  *  *  *  *  *  *  * \n"
			   "P-00KI\n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    MoveLogProbVector moves;
    BreakThreatmate::generateBreak(500, state, Move(Position(9,8), GOLD, WHITE), 
				   moves);
    CPPUNIT_ASSERT(! moves.empty());    
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * -KI-KE-OU\n"
			   "P2 *  *  *  *  *  * -KI-GI-KY\n"
			   "P3 *  *  *  *  *  * -FU-FU-FU\n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7+FU+FU+FU+FU *  *  *  *  * \n"
			   "P8+OU * +KI *  * -HI *  *  * \n"
			   "P9+KY+KE-GI *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    MoveLogProbVector moves;
    const Move threatmate_move(Position(4,8), Position(7,8), PROOK, 
			       GOLD, false, WHITE);
    BreakThreatmate::generateBreak(500, state, threatmate_move, moves);
    CPPUNIT_ASSERT(! moves.empty());    
    CPPUNIT_ASSERT(moves.isMember(MoveLogProb(Move(Position(5,8), GOLD, BLACK),
					      400))); 
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * -KI-KE-OU\n"
			   "P2 *  *  *  *  *  * -KI-GI-KY\n"
			   "P3 *  *  *  *  *  * -FU-FU-FU\n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 * -FU-KE-FU *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7+FU+FU+FU *  *  *  *  *  * \n"
			   "P8+KY+KA *  *  *  *  *  *  * \n"
			   "P9+OU+KE *  *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    MoveLogProbVector moves;
    const Move threatmate_move(Position(7,5), Position(8,7), KNIGHT, 
			       PAWN, false, WHITE);
    BreakThreatmate::generateBreak(500, state, threatmate_move, moves);
    CPPUNIT_ASSERT(! moves.empty());    
    CPPUNIT_ASSERT(moves.isMember(MoveLogProb(Move(Position(8,8),Position(7,9),
						   BISHOP, PTYPE_EMPTY, false,
						   BLACK), 100))); 
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
