#include "osl/category/mate.h"
#include "osl/category/categoryUtil.h"
#include "osl/category/classify.h"
#include "osl/category/classify.tcc"
#include "osl/record/csaString.h"
#include "osl/record/record.h"
#include "osl/record/csaRecord.h"
#include "osl/state/numEffectState.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/apply_move/applyMove.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <algorithm>
#include <fstream>

using namespace osl;
using namespace osl::category;
extern bool isShortTest;

class MateTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(MateTest);
  CPPUNIT_TEST(testDropPieceOnEscapeRoute);
  CPPUNIT_TEST_SUITE_END();
public:
  void testDropPieceOnEscapeRoute();
};

CPPUNIT_TEST_SUITE_REGISTRATION(MateTest);

void MateTest::testDropPieceOnEscapeRoute()
{
  NumEffectState state(CsaString("P1-KY-KE * -KI *  * -GI-KE-KY\n"
				 "P2 * -HI-GI * -OU * -KI-KA * \n"
				 "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
				 "P4 *  *  *  *  *  * -FU *  * \n"
				 "P5 *  *  *  *  *  *  *  *  * \n"
				 "P6 *  * +FU *  *  * +FU *  * \n"
				 "P7+FU+FU * +FU+FU+FU * +FU+FU\n"
				 "P8 *  *  *  * +OU * +GI+HI * \n"
				 "P9+KY+KE+GI+KI * +KI * +KE+KY\n"
				 "P+00KA\n"
				 "+\n").getInitialState());

  MoveLogProbVector moves;
  Classify<CategoryList<DropPieceOnEscapeRoute, NullCategory> > classify;
  CategoryEnv env(&state, osl::category::CategoryTraits::maxProb(), 0);
  classify.generate(env, moves);
  CPPUNIT_ASSERT_EQUAL(static_cast<size_t>(2u), moves.size());
  state.changeTurn();
  moves.clear();
  classify.generate(env, moves);
  CPPUNIT_ASSERT_EQUAL(static_cast<size_t>(0), moves.size());
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
