#include "osl/category/pawn.h"
#include "osl/category/categoryUtil.h"
#include "osl/category/classify.h"
#include "osl/category/classify.tcc"
#include "osl/record/csaString.h"
#include "osl/record/record.h"
#include "osl/record/csaRecord.h"
#include "osl/state/numEffectState.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/apply_move/applyMove.h"
#include "osl/oslConfig.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <algorithm>
#include <fstream>

using namespace osl;
using namespace osl::category;
extern bool isShortTest;

class PawnTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(PawnTest);
  CPPUNIT_TEST(testCount);
  CPPUNIT_TEST(testFindDangling);
  CPPUNIT_TEST(testDrop);
  CPPUNIT_TEST(testFrontRook);
  CPPUNIT_TEST(testMoveSaGWithPawn);
  CPPUNIT_TEST(testPawnStopBishopEffect);
  CPPUNIT_TEST(testPawnOnLance);
  CPPUNIT_TEST_SUITE_END();
public:
  void testCount();
  void testFindDangling();
  void testDrop();
  void testFrontRook();
  void testMoveSaGWithPawn();
  void testPawnStopBishopEffect();
  void testPawnOnLance();
};

CPPUNIT_TEST_SUITE_REGISTRATION(PawnTest);

void PawnTest::testCount()
{
  NumEffectState state(CsaString("P1+NY+TO *  *  *  * -OU-KE-KY\n"
				   "P2 *  * -FU *  * -GI-KI *  *\n"
				   "P3 * +RY *  * +UM * -KI-FU-FU\n"
				   "P4 *  * +FU-FU *  *  *  *  *\n"
				   "P5-FU * -KE * +FU *  * +FU *\n"
				   "P6-KE *  * +FU+GI-FU *  * +FU\n"
				   "P7 *  * -UM *  *  *  *  *  *\n"
				   "P8 *  *  *  *  *  *  *  *  * \n"
				   "P9 * +OU * -GI *  *  *  * -NG\n"
				   "P+00HI00KI00KE00KY00FU00FU00FU00FU\n"
				   "P-00KI00KY00FU00FU\n"
				   "P-00AL\n"
				   "+\n").getInitialState());

  MoveLogProbVector moves;
  CategoryEnv env(&state, 1000, 0);
  CategoryUtil<PawnBeforePawn>::generate(env,moves);
  CPPUNIT_ASSERT_EQUAL(static_cast<size_t>(3u), moves.size());
  state.changeTurn();
  moves.clear();
  CategoryUtil<PawnBeforePawn>::generate(env,moves);
  CPPUNIT_ASSERT_EQUAL(static_cast<size_t>(4u), moves.size());
}

void PawnTest::testFindDangling()
{
  extern bool isShortTest;
  if (isShortTest)
    return;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  std::string file_name;
  for (int i=0; i<1000 && (ifs >> file_name); i++){
    if (file_name == "") 
      break;
    file_name = OslConfig::testCsaFile(file_name);
    const Record record=CsaFile(file_name).getRecord();
    NumEffectState state(record.getInitialState());
    CategoryEnv env(&state, 1000, 0);
    const vector<osl::Move> moves=record.getMoves();
    for (unsigned int i=0;i<moves.size();i++)
    {
      if (DanglingPawn::isInCategory(env, moves[i]))
      {
#if 0
	// testLong μ
	std::cerr << state << moves[i] << "\n";
#endif
      }
      ApplyMoveOfTurn::doMove(state, moves[i]);
    }
  }
}

void PawnTest::testDrop()
{
  NumEffectState state(CsaString(
			   "P1-KY-KE * -KI-OU-KI *  * -KY\n"
			   "P2 * -HI-GI *  *  *  *  *  * \n"
			   "P3-FU * -FU-FU-FU-FU-GI * -FU\n"
			   "P4 *  *  *  *  *  * -FU-FU * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  * +FU *  *  *  *  *  * \n"
			   "P7+FU * +KA+FU+FU+FU+FU * +FU\n"
			   "P8 * +GI+KI *  * +GI * +HI * \n"
			   "P9+KY+KE *  * +OU+KI * +KE+KY\n"
			   "P+00FU\n"
			   "P-00KA00KE00FU00FU\n"
			   "+\n").getInitialState());

  MoveStack history;
  history.push(Move(Position(2,3), Position(2,4), PAWN, PAWN, false, WHITE));
  
  MoveLogProbVector moves;
  CategoryEnv env(&state, 1000, &history);
  CategoryUtil<DropAfterOpposingPawn>::generate(env, moves);
  CPPUNIT_ASSERT_EQUAL(static_cast<size_t>(2u), moves.size()); // 23, 22
}

void PawnTest::testFrontRook()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			 "P2 * -HI *  *  *  *  * -KA * \n"
			 "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
			 "P4 *  *  *  *  *  *  *  *  * \n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  *  *  *  *  *  *  *  * \n"
			 "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
			 "P8 * +KA *  *  *  *  * +HI * \n"
			 "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			 "+\n").getInitialState());
  const Move m26fu(Position(2,7), Position(2,6), PAWN, PTYPE_EMPTY, false, BLACK);
  const Move m36fu(Position(3,7), Position(3,6), PAWN, PTYPE_EMPTY, false, BLACK);
  CategoryEnv env(&state, 1000, 0);
  CPPUNIT_ASSERT(ForwardPawnWithRookSupport::isInCategory(env, m26fu));
  CPPUNIT_ASSERT(! ForwardPawnWithRookSupport::isInCategory(env, m36fu));

  const Move m84fu(Position(8,3), Position(8,4), PAWN, PTYPE_EMPTY, false, WHITE);
  const Move m74fu(Position(7,3), Position(7,4), PAWN, PTYPE_EMPTY, false, WHITE);
  state.changeTurn();
  CPPUNIT_ASSERT(ForwardPawnWithRookSupport::isInCategory(env, m84fu));
  CPPUNIT_ASSERT(! ForwardPawnWithRookSupport::isInCategory(env, m74fu));
}

void PawnTest::testMoveSaGWithPawn()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE *  * -KI *  *  * -KY\n"
			 "P2 * -OU-GI * -KI *  *  *  * \n"
			 "P3-FU-FU * -FU *  * +UM-FU-FU\n"
			 "P4 *  *  *  * -GI *  *  *  * \n"
			 "P5 *  *  *  *  *  * -KA *  * \n"
			 "P6 *  *  *  *  *  *  *  *  * \n"
			 "P7+FU+FU * +FU+GI+FU+FU * +FU\n"
			 "P8 *  * +OU * +KI *  * -HI * \n"
			 "P9+KY+KE+GI+KI *  *  * +KE+KY\n"
			 "P+00HI00KE00FU00FU00FU\n"
			 "P-00FU00FU00FU00FU\n"
			 "+\n").getInitialState());
  const Move m53fu(Position(5, 3), PAWN, BLACK);
  const Move m55fu(Position(5, 5), PAWN, BLACK);
  const Move m73fu(Position(7, 3), PAWN, BLACK);
  CategoryEnv env(&state, 1000, 0);
  CPPUNIT_ASSERT(MoveSaGWithPawn::isInCategory(env, m53fu));
  CPPUNIT_ASSERT(MoveSaGWithPawn::isInCategory(env, m55fu));
  CPPUNIT_ASSERT(!MoveSaGWithPawn::isInCategory(env, m73fu));

  const Move m56fu(Position(5, 6), PAWN, WHITE);
  state.changeTurn();
  CPPUNIT_ASSERT(MoveSaGWithPawn::isInCategory(env, m56fu));
}

void PawnTest::testPawnStopBishopEffect()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			 "P2 * -HI *  *  *  *  *  *  * \n"
			 "P3-FU-FU-FU-FU * -FU-KA * -FU\n"
			 "P4 *  *  *  * -FU * -FU *  * \n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  * +FU * +FU *  *  *  * \n"
			 "P7+FU+FU+GI+FU * +FU+FU+FU+FU\n"
			 "P8 *  *  *  *  *  *  * +HI * \n"
			 "P9+KY+KE+KA+KI+OU+KI+GI+KE+KY\n"
			 "P-00FU\n"
			 "+\n").getInitialState());
  const Move m66fu(Position(6, 7), Position(6, 6), PAWN,
		   PTYPE_EMPTY, false, BLACK);
  const Move m55fu(Position(5, 6), Position(5, 5), PAWN,
		   PTYPE_EMPTY, false, BLACK);
  CategoryEnv env(&state, 1000, 0);
  CPPUNIT_ASSERT(PawnStopBishopEffect::isInCategory(env, m66fu));
  CPPUNIT_ASSERT(!PawnStopBishopEffect::isInCategory(env, m55fu));

  const Move m24fu(Position(2, 4), PAWN, WHITE);
  state.changeTurn();
  CPPUNIT_ASSERT(PawnStopBishopEffect::isInCategory(env, m24fu));
}

void PawnTest::testPawnOnLance()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			 "P2 * -HI *  *  *  *  *  *  * \n"
			 "P3-FU-FU-FU-FU * -FU-KA *  * \n"
			 "P4 *  *  *  * -FU * -FU * -FU\n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  * +FU * +FU *  *  *  * \n"
			 "P7+FU+FU+GI+FU * +FU+FU+FU * \n"
			 "P8 *  *  *  *  *  *  * +HI * \n"
			 "P9+KY+KE+KA+KI+OU+KI+GI+KE+KY\n"
			 "P+00FU\n"
			 "P-00FU\n"
			 "+\n").getInitialState());
  const Move m12fu(Position(1, 2),PAWN, BLACK);
  const Move m13fu(Position(1, 3),PAWN, BLACK);
  CategoryEnv env(&state, 1000, 0);
  CPPUNIT_ASSERT(PawnOnLance::isInCategory(env, m12fu));
  CPPUNIT_ASSERT(!PawnOnLance::isInCategory(env, m13fu));
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
