#include "osl/container/linearMoveMap.h"
#include "osl/container/moveLogProbSet.h"
#include <set>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
using namespace osl;

class moveMapTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(moveMapTest);
  CPPUNIT_TEST(testCreation);
  CPPUNIT_TEST(testInsert);
  CPPUNIT_TEST(testIteration);
  CPPUNIT_TEST(testAssignIfBetter);
  CPPUNIT_TEST_SUITE_END();
public:
  void testCreation();
  void testInsert();
  void testKey();
  void testIteration();
  void testAssignIfBetter();
};

CPPUNIT_TEST_SUITE_REGISTRATION(moveMapTest);

const Move m61hi = Move(Position(6,1),ROOK,BLACK);
const Move m6261hi = Move(Position(6,2),Position(6,1),PTYPE_EMPTY,ROOK,false,BLACK);
typedef LinearMoveMap<MoveLogProb> map_t;

void moveMapTest::testCreation()
{
  map_t map;
}

void moveMapTest::testInsert()
{
  map_t map;
  MoveLogProb m1(m61hi, 100);
  CPPUNIT_ASSERT(! map.find(m1.getMove()));
  map.assign(m1);
  CPPUNIT_ASSERT(map.find(m1.getMove()));
  CPPUNIT_ASSERT_EQUAL(m1, *map.find(m1.getMove()));

  MoveLogProb m2(m61hi, 200);
  map.assign(m2);
  CPPUNIT_ASSERT_EQUAL(m2, *map.find(m2.getMove()));
}

void moveMapTest::testIteration()
{
  map_t map;
  CPPUNIT_ASSERT(map.begin() == map.end());

  MoveLogProb m61hi100(m61hi, 100);
  map.assign(m61hi100);
  map_t::const_iterator p = map.begin();
  CPPUNIT_ASSERT(p != map.end());
  CPPUNIT_ASSERT_EQUAL(m61hi100, *p);
  ++p;
  CPPUNIT_ASSERT(p == map.end());

  // move Ʊʤ񤭤 (Ǥʤ)
  MoveLogProb m61hi200(m61hi, 200);
  map.assign(m61hi200);
  p = map.begin();
  CPPUNIT_ASSERT(p != map.end());
  CPPUNIT_ASSERT_EQUAL(m61hi200, *p);
  ++p;
  CPPUNIT_ASSERT(p == map.end());

  // 㤦move Ǥ
  MoveLogProb m6261hi100(m6261hi, 100);
  map.assign(m6261hi100);
  p = map.begin();
  CPPUNIT_ASSERT(p != map.end());
  CPPUNIT_ASSERT_EQUAL(m6261hi100, *p);
  ++p;
  CPPUNIT_ASSERT(p != map.end());
  CPPUNIT_ASSERT_EQUAL(m61hi200, *p);
  ++p;
  CPPUNIT_ASSERT(p == map.end());
}

void moveMapTest::testAssignIfBetter()
{
  MoveLogProbSet set;
  CPPUNIT_ASSERT(set.begin() == set.end());

  MoveLogProb m61hi100(m61hi, 100);
  CPPUNIT_ASSERT(set.assignIfBetter(m61hi100));
  MoveLogProbSet::const_iterator p = set.begin();
  CPPUNIT_ASSERT(p != set.end());
  CPPUNIT_ASSERT_EQUAL(m61hi100, *p);
  ++p;
  CPPUNIT_ASSERT(p == set.end());

  // move ƱǳΨ⤯ʤʤ Τޤ
  CPPUNIT_ASSERT(! set.assignIfBetter(m61hi100));
  MoveLogProb m61hi200(m61hi, 200);
  CPPUNIT_ASSERT(! set.assignIfBetter(m61hi200));

  MoveLogProb m61hi80(m61hi, 80);
  CPPUNIT_ASSERT(set.assignIfBetter(m61hi80));

  p = set.begin();
  CPPUNIT_ASSERT(p != set.end());
  CPPUNIT_ASSERT_EQUAL(m61hi80, *p);
  ++p;
  CPPUNIT_ASSERT(p == set.end());
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
