/* progressEval.t.cc
 */
#include "osl/eval/endgame/attackDefense.h"
#include "osl/eval/progressEval.h"
#include "osl/record/csaString.h"
#include "consistencyTest.h"

#include <cppunit/TestCase.h>

class ProgressEvalTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(ProgressEvalTest);
  CPPUNIT_TEST(testLoad);
  // ʹߤ testLoad θ˥ƥȤ뤳
  CPPUNIT_TEST(testBeginning);
  CPPUNIT_TEST(testConsistentUpdate);
  CPPUNIT_TEST(testConsistentExpect);
  // 
  CPPUNIT_TEST(testInitial);
  CPPUNIT_TEST(testPinBonus);
  CPPUNIT_TEST_SUITE_END();
public:
  void testLoad();
  void testBeginning();
  void testConsistentUpdate();
  void testConsistentExpect();
  void testInitial();
  void testPinBonus();
};

CPPUNIT_TEST_SUITE_REGISTRATION(ProgressEvalTest);

using namespace osl;
using namespace osl::eval;
using namespace osl::eval::ppair;

void ProgressEvalTest::testBeginning()
{
  const NumEffectState state(CsaString(
			       "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			       "P2 * -HI *  *  *  *  * -KA * \n"
			       "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
			       "P4 *  *  *  *  *  *  *  *  * \n"
			       "P5 *  *  *  *  *  *  *  *  * \n"
			       "P6 *  *  *  *  *  *  *  *  * \n"
			       "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
			       "P8 * +KA *  *  *  *  * +HI * \n"
			       "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			       "+\n").getInitialState());
  ProgressEval eval(state);
  CPPUNIT_ASSERT_EQUAL(0, eval.progress16().value());
  ProgressEval::opening_eval_t opening(state);
  CPPUNIT_ASSERT(opening.value() != 0);
  CPPUNIT_ASSERT(abs(opening.value()*16 - eval.value())
		 < ProgressEval::ROUND_UP);
}

void ProgressEvalTest::testInitial()
{
  const NumEffectState state(CsaString(
			       "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			       "P2 * -HI *  *  *  *  * -KA * \n"
			       "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
			       "P4 *  *  *  *  *  *  *  *  * \n"
			       "P5 *  *  *  *  *  *  *  *  * \n"
			       "P6 *  *  *  *  *  *  *  *  * \n"
			       "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
			       "P8 * +KA *  *  *  *  * +HI * \n"
			       "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			       "+\n").getInitialState());
  ProgressEval eval(state);
  CPPUNIT_ASSERT_EQUAL(0, eval.endgameValue());
}


void ProgressEvalTest::testLoad()
{
  CPPUNIT_ASSERT(ProgressEval::setUp());
}

void ProgressEvalTest::testConsistentUpdate()
{
  consistencyTestUpdate<ProgressEval>();
}

void ProgressEvalTest::testConsistentExpect()
{
  consistencyTestExpect<ProgressEval>(16000); // 48000
}

void ProgressEvalTest::testPinBonus()
{
  {
    const NumEffectState state(
      CsaString("P1-KY-KE *  *  *  *  * -KE-KY\n"
		"P2 * -HI *  *  *  * -KI-OU * \n"
		"P3-FU-FU-FU-FU * -KI-GI-FU-FU\n"
		"P4 *  *  *  * -FU-KA-FU *  * \n"
		"P5 *  *  *  *  * -FU *  *  * \n"
		"P6 *  * +FU+FU+FU *  *  *  * \n"
		"P7+FU+FU * +KI * +FU+FU+FU+FU\n"
		"P8 * +OU+GI *  *  *  * -HI * \n"
		"P9+KY+KE+KA+KI *  *  * +KE+KY\n"
		"P+00GI\n"
		"P-00GI\n"
		"+\n").getInitialState());
    ProgressEval eval(state);
    endgame::AttackDefense endgame(state);
    const Piece black_king = state.getKingPiece<BLACK>();
    const Piece white_king = state.getKingPiece<WHITE>();
    const int bonus = eval.calculatePinBonus(state);
    CPPUNIT_ASSERT(bonus < 0);
    CPPUNIT_ASSERT_EQUAL(
      -(endgame.valueOf(black_king, white_king,
			state.getPieceAt(Position(6, 6))) +
	endgame.valueOf(black_king, white_king,
			state.getPieceAt(Position(7, 8)))) / 4,
      bonus);
  }

  {
    const NumEffectState state(
      CsaString("P1-KY-KE *  *  *  *  * -KE-KY\n"
		"P2 * +HI *  *  *  * -KI-OU * \n"
		"P3-FU-FU-FU-FU * -KI-GI-FU-FU\n"
		"P4 *  *  *  * -FU-KA-FU *  * \n"
		"P5 *  *  *  *  * -FU *  *  * \n"
		"P6 *  * +FU+FU+FU *  *  *  * \n"
		"P7+FU+FU+KE+KI * +FU+FU+FU+FU\n"
		"P8 * +OU+GI *  *  *  * +HI * \n"
		"P9+KY * +KA+KI *  *  * +KE+KY\n"
		"P+00GI\n"
		"P-00GI\n"
		"+\n").getInitialState());
    ProgressEval eval(state);
    endgame::AttackDefense endgame(state);
    const Piece black_king = state.getKingPiece<BLACK>();
    const Piece white_king = state.getKingPiece<WHITE>();
    const int bonus = eval.calculatePinBonus(state);
    CPPUNIT_ASSERT(bonus > 0);
    CPPUNIT_ASSERT_EQUAL(
      -(endgame.valueOf(black_king, white_king,
		       state.getPieceAt(Position(3, 2))) / 4),
      bonus);
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
