#include "osl/move.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace osl;

class MoveTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(MoveTest);
  CPPUNIT_TEST(testPass);
  CPPUNIT_TEST(testValid);
  CPPUNIT_TEST(testDeclareWin);
  CPPUNIT_TEST(testNormal);
  CPPUNIT_TEST_SUITE_END();
public:
  void testValid() {
    const Move drop(Position(5,5), KNIGHT, WHITE);
    CPPUNIT_ASSERT(drop.isValid()); 

    const Move bug = Move::makeDirect(-58698240);
    CPPUNIT_ASSERT(! bug.isValid()); // KING55
  }
  void testNormal()
  {
    const Move m76fu(Position(7,7),Position(7,6),PAWN,PTYPE_EMPTY,false,BLACK);
    CPPUNIT_ASSERT(! m76fu.isPass());
    CPPUNIT_ASSERT(m76fu.isNormal());
    CPPUNIT_ASSERT(! m76fu.isInvalid());

    const Move m34fu(Position(3,3),Position(3,4),PAWN,PTYPE_EMPTY,false,WHITE);
    CPPUNIT_ASSERT(! m34fu.isPass());
    CPPUNIT_ASSERT(m34fu.isNormal());
    CPPUNIT_ASSERT(! m34fu.isInvalid());
  }
  void testPass()
  {
    const Move pass_black = Move::PASS(BLACK);
    CPPUNIT_ASSERT_EQUAL(PTYPE_EMPTY, pass_black.ptype());
    CPPUNIT_ASSERT_EQUAL(PTYPE_EMPTY, pass_black.oldPtype());
    CPPUNIT_ASSERT_EQUAL(Position::STAND(), pass_black.from());
    CPPUNIT_ASSERT_EQUAL(Position::STAND(), pass_black.to());
    CPPUNIT_ASSERT_EQUAL(BLACK, pass_black.player());

    CPPUNIT_ASSERT(pass_black.isPass());
    CPPUNIT_ASSERT(! pass_black.isNormal());
    CPPUNIT_ASSERT(! pass_black.isInvalid());

    const Move pass_white = Move::PASS(WHITE);
    CPPUNIT_ASSERT_EQUAL(PTYPE_EMPTY, pass_white.ptype());
    CPPUNIT_ASSERT_EQUAL(PTYPE_EMPTY, pass_white.oldPtype());
    CPPUNIT_ASSERT_EQUAL(Position::STAND(), pass_white.from());
    CPPUNIT_ASSERT_EQUAL(Position::STAND(), pass_white.to());
    CPPUNIT_ASSERT_EQUAL(WHITE, pass_white.player());

    CPPUNIT_ASSERT(pass_white.isPass());
    CPPUNIT_ASSERT(! pass_white.isNormal());
    CPPUNIT_ASSERT(! pass_white.isInvalid());
  }
  void testDeclareWin()
  {
    CPPUNIT_ASSERT(Move::DeclareWin().isInvalid());
    CPPUNIT_ASSERT(! Move::DeclareWin().isNormal());
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(MoveTest);

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
