#include "osl/move_generator/allMoves.h"
#include "osl/move_action/store.h"
#include "osl/move_generator/addEffect8.h"
#include "osl/record/record.h"
#include "osl/record/csaRecord.h"
#include "osl/record/csaString.h"
#include "osl/container/pieceVector.h"
#include "osl/container/moveVector.h"
#include "osl/apply_move/doUndoMoveLock.h"
#include "osl/apply_move/applyMove.h"
#include "osl/effect_util/effectUtil.h"
#include "osl/effect_util/additionalEffect.h"
#include "osl/oslConfig.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
#include <fstream>


class AddEffect8Test : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(AddEffect8Test);
  CPPUNIT_TEST(testAdditional);
  CPPUNIT_TEST(testIsAddEffect8);
  CPPUNIT_TEST(testLong);
  CPPUNIT_TEST(testDropLong);
  CPPUNIT_TEST(testDropShort);
  CPPUNIT_TEST(testDropMember);
  CPPUNIT_TEST(testShort);
  CPPUNIT_TEST(testMoveMember);
  CPPUNIT_TEST_SUITE_END();
public:
  void testIsAddEffect8();
  void testDropShort();
  void testDropLong();
  void testDropMember();
  void testShort();
  void testLong();
  void testAdditional();
  void testMoveMember();
};

CPPUNIT_TEST_SUITE_REGISTRATION(AddEffect8Test);
using namespace osl;
using namespace osl::state;
using namespace osl::move_generator;
using namespace osl::move_action;

void AddEffect8Test::testDropShort()
{
  {
    SimpleState state=
      CsaString(
		"P1+NY+TO *  *  *  * -OU-KE-KY\n"
		"P2 *  *  *  *  * -GI-KI *  *\n"
		"P3 * +RY *  * +UM * -KI-FU-FU\n"
		"P4 *  * +FU-FU *  *  *  *  *\n"
		"P5 *  * -KE * +FU *  * +FU *\n"
		"P6+KE *  * +FU+GI-FU *  * +FU\n"
		"P7 *  * -UM *  *  *  *  *  *\n"
		"P8 *  *  *  *  *  *  *  *  * \n"
		"P9 * +OU * -GI *  *  *  * -NG\n"
		"P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
		"P-00KI00KY00FU00FU\n"
		"P-00AL\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,3),PAWN,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(1,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(2,4),KNIGHT,BLACK)));
    // produceʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,2),GOLD,BLACK)));
    // 
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,2),PAWN,BLACK)));
  }
}

void AddEffect8Test::testDropLong()
{
  {
    SimpleState state=
      CsaString(
		"P1-KY *  *  *  * -OU * -KE-KY\n"
		"P2 * -HI *  *  *  * -KI *  * \n"
		"P3-FU *  *  * +KI-GI * -FU-FU\n"
		"P4 *  * -FU-FU *  * -FU *  * \n"
		"P5 *  *  * -KE *  *  * +FU * \n"
		"P6+GI-FU+FU *  *  * +FU *  * \n"
		"P7+FU *  * +FU-KI *  *  * +FU\n"
		"P8 * +OU+KI * -FU *  *  *  * \n"
		"P9+KY+KE *  * -RY *  *  * +KY\n"
		"P+00KA00GI00GI00KE00FU00FU00FU00FU\n"
		"P-00KA\n"
		"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,6),BISHOP,WHITE)) ||
		   (std::cerr << moves << std::endl,0)
		   );
  }
  {
    SimpleState state=
      CsaString(
		"P1+NY+TO *  *  *  * -OU-KE *\n"
		"P2 *  *  *  *  * -GI-KI * -KY\n"
		"P3 * +RY *  *  *  *  * -FU-FU\n"
		"P4 *  * +FU-FU *  *  *  *  *\n"
		"P5 *  * -KE * +FU *  * +FU *\n"
		"P6+KE *  * +FU+GI-FU *  * +FU\n"
		"P7 *  * -UM *  *  *  *  *  *\n"
		"P8 *  *  *  *  *  *  *  *  * \n"
		"P9 * +OU * -GI *  *  *  * -NG\n"
		"P+00HI00KA00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
		"P-00KI00KI00KY00FU00FU\n"
		"P-00AL\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,2),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(2,2),LANCE,BLACK)));

    // ֤ΤΤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,3),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,3),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,4),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,4),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,5),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,5),LANCE,BLACK)));

    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,2),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(6,2),ROOK,BLACK)));

    CPPUNIT_ASSERT(!moves.isMember(Move(Position(7,2),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(8,2),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(9,2),ROOK,BLACK)));

    CPPUNIT_ASSERT(moves.isMember(Move(Position(1,1),BISHOP,BLACK)));
    //
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,3),BISHOP,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,4),BISHOP,BLACK)));

    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,3),BISHOP,BLACK)));

    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,4),BISHOP,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(6,5),BISHOP,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(7,6),BISHOP,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(8,7),BISHOP,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(9,8),BISHOP,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,1),BISHOP,BLACK)));

    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,4),BISHOP,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(1,5),BISHOP,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,1),BISHOP,BLACK)));

    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,2),BISHOP,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(6,3),BISHOP,BLACK)));

    // direct
    CPPUNIT_ASSERT(moves.isMember(Move(Position(1,1),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,3),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,3),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,4),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,4),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,5),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,5),LANCE,BLACK)));
    // 2İʾʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,6),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,7),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,8),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,9),ROOK,BLACK)));

    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,6),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,7),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,8),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,9),LANCE,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,3),BISHOP,BLACK)));


    // produceʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,2),BISHOP,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,1),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,1),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(6,1),ROOK,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(7,1),ROOK,BLACK)));

    // ׳ؤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,1),LANCE,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(6,2),BISHOP,BLACK)));
  }
}

void AddEffect8Test::testShort()
{
  {
    SimpleState state=
      CsaString(
"P1 *  * +RY-KE *  *  * -KE-KY\n"
"P2 *  *  * +GI * -OU-KI *  * \n"
"P3-FU * -FU-FU * -KI *  *  * \n"
"P4 *  *  * -UM * -FU+FU * -FU\n"
"P5+OU *  *  *  *  *  *  *  * \n"
"P6 *  * +FU-KI *  * +KY *  * \n"
"P7+FU+FU *  * +FU+FU *  * +FU\n"
"P8 * +KI *  *  *  *  *  *  * \n"
"P9+KY+KE *  * -RY *  *  * +KY\n"
"P+00GI00KE00FU\n"
"P-00KA00GI00GI00FU00FU00FU00FU00FU\n"
"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    // 
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(6,2),Position(6,1),PSILVER,KNIGHT,true,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
		"P1 *  *  *  *  *  *  *  *  *\n"
		"P2 *  *  *  *  *  *  *  *  * \n"
		"P3+NY *  *  *  *  * -OU-KE-KY\n"
		"P4 * +HI *  * -FU-GI-KI-FU *\n"
		"P5 * +RY *  * +UM-FU-KI * -FU\n"
		"P6 *  * +FU-FU *  *  * +FU *\n"
		"P7 *  * -KE * +FU+KY *  *  *\n"
		"P8+KE * +KA+FU+GI *  *  * +FU\n"
		"P9 *  * +OU-GI *  *  *  * -NG\n"
		"P+00KI00KE00FU00FU00FU00FU00FU00FU\n"
		"P-00KI00KY00FU\n"
		"P-00AL\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,4),Position(5,4),ROOK,PAWN,false,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(7,8),Position(4,5),BISHOP,PAWN,false,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
		"P1+NY+TO *  *  *  * -OU-KE-KY\n"
		"P2 *  *  *  *  * -GI-KI *  *\n"
		"P3 * +RY *  * +UM * -KI-FU-FU\n"
		"P4 *  * +FU-FU *  *  *  *  *\n"
		"P5 *  * -KE * +FU *  * +FU *\n"
		"P6+KE *  * +FU+GI-FU *  * +FU\n"
		"P7 *  * -UM *  *  *  *  *  *\n"
		"P8 *  *  *  *  *  *  *  *  * \n"
		"P9 * +OU * -GI *  *  *  * -NG\n"
		"P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
		"P-00KI00KY00FU00FU\n"
		"P-00AL\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(3,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(1,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(2,4),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,3),PAWN,BLACK)));
    // produceʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,3),KNIGHT,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,2),GOLD,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
		"P1+NY *  *  *  *  * -OU-KE-KY\n"
		"P2 * +HI *  * -FU-GI-KI-FU *\n"
		"P3 * +RY *  * +UM-FU-KI * -FU\n"
		"P4 *  * +FU-FU *  *  * +FU *\n"
		"P5 *  * -KE * +FU+KY *  *  *\n"
		"P6+KE * +KA+FU+GI *  *  * +FU\n"
		"P7 *  *  *  *  *  *  *  *  *\n"
		"P8 *  *  *  *  *  *  *  *  * \n"
		"P9 * +OU * -GI *  *  *  * -NG\n"
		"P+00KI00KE00FU00FU00FU00FU00FU00FU\n"
		"P-00KI00KY00FU\n"
		"P-00AL\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    // ܤ˹֤ʤϤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(4,5),Position(4,2),LANCE,SILVER,false,BLACK)));
    // Ͼ
    CPPUNIT_ASSERT(moves.isMember(Move(Position(2,4),Position(2,3),PPAWN,PTYPE_EMPTY,true,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,4),Position(2,3),PAWN,PTYPE_EMPTY,false,BLACK)));
    // ֤
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,2),Position(5,2),PROOK,PAWN,true,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(8,2),Position(5,2),ROOK,PAWN,false,BLACK)));
    // Ѥ
    CPPUNIT_ASSERT(moves.isMember(Move(Position(7,6),Position(4,3),PBISHOP,PAWN,true,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(7,6),Position(4,3),BISHOP,PAWN,false,BLACK)));
  }
}
void AddEffect8Test::testLong()
{
  {
    SimpleState state=
      CsaString(
"P1 *  *  *  * +RY *  *  * -KY\n"
"P2 *  *  * -HI *  *  *  *  * \n"
"P3-KY *  * -OU-KA * +KI *  * \n"
"P4 * -FU-FU-GI-KI *  *  *  * \n"
"P5 *  *  * -KE-FU-FU-FU *  * \n"
"P6-KY * +FU+GI *  *  *  *  * \n"
"P7+KE+FU * +KI *  *  *  * +FU\n"
"P8+FU+OU+KI+KA-GI *  *  *  * \n"
"P9 * +KE *  *  *  *  *  * +KY\n"
"P+00GI00KE00FU00FU00FU00FU00FU00FU00FU\n"
"P-00FU00FU\n"
"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    // 2Ťɲ
    CPPUNIT_ASSERT(moves.isMember(Move(Position(6,2),Position(9,2),ROOK,PTYPE_EMPTY,false,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
"P1 *  *  *  *  *  *  * +TO+RY\n"
"P2 *  *  *  *  *  *  *  *  * \n"
"P3+OU+NK * +TO *  *  *  *  * \n"
"P4+GI+KI *  *  *  *  *  *  * \n"
"P5-KI *  *  *  * +GI *  *  * \n"
"P6 *  * -UM * +FU * +TO-KI * \n"
"P7 *  *  *  *  *  * -NK-NY * \n"
"P8 *  *  *  *  *  * -NY-OU-KI\n"
"P9-RY *  *  *  *  *  *  *  * \n"
"P+00KA00KY00KY00FU00FU00FU00FU\n"
"P-00GI00GI00KE00KE00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU\n"
"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    // openľ򸺤餷Ƥ롥ʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(9,5),Position(8,6),GOLD,PTYPE_EMPTY,false,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
"P1 *  *  *  *  *  *  * +TO+RY\n"
"P2 *  *  *  *  *  *  *  *  * \n"
"P3+OU+NK * +TO *  *  *  *  * \n"
"P4+GI+KI *  *  *  *  *  *  * \n"
"P5-GI *  *  *  * +GI *  *  * \n"
"P6 *  * -UM * +FU * +TO-KI * \n"
"P7 *  *  *  *  *  * -NK-NY * \n"
"P8 *  *  *  *  *  * -NY-OU-KI\n"
"P9-RY *  *  *  *  *  *  *  * \n"
"P+00KA00KY00KY00FU00FU00FU00FU\n"
"P-00KI00GI00KE00KE00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU\n"
"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    // open
    CPPUNIT_ASSERT(moves.isMember(Move(Position(9,5),Position(8,6),SILVER,PTYPE_EMPTY,false,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
"P1 *  *  *  * -FU-OU * -KE-KY\n"
"P2 *  *  *  *  * -GI-KI *  * \n"
"P3-FU * -FU-FU * -KI *  *  * \n"
"P4 * +RY *  *  * -FU+FU * -FU\n"
"P5 *  *  *  *  *  *  *  *  * \n"
"P6 *  * +FU+FU *  * +KY *  * \n"
"P7+FU+FU *  *  * +FU+GI * +FU\n"
"P8 *  * +KI+GI+KI *  *  *  * \n"
"P9+KY+KE * +OU+KE-KA * -RY+KY\n"
"P+00GI00FU\n"
"P-00KA00KE00FU00FU00FU00FU\n"
"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    // ¤openäΤ
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,9),Position(3,8),BISHOP,PTYPE_EMPTY,false,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
"P1-KY-KE *  *  *  *  *  * -KY\n"
"P2 *  *  *  *  *  * -KI-OU * \n"
"P3 *  * -FU *  * -KI *  *  * \n"
"P4-FU-HI *  * -FU * -FU+GI-FU\n"
"P5 * -FU+FU * +GI-FU * -FU+KY\n"
"P6+FU *  *  *  *  * +KE *  * \n"
"P7 * +FU * -TO * +FU+FU+FU * \n"
"P8+KY *  *  *  * +KI+KI+GI * \n"
"P9 * +KE-UM * +KA *  * +KE+OU\n"
"P+00FU\n"
"P-00HI00GI00FU00FU\n"
"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    // ֳѤɲϼȤˤϰưƤɤ
    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,9),Position(6,8),BISHOP,PTYPE_EMPTY,false,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
"P1 * +RY *  * -FU-OU * -KE-KY\n"
"P2 *  *  *  * +FU-GI-KI *  * \n"
"P3-FU * -FU-FU * -KI *  *  * \n"
"P4 *  *  *  *  * -FU+FU * -FU\n"
"P5 *  *  *  *  *  *  *  *  * \n"
"P6 *  * +FU+FU *  * +KY *  * \n"
"P7+FU+FU *  *  * +FU-UM * +FU\n"
"P8 * +KI * +GI+OU *  *  *  * \n"
"P9+KY+KE *  * +KE *  * -RY+KY\n"
"P+00KA00GI00FU\n"
"P-00KI00GI00KE00FU00FU00FU\n"
"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(2,9),Position(2,7),PROOK,PTYPE_EMPTY,false,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
		"P1-KY-KE * -KI *  *  *  * +RY\n"
		"P2 * -OU-GI * -KI *  *  *  * \n"
		"P3 * -FU-FU-FU-FU-FU * -FU * \n"
		"P4-FU *  *  * +KE-GI *  * +FU\n"
		"P5 * +FU *  *  *  * -FU+FU * \n"
		"P6+FU * -KY * +FU+FU *  *  * \n"
		"P7 * +GI+KA+FU *  * +FU *  * \n"
		"P8 * +OU+KI+GI *  *  *  *  * \n"
		"P9+KY+KE *  * +KI *  *  * -RY\n"
		"P+00KA00KY\n"
		"P-00KE00FU00FU\n"
		"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(7,6),Position(7,7),LANCE,BISHOP,false,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
		"P1-KY-KE * -KI *  * -GI+RY-KY\n"
		"P2 *  *  * -OU *  * -KI *  * \n"
		"P3-FU * -GI-FU-FU-FU *  * -FU\n"
		"P4 *  * -FU *  *  *  *  *  * \n"
		"P5 *  *  *  * +KA *  *  *  * \n"
		"P6 *  *  *  *  *  *  *  *  * \n"
		"P7+FU * +FU+FU+FU+FU+FU * +FU\n"
		"P8 * +GI+KI *  *  *  *  *  * \n"
		"P9+KY+KE *  * +OU+KI+GI * +KY\n"
		"P+00KE00FU00FU\n"
		"P-00HI00KA00KE00FU00FU00FU\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,5),Position(6,4),BISHOP,PTYPE_EMPTY,false,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,5),Position(4,4),BISHOP,PTYPE_EMPTY,false,BLACK)));
    // ֳѤΤȤˤϰưƤɤ
    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,5),Position(3,3),PBISHOP,PTYPE_EMPTY,true,BLACK)));
  }

  {
    SimpleState state=
      CsaString(
		"P1+NY+TO *  *  *  * -OU-KE-KY\n"
		"P2 *  *  *  *  * -GI-KI *  *\n"
		"P3 * +RY *  * +UM * -KI-FU-FU\n"
		"P4 *  * +FU-FU *  *  *  *  *\n"
		"P5 *  * -KE * +FU *  * +FU *\n"
		"P6+KE *  * +FU+GI-FU *  * +FU\n"
		"P7 *  * -UM *  *  *  *  *  *\n"
		"P8 *  *  *  *  *  *  *  *  * \n"
		"P9 * +OU * -GI *  *  *  * -NG\n"
		"P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
		"P-00KI00KY00FU00FU\n"
		"P-00AL\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,3),Position(5,2),PBISHOP,PTYPE_EMPTY,false,BLACK))||
		   (std::cerr << moves << std::endl,0)
		   );
    CPPUNIT_ASSERT(moves.isMember(Move(Position(5,3),Position(4,3),PBISHOP,PTYPE_EMPTY,false,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,3),Position(8,2),PROOK,PTYPE_EMPTY,false,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,3),Position(7,2),PROOK,PTYPE_EMPTY,false,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,3),Position(9,2),PROOK,PTYPE_EMPTY,false,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(2,2),ROOK,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(2,2),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,5),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,4),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(4,3),LANCE,BLACK)));
    // produceʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,3),Position(4,2),PBISHOP,PTYPE_EMPTY,false,BLACK)));
    // ׳դʤ
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,3),Position(6,2),PBISHOP,PTYPE_EMPTY,false,BLACK)));
    // blockƤ뤳Ȥϥå
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,3),Position(4,4),PBISHOP,PTYPE_EMPTY,false,BLACK)));
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(3,4),LANCE,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
		"P1-KY-KE+GI *  *  * +RY * -KY\n"
		"P2 *  * -OU * -KI * +NK *  * \n"
		"P3-FU * -GI-FU-FU-FU *  * -FU\n"
		"P4 *  * -FU *  *  *  *  *  * \n"
		"P5 *  *  *  * +KA *  *  *  * \n"
		"P6 *  *  *  *  *  *  *  *  * \n"
		"P7+FU * +FU+FU+FU+FU+FU * +FU\n"
		"P8 *  * -NK * +OU *  *  *  * \n"
		"P9+KY+KE * -HI * +KI+GI * +KY\n"
		"P+00KA00FU00FU00FU\n"
		"P-00KI00KI00GI00FU00FU\n"
		"+\n").getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(5,5),Position(8,2),PBISHOP,PTYPE_EMPTY,true,BLACK)) ||
		   (std::cerr << moves << std::endl,0)
		   );
  }
}
void AddEffect8Test::testAdditional()
{
  {
    SimpleState state=
      CsaString(
		"P1-KY-KE * -KI *  *  *  * +RY\n"
		"P2 * -OU-GI-KI *  *  *  *  * \n"
		"P3 *  * -FU-FU * -FU * -FU * \n"
		"P4-FU+FU *  * -FU-GI *  * +FU\n"
		"P5 *  *  *  *  *  * -FU+FU * \n"
		"P6+FU *  *  * +FU+FU *  *  * \n"
		"P7 * +GI+KE+FU+KA * +FU *  * \n"
		"P8 *  * +OU+GI *  *  *  *  * \n"
		"P9+KY *  *  * +KI *  *  * -RY\n"
		"P+00KE00KY\n"
		"P-00KA00KI00KE00KY00FU00FU00FU\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(BLACK,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,5),LANCE,BLACK)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(8,6),LANCE,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
		"P1-KY-KE * -KI *  *  *  * +RY\n"
		"P2 * -OU-GI * -KI *  *  *  * \n"
		"P3 * -FU-FU-FU-FU-FU * -FU * \n"
		"P4-FU *  *  * +KE-GI *  * +FU\n"
		"P5 * +FU *  *  *  * -FU+FU * \n"
		"P6+FU * -KY * +FU+FU *  *  * \n"
		"P7 * +GI+KA+FU *  * +FU *  * \n"
		"P8 * +OU+KI+GI *  *  *  *  * \n"
		"P9+KY+KE *  * +KI *  *  * -RY\n"
		"P+00KA\n"
		"P-00KE00KY00FU00FU\n"
		"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    CPPUNIT_ASSERT(moves.isMember(Move(Position(7,4),LANCE,WHITE)));
  }
  {
    SimpleState state=
      CsaString(
		"P1-KY-KE * -KI *  *  *  * +RY\n"
		"P2 * -OU-GI * -KI *  *  *  * \n"
		"P3 * -FU * -FU-FU-FU * -FU * \n"
		"P4 *  *  *  * +KE-GI *  * +FU\n"
		"P5 * +FU-FU *  *  * -FU+FU * \n"
		"P6-FU *  *  * +FU+FU *  *  * \n"
		"P7+FU+GI+KA+FU *  * +FU *  * \n"
		"P8 * +OU+KI+GI *  *  *  *  * \n"
		"P9+KY+KE *  * +KI *  *  * -RY\n"
		"P+00KA\n"
		"P-00KE00KY00KY00FU00FU\n"
		"-\n"
		).getInitialState();
    NumEffectState eState(state);
    MoveVector moves;
    move_action::Store store(moves);
    GenerateAddEffect8::generate(WHITE,eState,store);
    CPPUNIT_ASSERT(!moves.isMember(Move(Position(7,4),LANCE,WHITE)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(7,6),LANCE,WHITE)));
    CPPUNIT_ASSERT(moves.isMember(Move(Position(9,4),LANCE,WHITE)));
  }
}
static bool isCheckMove(NumEffectState& state,Move move){
  // ʬưƸpositionĤϴޤޤʤ
  Player pl=state.getTurn();
  Position kingPosition=state.getKingPosition(alt(pl));
  NumEffectState savedState(state);
  DoUndoMoveLock lock(state, move);
  // ʤ٤ʤƤޤȤ⤢롥
  return state.hasEffectBy(pl,kingPosition);
}

/**
 * ʤǽΤmove
 * ǽ⤢롥
 */
static bool mayNotGeneratedMove(NumEffectState& state,Move move){
  Player pl=state.getTurn();
  Position kingPosition=state.getKingPosition(alt(pl));
  Ptype ptype=move.ptype();
  // ưʤȤ롥
  if(ptype==KING) return true;
  // ֳѤdrop2ĤʤΤ
  if((ptype==ROOK || ptype==BISHOP) &&
     !move.from().isOnBoard()) return true;
  // ֳ⤬promoteǽʤΤpromoteʤʤ
  if(move.from().isOnBoard() &&
     (ptype==ROOK || ptype==BISHOP || ptype==PAWN) &&
     (move.from().canPromote(pl) || move.to().canPromote(pl))) return true;
  // ܤι֤ɬ
  if(move.from().isOnBoard() &&
     ptype==LANCE &&
     ((pl==BLACK && move.to().y()==2) ||
      (pl==WHITE && move.to().y()==8))) return true;
  NumEffectState savedState(state);
  DoUndoMoveLock lock(state, move);
  // ʤ٤ʤƤޤȤ⤢롥
  if(state.hasEffectBy(pl,kingPosition)) return true;
  for(int i=0;i<8;i++){
    Direction d=static_cast<Direction>(i);
    Position pos=kingPosition+Board_Table.getOffsetForBlack(d);
    if(!pos.isOnBoard()) continue;
    if(effect_util::AdditionalEffect::count2(savedState,pos,pl)<
       effect_util::AdditionalEffect::count2(state,pos,pl)){
      // ζΤäȤưƤĤɲäѤ
      // ζľäƤΤѤ
      // ̤б
      if(move.from().isOnBoard() &&
	 savedState.hasEffectByPiece(savedState.getPieceAt(move.from()),pos)){
	return true;
      }
    }
    for(int num=0;num<40;num++){
      Piece p=state.getPieceOf(num);
      if(p.ptype()==KING) continue;
      if(p.isOnBoardByOwner(pl) &&
	 state.hasEffectByPiece(p,pos)){
	if(!savedState.hasEffectByPiece(savedState.getPieceOf(num),pos)){
	  // ưʤʤäˤϡ
	  if(move.from().isOnBoard() &&
	     savedState.hasEffectByPiece(savedState.getPieceAt(move.from()),pos) &&
	     !state.hasEffectByPiece(state.getPieceAt(move.to()),pos))
	    return true;
	}
      }
    }
  }
  return false;
}
/**
 * 8˵Ĥ꤫ɤΥå.
 * ߤͤ8˵ΤɤˤޤΤʤäɲä뤳ȡ
 * ΤȤƤΥޥؤäƤġ
 * Ͻ롥
 * ˤä8˵Ĥ롥
 * ƤޤȤϤäƤ褤
 * @param state - moveζ
 * @param move - åоݤμ
 */
static bool isAddEffect8Move(NumEffectState& state,Move move){
  // ʬưƸpositionĤϴޤޤʤ
  Player pl=state.getTurn();
  Position kingPosition=state.getKingPosition(alt(pl));
  NumEffectState savedState(state);
  Ptype ptype=move.ptype();
  // ֳѤΤȤˤdropʤ
  if(!move.from().isOnBoard() && (ptype==ROOK || ptype==BISHOP) &&
     state.hasEffectBy(alt(pl),move.to())) return false;
  DoUndoMoveLock lock(state, move);
  for(int i=0;i<8;i++){
    Direction d=static_cast<Direction>(i);
    Position pos=kingPosition+Board_Table.getOffsetForBlack(d);
    if(!pos.isOnBoard()){
      PtypeO ptypeO=newPtypeO(pl,ptype);
      // ׳ǤäƤûOK
      Position from=move.from();
      Position to=move.to();
      if(move.from().isOnBoard() &&
	 (abs(from.x()-pos.x())>1 ||
	  abs(from.y()-pos.y())>1 ||
	 !Ptype_Table.getEffect(ptypeO,Offset32(pos,move.from())).hasUnblockableEffect()) &&
	 abs(to.x()-pos.x())<=1 &&
	 abs(to.y()-pos.y())<=1 &&
	 Ptype_Table.getEffect(ptypeO,Offset32(pos,move.to())).hasUnblockableEffect() &&
	 (!(ptype==ROOK || ptype==BISHOP || ptype==PAWN) ||
	  !(move.from().canPromote(pl) || move.to().canPromote(pl)))){
	//	std::cerr << "returning true,d=" << d << ",ptype=" << ptype << ",move=" << move << std::endl;
	return true;
      }
      continue;
    }
    if(state.hasEffectBy(pl,pos) &&
       (effect_util::AdditionalEffect::count2(savedState,pos,pl)<
	effect_util::AdditionalEffect::count2(state,pos,pl))){
      return true;
    }
    for(int num=0;num<40;num++){
      Piece p=state.getPieceOf(num);
      if(p.ptype()==KING) continue;
      if(p.isOnBoardByOwner(pl) &&
	 state.hasEffectByPiece(p,pos)){
	if(!savedState.hasEffectByPiece(savedState.getPieceOf(num),pos) &&
	   ((p.position()!=move.to() && move.to() != pos)||
	    !move.from().isOnBoard() ||
	    !(ptype==ROOK || ptype==BISHOP || ptype==PAWN) ||
	    !(move.from().canPromote(pl) || move.to().canPromote(pl)))){
	  //	  std::cerr << "returning true,d=" << d << ",ptype=" << ptype << ",move=" << move << std::endl;
	  //	  std::cerr << "p=" << p << std::endl;
	  // ΤƤ֡Ѥdropʤ
	  return true;
	}
	// PROOKФưƼФĤ
	// Ĺ⤢뤬trueˤ롥
	if(p.position()==move.to() && ptype==PROOK && 
	   abs(move.from().x()-move.to().x())==1 &&
	   abs(move.from().y()-move.to().y())==1 &&
	   abs(move.to().x()-pos.x())==1 &&
	   abs(move.to().y()-pos.y())==1){
	  return true;
	}
      }
    }
  }
  return false;
}

void AddEffect8Test::testIsAddEffect8()
{
  {
    SimpleState state=
      CsaString(
"P1-KY-KE-OU-KY+RY *  *  *  * \n"
"P2 *  * -KI-KI *  *  *  *  * \n"
"P3 * +GI-FU *  * +TO * -FU * \n"
"P4-FU *  *  * -FU *  *  * +FU\n"
"P5 * +KE * -KA *  * -FU+FU * \n"
"P6+FU * +KY * +FU *  *  *  * \n"
"P7 * +OU * +FU+KA * +FU *  * \n"
"P8 *  *  * +GI *  *  *  *  * \n"
"P9+KY *  *  * +KI *  *  * -RY\n"
"P+00GI00KE00FU00FU\n"
"P-00KI00GI00KE00FU00FU00FU00FU\n"
"+\n"
		).getInitialState();
    NumEffectState eState(state);
    CPPUNIT_ASSERT(!isAddEffect8Move(eState,Move(Position(5,1),Position(4,2),PROOK,PTYPE_EMPTY,false,BLACK)));
  }
  {
    SimpleState state=
      CsaString(
		"P1 *  * +RY-KE *  *  * -KE-KY\n"
		"P2 *  *  * +GI * -OU-KI *  * \n"
		"P3-FU * -FU-FU * -KI *  *  * \n"
		"P4 *  *  * -UM * -FU+FU * -FU\n"
		"P5+OU *  *  *  *  *  *  *  * \n"
		"P6 *  * +FU-KI *  * +KY *  * \n"
		"P7+FU+FU *  * +FU+FU *  * +FU\n"
		"P8 * +KI *  *  *  *  *  *  * \n"
		"P9+KY+KE *  * -RY *  *  * +KY\n"
		"P+00GI00KE00FU\n"
		"P-00KA00GI00GI00FU00FU00FU00FU00FU\n"
		"+\n"
		).getInitialState();
    NumEffectState eState(state);
    //    CPPUNIT_ASSERT(!isAddEffect8Move(eState,Move(Position(3,4),Position(3,3),PAWN,PTYPE_EMPTY,false,BLACK)));
  }
}


static bool isAddEffect8Drop(NumEffectState& state,Move move){
  // ʬưƸpositionĤϴޤޤʤ
  if(!move.isDrop()) return false;
  Player pl=state.getTurn();
  Position kingPosition=state.getKingPosition(alt(pl));
  NumEffectState savedState(state);
  DoUndoMoveLock lock(state, move);
  if(state.hasEffectBy(pl,kingPosition)) return false;
  // ΤƤֳ
  if(!move.from().isOnBoard() && 
     (move.ptype()==ROOK || move.ptype()==BISHOP) &&
     state.hasEffectBy(alt(pl),move.to())) return false;
  for(int i=0;i<8;i++){
    Direction d=static_cast<Direction>(i);
    Position pos=kingPosition+Board_Table.getOffsetForBlack(d);
    if(!pos.isOnBoard()) continue;
    if(effect_util::AdditionalEffect::count2(savedState,pos,pl)<
       effect_util::AdditionalEffect::count2(state,pos,pl)) return true;
    for(int num=0;num<40;num++){
      Piece p=state.getPieceOf(num);
      if(p.ptype()==KING) continue;
      if(p.isOnBoardByOwner(pl) &&
	 state.hasEffectByPiece(p,pos) &&
	 !savedState.hasEffectByPiece(p,pos))
	return true;
    }
  }
  return false;
}


static void testDropFile(const std::string& fileName){
  Record rec=CsaFile(fileName).getRecord();
  NumEffectState state(rec.getInitialState());
  vector<osl::Move> moves=rec.getMoves();
  for(unsigned int i=0;i<moves.size();i++){
    if (! EffectUtil::isKingInCheck(state.getTurn(), state))
    {				// ʳξǥƥ
      MoveVector allMoves;
      Store store(allMoves);
      AllMoves<NumEffectState,Store>::
	generateMoves(state.getTurn(),state,store);
	  // 1ԤaddEffect.hΥХϰϤͤߤ뤿Υƥ
	  //	if(state.hasEffectBy(state.getTurn(),pos)) continue;
      MoveVector effectMoves;
      Store storeEffect(effectMoves);
      GenerateAddEffect8::
	generate(state.getTurn(),state,storeEffect);
      
      size_t count1=0;
      for(size_t j=0;j<effectMoves.size();j++){
	if(effectMoves[j].isDrop()){
	  count1++;
	  CPPUNIT_ASSERT((state.isValidMove(effectMoves[j],true)  
			 && isAddEffect8Drop(state,effectMoves[j])) ||
			 (std::cerr << std::endl << state  << std::endl << effectMoves[j],0)
			 );
	}
      }
      // 
      size_t count=0;
      MoveVector tmpMoves;
      for(size_t j=0;j<allMoves.size();j++){
	if(isAddEffect8Drop(state,allMoves[j])){
	  tmpMoves.push_back(allMoves[j]);
	  count++;
	  CPPUNIT_ASSERT(effectMoves.isMember(allMoves[j]) ||
			 ((allMoves[j].ptype()==ROOK ||allMoves[j].ptype()==BISHOP) &&
			  !allMoves[j].from().isOnBoard()) ||
			 (std::cerr << allMoves[j] << std::endl <<
			  state << std::endl,0)
			 );
	}
      }
      //      CPPUNIT_ASSERT(count == count1);
    }
    Move move=moves[i];
    ApplyMoveOfTurn::doMove(state, move);
  }
}

void AddEffect8Test::testDropMember(){
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=20;
  if (isShortTest) 
    count=2;
  std::string fileName;
  while((ifs >> fileName) && ++i<count){
    if(fileName == "") 
      break;
    if (! isShortTest)
      std::cerr << fileName << " ";
    testDropFile(OslConfig::testCsaFile(fileName));
  }
}

static void testMoveFile(const std::string& fileName){
  Record rec=CsaFile(fileName).getRecord();
  NumEffectState state(rec.getInitialState());
  vector<osl::Move> moves=rec.getMoves();
  for(unsigned int i=0;i<moves.size();i++){
    if (! EffectUtil::isKingInCheck(state.getTurn(), state))
    {				// ʳξǥƥ
      MoveVector allMoves;
      Store store(allMoves);
      AllMoves<NumEffectState,Store>::
	generateMoves(state.getTurn(),state,store);
	  // 1ԤaddEffect.hΥХϰϤͤߤ뤿Υƥ
	  //	if(state.hasEffectBy(state.getTurn(),pos)) continue;
      MoveVector effectMoves;
      Store storeEffect(effectMoves);
      GenerateAddEffect8::
	generate(state.getTurn(),state,storeEffect);
      
      for(size_t j=0;j<effectMoves.size();j++){
	CPPUNIT_ASSERT((state.isValidMove(effectMoves[j],true)  
			&& (isAddEffect8Move(state,effectMoves[j]) ||
			    isCheckMove(state,effectMoves[j]))
			) ||
		       (std::cerr << std::endl << state << std::endl << effectMoves[j] << std::endl,0)
		       );
      }
      // 
      for(size_t j=0;j<allMoves.size();j++){
	if(isAddEffect8Move(state,allMoves[j]))
	  CPPUNIT_ASSERT(effectMoves.isMember(allMoves[j]) ||
			 mayNotGeneratedMove(state,allMoves[j]) ||
			 (std::cerr << std::endl << state << std::endl << allMoves[j] << std::endl,0)
			 );
      }
    }
    Move move=moves[i];
    ApplyMoveOfTurn::doMove(state, move);
  }
}

void AddEffect8Test::testMoveMember(){
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=20;
  if (isShortTest) 
    count=2;
  std::string fileName;
  while((ifs >> fileName) && ++i<count){
    if(fileName == "") 
      break;
    if (! isShortTest)
      std::cerr << fileName << " ";
    testMoveFile(OslConfig::testCsaFile(fileName));
  }
}
