/* usi.t.cc
 */
#include "osl/record/usi.h"
#include "osl/state/simpleState.h"
#include "osl/record/csaString.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

class UsiTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(UsiTest);
  CPPUNIT_TEST(testMove);
  CPPUNIT_TEST(testParseBoard);
  CPPUNIT_TEST(testParse);
  CPPUNIT_TEST_SUITE_END();
public:
  void testMove();
  void testParseBoard();
  void testParse();
};

CPPUNIT_TEST_SUITE_REGISTRATION(UsiTest);

using namespace osl;
using namespace osl::record;

void UsiTest::testMove()
{
  SimpleState state(HIRATE);
  const Move m76fu = Move(Position(7,7), Position(7,6), PAWN, 
			  PTYPE_EMPTY, false, BLACK);
  const std::string usi76fu = "7g7f";
  const std::string usi76fu2 = usi::show(m76fu);
  CPPUNIT_ASSERT_EQUAL(usi76fu, usi76fu2);

  const Move m76fu2 = usi::strToMove(usi76fu, state);
  CPPUNIT_ASSERT_EQUAL(m76fu, m76fu2);
}

void UsiTest::testParseBoard()
{
  SimpleState state(HIRATE);
  const std::string hirate = "lnsgkgsnl/1r5b1/ppppppppp/9/9/9/PPPPPPPPP/1B5R1/LNSGKGSNL";

  SimpleState state2;
  usi::parseBoard(hirate, state2);
  CPPUNIT_ASSERT_EQUAL(state, state2);
}

void UsiTest::testParse()
{
  {
    SimpleState state(CsaString(
			"P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			"P2 * -HI *  *  *  *  * -KA * \n"
			"P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
			"P4 *  *  *  *  *  * -FU *  * \n"
			"P5 *  *  *  *  *  *  *  *  * \n"
			"P6 *  * +FU *  *  *  * +FU * \n"
			"P7+FU+FU * +FU+FU+FU+FU * +FU\n"
			"P8 * +KA *  *  *  *  * +HI * \n"
			"P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			"-\n").getInitialState());

    const std::string s763426 = "startpos moves 7g7f 3c3d 2g2f";

    SimpleState state2;
    usi::parse(s763426, state2);
    CPPUNIT_ASSERT_EQUAL(state, state2);
  }
  {
    SimpleState state(CsaString(
			"P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			"P2 * -HI *  *  *  *  *  *  * \n"
			"P3 *  *  * -FU-FU-FU-FU-FU-FU\n"
			"P4 *  *  *  *  *  *  *  *  * \n"
			"P5 *  *  *  *  *  *  *  *  * \n"
			"P6 *  *  *  *  *  *  *  *  * \n"
			"P7+FU+FU+FU+FU+FU+FU+FU *  * \n"
			"P8 * +KA *  *  *  *  * +HI * \n"
			"P9+KY+KE * +KI+OU+KI+GI+KE+KY\n"
			"P+00GI00FU00FU\n"
			"P-00KA00FU00FU00FU\n"
			"-\n").getInitialState());

    const std::string stest = "sfen lnsgkgsnl/1r7/3pppppp/9/9/9/PPPPPPP2/1B5R1/LN1GKGSNL w S2Pb3p 1";

    SimpleState state2;
    usi::parse(stest, state2);
    CPPUNIT_ASSERT_EQUAL(state, state2);
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
