#include "problems.h"
#include "osl/search/mtdf.h"
#include "osl/search/mtdf.tcc"
#include "osl/search/nullWindowSearch.tcc"

#include "osl/search/searchRecorder.h"
#include "osl/search/simpleHashTable.h"
#include "osl/search/analyzer/tableAnalyzer.h"
#include "osl/search/analyzer/dotAnalyzerAll.h"
#include "osl/state/hashEffectState.h"
#include "osl/eval/progressEval.h"
#include "osl/eval/ppair/piecePairPieceEval.h"
#include "osl/category/standardMoveGenerator.h"
#include "osl/record/csaString.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
#include <fstream>

using namespace osl;
using namespace search;

class ProgressMtdfRpIDTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(ProgressMtdfRpIDTest);
  CPPUNIT_TEST(testProblems1);
  CPPUNIT_TEST(testProblems2);
  CPPUNIT_TEST(testProblems3);
  CPPUNIT_TEST_SUITE_END();
  void testProblems(SearchRecorder& recorder,
		    const Problem *problems, int numProblems, int depth);
  void tryLoad();
public:
  void testProblems1();
  void testProblems2();
  void testProblems3();
  bool verbose() const
  {
    extern bool isShortTest;
    return ! isShortTest;
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(ProgressMtdfRpIDTest);

void ProgressMtdfRpIDTest::tryLoad()
{
  CPPUNIT_ASSERT(eval::ProgressEval::setUp());
}

typedef MTDF<eval::ProgressEval,StandardMoveGenerator,
 	     SimpleHashTable,SearchRecorder,RealizationProbability> RpMtdf;


void ProgressMtdfRpIDTest::
testProblems(SearchRecorder& recorder,
	     const Problem *problems, int numProblems, int depth)
{
  tryLoad();
#ifndef OSL_SMP
  // mtdf often crashes in 8 smp
  for (int i=0; i<numProblems; ++i)
  {
    HashEffectState state((CsaString(problems[i].state).getInitialState()));
    SimpleHashTable table(100000,-4,verbose());
    RpMtdf::checkmate_t checkmate;
    RpMtdf mtdf_searcher(state, checkmate, table, recorder);

    recorder.stream() << "\nfile " << i << " limit " << depth << "\n";
    recorder.stream() << state;
    mtdf_searcher.setInitialGuess();
    const Move bestMove = mtdf_searcher.computeBestMoveIteratively(depth,200);

    if (! problems[i].acceptable(bestMove))
    {
      std::ofstream os("debug.dot");
      search::analyzer::DotAnalyzerAll analyzer(table, os);
      analyzer.analyze(state.getHash(), 1000);
    }

    CPPUNIT_ASSERT(problems[i].acceptable(bestMove));
  }
#endif
}

void ProgressMtdfRpIDTest::testProblems1()
{
  osl::SearchRecorder recorder("ProgressMtdfRpIDTest.testProblems500");
  recorder.setLogMargin(100);
  testProblems(recorder, problems1, numProblems1, 500);
}
void ProgressMtdfRpIDTest::testProblems2()
{
  osl::SearchRecorder recorder("ProgressMtdfRpIDTest.testProblems800");
  recorder.setLogMargin(100);
  testProblems(recorder, problems1, numProblems1, 800);
}
void ProgressMtdfRpIDTest::testProblems3()
{
  osl::SearchRecorder recorder("ProgressMtdfRpIDTest.testProblems1200");
  recorder.setLogMargin(100);
  testProblems(recorder, problems1, numProblems1, 1000);
  recorder.setLogMargin(100);
  testProblems(recorder, problems3, numProblems3, 1200);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
