/* searchMoveSorterTest.cc
 */
#include "osl/search/searchMoveSorter.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <boost/scoped_ptr.hpp>

class SearchMoveSorterTest : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE(SearchMoveSorterTest);
  CPPUNIT_TEST(testCreation);
  CPPUNIT_TEST(testInsertBlack);
  CPPUNIT_TEST(testInsertWhite);
  CPPUNIT_TEST_SUITE_END();
public:
  void testCreation();
  void testInsertBlack();
  void testInsertWhite();
};

CPPUNIT_TEST_SUITE_REGISTRATION(SearchMoveSorterTest);
using namespace osl;
using namespace osl::search;

void SearchMoveSorterTest::testCreation()
{
    SearchMoveSorter<BLACK> sorter;
    SearchMoveSorter<WHITE> sorter2;
}

void SearchMoveSorterTest::testInsertBlack()
{
    SearchMoveSorter<BLACK> sorter;
    SearchMove m((MoveLogProb(Move(Position(5,5), BISHOP, BLACK),100)));
    sorter.insert(std::make_pair(0, &m));
    sorter.insert(std::make_pair(1, &m));

    SearchMoveSorter<BLACK>::iterator p=sorter.begin();
    int best = p->first;
    ++p;
    int second_best = p->first;
    CPPUNIT_ASSERT(EvalTraits<BLACK>::betterThan(best, second_best));

    sorter.insert(std::make_pair(0, &m));
    sorter.insert(std::make_pair(1, &m));
    p=sorter.begin();
    best = p->first;
    ++p;
    second_best = p->first;
    CPPUNIT_ASSERT_EQUAL(best, second_best);
}

void SearchMoveSorterTest::testInsertWhite()
{
    SearchMoveSorter<WHITE> sorter;
    SearchMove m((MoveLogProb(Move(Position(5,5), BISHOP, BLACK),100)));
    sorter.insert(std::make_pair(0, &m));
    sorter.insert(std::make_pair(1, &m));

    SearchMoveSorter<WHITE>::iterator p=sorter.begin();
    int best = p->first;
    ++p;
    int second_best = p->first;
    CPPUNIT_ASSERT(EvalTraits<WHITE>::betterThan(best, second_best));

    sorter.insert(std::make_pair(0, &m));
    sorter.insert(std::make_pair(1, &m));
    p=sorter.begin();
    best = p->first;
    ++p;
    second_best = p->first;
    CPPUNIT_ASSERT_EQUAL(best, second_best);
}


/* ------------------------------------------------------------------------- */
